@@ -13,10 +13,29 @@ import xcodeml.util.XmOption;
  */
 public class Ident extends Xobject
 {
+    public static final int AS_NONE      = 0;
+    public static final int AS_PUBLIC    = 1;
+    public static final int AS_PROTECTED = 2;
+    public static final int AS_PRIVATE   = 3;
+    public static final int as_num(String access)
+    {
+        if (access == null || access.equals("")) return AS_NONE;
+        return access.equals("public"   ) ? AS_PUBLIC    :
+              (access.equals("protected") ? AS_PROTECTED :
+              (access.equals("private"  ) ? AS_PRIVATE   : AS_NONE));
+    }
+
     /** stroage class */
     private StorageClass stg_class;
     /** key */
     private String name;
+    /** for coarray */
+    private String alias;
+    private BlockList declared_block;
+    /** for C++ */
+    private String fullName;
+    private int access = AS_NONE;
+    private boolean is_op = false;
     /** base address expression value */
     private Xobject value;
     /** declared in (VAR_DECL ) */
@@ -62,10 +81,22 @@ public class Ident extends Xobject
                  int optionalFlags, Xobject gccAttrs,
                  int bit_field, Xobject bit_field_expr, Xobject enum_value,
                  Xobject fparam_value, Xobject codimensions)
+    {
+        this(name, null, AS_NONE, false, stg_class, type, v, optionalFlags, gccAttrs, bit_field, bit_field_expr, 
+        enum_value, fparam_value, codimensions);
+    }
+
+    public Ident(String name, String full_name, int access, boolean is_op, StorageClass stg_class, Xtype type, Xobject v,
+                 int optionalFlags, Xobject gccAttrs,
+                 int bit_field, Xobject bit_field_expr, Xobject enum_value,
+                 Xobject fparam_value, Xobject codimensions)
     {
         super(null, type, optionalFlags);
         if(name != null)
             this.name = name.intern();
+        this.fullName = full_name;
+        this.access = access;
+        this.is_op = is_op;
         this.stg_class = stg_class;
         this.value = v;
         this.declared = false;
@@ -148,6 +179,33 @@ public class Ident extends Xobject
             return name;
     }
 
+    public String getAlias()
+    {
+        return alias;
+    }
+
+    public BlockList getDeclaredBlock()
+    {
+        return declared_block;
+    }
+
+    public String getFullName()
+    {
+        return fullName;
+    }
+
+    public String getAccessStr()
+    {
+        return access == AS_PUBLIC    ? "public"    :
+              (access == AS_PROTECTED ? "protected" :
+              (access == AS_PRIVATE   ? "private"   : null));
+    }
+
+    public boolean isOp()
+    {
+        return is_op;
+    }
+
     public int getFrank()
     {
         return type.getNumDimensions();
@@ -158,6 +216,16 @@ public class Ident extends Xobject
         this.name = name;
     }
 
+    public void setAlias(String newAlias)
+    {
+        alias = newAlias;
+    }
+
+    public void setDeclaredBlock(BlockList bl)
+    {
+        declared_block = bl;
+    }
+
     @Override
     public String getSym()
     {
@@ -277,6 +345,8 @@ public class Ident extends Xobject
     @Override
     public Xobject cfold(Block block)
     {
+      // System.out.println("cfold: name='"+name+"' fparam_value="+fparam_value);
+      
       if (Type().isFparameter() && fparam_value != null) {
         // I don't know why but fparam_value is always in this form.
         if (fparam_value.Nargs() == 2 && fparam_value.getArg(1) == null) {
@@ -287,22 +357,22 @@ public class Ident extends Xobject
         }
       }
 
-      if (declared_module != null) {
-        XobjectDefEnv xobjDefEnv = ((FunctionBlock)block).getEnv();
-        XobjectFile xobjFile = (XobjectFile)xobjDefEnv;
+      // if (declared_module != null) {
+      //   XobjectDefEnv xobjDefEnv = ((FunctionBlock)block).getEnv();
+      //   XobjectFile xobjFile = (XobjectFile)xobjDefEnv;
 
-        if (xobjFile.findVarIdent(declared_module) == null)
-          XmLog.fatal("Ident.cfold: not found module name in globalSymbols: " + 
-                      declared_module);
+      //   if (xobjFile.findVarIdent(declared_module) == null)
+      //     XmLog.fatal("Ident.cfold: not found module name in globalSymbols: " + 
+      //                 declared_module);
 
-        for (XobjectDef punit: xobjFile.getDefs()) {
-          if (declared_module.equals(punit.getName())) {
-            // found the module that declares this ident
-            Ident ident2 = punit.getDef().findVarIdent(name);
-            return ident2.cfold(block);
-          }
-        }
-      }
+      //   for (XobjectDef punit: xobjFile.getDefs()) {
+      //     if (declared_module.equals(punit.getName())) {
+      //       // found the module that declares this ident
+      //       Ident ident2 = punit.getDef().findVarIdent(name);
+      //       return ident2.cfold(block);
+      //     }
+      //   }
+      //}
 
       return this.copy();
     }
@@ -332,9 +402,14 @@ public class Ident extends Xobject
         StringBuilder b = new StringBuilder(256);
         b.append("[");
         b.append(name == null ? "*" : name);
+        b.append(fullName == null ? "" : "(\"" + getFullName() + "\")");
         b.append(" ");
         b.append(stg_class == null ? "*" : stg_class.toXcodeString());
         b.append(" ");
+        b.append(access == AS_NONE ? "*" : getAccessStr());
+        b.append(" ");
+        b.append(is_op ? "Operator" : "*");
+        b.append(" ");
         b.append(type == null ? "*" : type.toString());
         b.append(" ");
         b.append(value == null ? "()" : value.toString());
