@@ -1,14 +1,8 @@
-/* 
- * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
- * $TSUKUBA_Copyright:
- *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
- *  $
- */
 package exc.object;
 
 public class UnionType extends CompositeType
 {
-    public UnionType(String id, XobjList id_list, int typeQualFlags, Xobject gccAttrs)
+    public UnionType(String id, XobjList id_list, long typeQualFlags, Xobject gccAttrs)
     {
         super(Xtype.UNION, id, id_list, typeQualFlags, gccAttrs);
     }
