@@ -27,6 +27,15 @@ public class Xcons
         return x;
     }
 
+    public static XobjString Symbol(Xcode code, Xtype type, String value, VarScope scope, String alias, boolean is_op)
+    {
+        XobjString x = new XobjString(code, type, value);
+        x.setScope(scope);
+        x.setAlias(alias);
+        x.setOp(is_op);
+        return x;
+    }
+
     public static XobjString StringConstant(String value)
     {
         return StringConstant(Xtype.stringType, value);
@@ -44,7 +53,7 @@ public class Xcons
 
     public static XobjString String(String value)
     {
-        return new XobjString(Xcode.STRING, value);
+        return (value != null) ? new XobjString(Xcode.STRING, value) : null;
     }
 
     public static XobjInt Int(Xcode code, Xtype type, int value)
@@ -77,6 +86,11 @@ public class Xcons
         return new XobjLong(code, type, high, low);
     }
 
+    public static XobjLong LongConstant(long value)
+    {
+        return Long(Xcode.LONG_CONSTANT, Xtype.longType, value);
+    }
+
     public static XobjLong LongLongConstant(long high, long low)
     {
         return new XobjLong(high, low);
@@ -198,9 +212,9 @@ public class Xcons
             type = type.getRef();
         switch(x.Opcode()) {
         case ARRAY_ADDR:
-            return Xcons.Symbol(Xcode.ARRAY_ADDR, type, x.getSym());
+            return Xcons.Symbol(Xcode.ARRAY_ADDR, type, x.getSym(), x.getScope());
         case VAR_ADDR:
-            return Xcons.Symbol(Xcode.VAR, type, x.getSym());
+            return Xcons.Symbol(Xcode.VAR, type, x.getSym(), x.getScope());
         case FUNC_ADDR:
             fatal("PointerRef(): FUNC_ADDR");
             break;
@@ -517,7 +531,7 @@ public class Xcons
                 if(x.isZeroConstant())
                     return Xcons.List(Xcode.UNARY_MINUS_EXPR, t, y);
             }
-            if(y.isZeroConstant()) {
+            if(y.isZeroConstant() && x.code != Xcode.ARRAY_ADDR) {
                 if (lt.equals(t))
                     return x;
                 else
