@@ -1,11 +1,4 @@
 package exc.openacc;
-/*
- * $TSUKUBA_Release: $
- * $TSUKUBA_Copyright:
- *  $
- */
-
-// package exc.xcalablemp;
 
 import exc.object.*;
 import java.io.*;
@@ -364,6 +357,7 @@ class ACCgpuDecompileWriter extends PrintWriter {
 //      break;
 
     case MEMBER_REF:      /* (MEMBER_REF v member), v.member */
+    case MEMBER_ARRAY_REF:
       print("((");
       print(v.left());
       print(")->");
