@@ -1,6 +1,7 @@
 package exc.openacc;
 
 import exc.block.*;
+import exc.object.StorageClass;
 
 class AccEnterData extends AccData{
 
@@ -16,15 +17,16 @@ class AccEnterData extends AccData{
       if(var.getParent() != null) continue;
 
       String varName = var.getName();
-      var.setHostDesc(declHostDesc(varName));
-      var.setDevicePtr(declDevicePtr(varName));
+      StorageClass storageClass = var.getId().getStorageClass();
+      var.setHostDesc(declHostDesc(varName, storageClass));
+      var.setDevicePtr(declDevicePtr(varName, storageClass));
 
       initBlockList.add(makeInitFuncCallBlock(var));
 
       int finalizeKind = 1;
       finalizeBlockList.add(makeFinalizeFuncCallBlock(var, finalizeKind));
 
-      copyinBlockList.add(makeCopyBlock(var, true));
+      copyinBlockList.add(makeCopyBlock(var, true, getAsyncExpr()));
     }
   }
 
