@@ -1,9 +1,4 @@
-/* 
- * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
- * $TSUKUBA_Copyright:
- *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
- *  $
- */
+/* -*- Mode: java; c-basic-offset:2 ; indent-tabs-mode:nil ; -*- */
 package exc.util;
 
 import java.io.*;
@@ -27,14 +22,13 @@ import exc.xcodeml.XcodeMLtools;
 import exc.xcodeml.XcodeMLtools_F;
 import exc.xcodeml.XcodeMLtools_Fmod;
 import exc.xcodeml.XcodeMLtools_C;
-import xcodeml.IXobject;
-import xcodeml.XmLanguage;
-import xcodeml.binding.XmXcodeProgram;
+
 import xcodeml.util.*;
 
 import exc.xcodeml.XmXobjectToXcodeTranslator;
 import exc.xcodeml.XmfXobjectToXcodeTranslator;
 import exc.xcodeml.XmcXobjectToXcodeTranslator;
+
 import org.w3c.dom.Document;
 import javax.xml.transform.OutputKeys;
 import javax.xml.transform.TransformerConfigurationException;
@@ -43,11 +37,7 @@ import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.Transformer;
 import javax.xml.transform.dom.DOMSource;
 import javax.xml.transform.stream.StreamResult;
-import xcodeml.XmException;
 
-/**
- * OpenMP-supported XcodeML to XcodeML translator
- */
 public class omompx
 {
   private static void error(String s)
@@ -79,6 +69,9 @@ public class omompx
       "  -M dir       specify where to search for .xmod files",
       "  -max_assumed_shape=N  set max number of assumed-shape arrays of a proedure (for Fortran).",
       "  -decomp      output decompiled source code.",
+      "  -silent      no output.",
+      "  -rename_main=NAME",
+      "               rename the main function NAME.",
       "",
       " Debug Options:",
       "  -d           enable output debug message.",
@@ -91,8 +84,9 @@ public class omompx
       "  -tlog-all           : output results in tlog format for all directives.",
       "  -tlog-selective     : output results in tlog format for selected directives.",
       "",
-      "  -enable-threads  enable 'threads' clause",
-      "  -enable-gpu      enable xmp-dev directive/clauses"
+      "  -enable-threads     : enable 'threads' clause",
+      "  -enable-gpu         : enable xmp-dev directive/clauses",
+      "  -enable-Fonesided   : enable one-sided functions (Only Fortran)"
     };
         
     for(String line : lines) {
@@ -103,27 +97,30 @@ public class omompx
     
   public static void main(String[] args) throws Exception
   {
-    String inXmlFile = null;
-    String outXmlFile = null;
-    String lang = "C";
-    boolean openMP = false;
-    boolean openACC = false;
-    boolean coarray = false;
-    boolean xcalableMP = false;
-    boolean xcalableMPthreads = false;
-    boolean xcalableMPGPU = false;
-    boolean xmpf = false;
-    boolean async = false;
-    boolean outputXcode = false;
-    boolean outputDecomp = false;
-    boolean dump = false;
-    boolean all_profile = false;
-    boolean selective_profile = false;
-    boolean doScalasca = false;
-    boolean doTlog = false;
-    int maxColumns = 0;
-    String coarray_suboption = "";            // HIDDEN
-    boolean coarray_noUseStmt = false;     // TEMPORARY
+    String inXmlFile           = null;
+    String outXmlFile          = null;
+    String lang                = "C";
+    boolean openMP             = false;
+    boolean openACC            = false;
+    boolean coarray            = true;
+    boolean xcalableMP         = false;
+    boolean xcalableMPthreads  = false;
+    boolean xcalableMPGPU      = false;
+    boolean xmpf               = false;
+    boolean async              = false;
+    boolean xcalableACC        = false;
+    boolean outputXcode        = false;
+    boolean outputDecomp       = false;
+    boolean dump               = false;
+    boolean all_profile        = false;
+    boolean selective_profile  = false;
+    boolean doScalasca         = false;
+    boolean doTlog             = false;
+    boolean silent             = false;
+    boolean Fonesided          = false;
+    int maxColumns             = 0;
+    String coarray_suboption   = "";        // HIDDEN
+    boolean coarray_noUseStmt  = false;     // TEMPORARY
     int accDefaultVectorLength = 0;
     boolean accDisableReadOnlyDataCache = false;
         
@@ -149,6 +146,7 @@ public class omompx
         coarray = true;
       } else if(arg.equals("-fnocoarray")) {
         coarray = false;
+        coarray_noUseStmt = true;
       } else if(arg.startsWith("-fcoarray=")) {                  // HIDDEN
         coarray_suboption += arg.substring(arg.indexOf("=")+1);
       } else if(arg.equals("-fcoarray-no-use-statement")) {       // TEMPORARY
@@ -162,10 +160,14 @@ public class omompx
         xcalableMPthreads = true;
       } else if(arg.equals("-enable-gpu")) {
         xcalableMPGPU = true;
+      } else if(arg.equals("-enable-Fonesided")) {
+        Fonesided = true;
       } else if(arg.equals("-fxmpf")) {
         xmpf = true;
       } else if(arg.equals("-fasync")) {
         async = true;
+      } else if(arg.equals("-fxacc")) {
+        xcalableACC = true;
       } else if(arg.equals("-w")) {
         if(narg == null)
           error("needs argument after -w");
@@ -175,6 +177,11 @@ public class omompx
         outputXcode = true;
       } else if(arg.equals("-decomp")) {
         outputDecomp = true;
+      } else if(arg.equals("-silent")){
+        silent = true;
+      } else if(arg.startsWith("-rename_main=")) {
+        String main_name = arg.substring(arg.indexOf("=") + 1);
+        XmOption.setMainName(main_name);
       } else if(arg.equals("-dump")) {
         dump = true;
         outputXcode = true;
@@ -231,7 +238,7 @@ public class omompx
         ACC.platform = ACC.Platform.valueOf(n);
       } else if (arg.startsWith("-device=")){
         String n = arg.substring("-device=".length());
-        ACC.device = AccDevice.valueOf(n);
+        ACC.device = AccDevice.getDevice(n);
       } else if(arg.startsWith("-")){
         error("unknown option " + arg);
       } else if(inXmlFile == null) {
@@ -241,31 +248,20 @@ public class omompx
       }
     }
         
-    if (all_profile == true || selective_profile == true) {
-      if (doScalasca == false && doTlog == false) {
-        doScalasca = true;
-      }
-    }
+    doScalasca = (all_profile == true || selective_profile == true) 
+      && (doScalasca == false && doTlog == false);
 
     Reader reader = null;
-    Writer xmlWriter = null;
-    Writer xcodeWriter = null;
-    Writer decompWriter = null;
-    File dir = null;
-        
+    File dir      = null;
     if(inXmlFile == null) {
       reader = new InputStreamReader(System.in);
-    } else {
+    }
+    else {
       reader = new BufferedReader(new FileReader(inXmlFile));
       dir = new File(inXmlFile).getParentFile();
     }
-        
-    if(outXmlFile == null) {
-      xmlWriter = new OutputStreamWriter(System.out);
-    } else {
-      xmlWriter = new BufferedWriter(new FileWriter(outXmlFile));
-    }
-    
+
+    Writer xcodeWriter = null;
     if(dump || outputXcode) {
       if(dump) {
         xcodeWriter = new OutputStreamWriter(System.out);
@@ -273,8 +269,7 @@ public class omompx
         xcodeWriter = new BufferedWriter(new FileWriter(inXmlFile + ".x"));
       }
     }
-    
-    XmToolFactory toolFactory = new XmToolFactory(lang);
+   
     XmOption.setLanguage(XmLanguage.valueOf(lang));
     XmOption.setIsOpenMP(openMP);
     XmOption.setIsCoarray(coarray);
@@ -283,39 +278,33 @@ public class omompx
     XmOption.setIsXcalableMPthreads(xcalableMPthreads);
     XmOption.setIsXcalableMPGPU(xcalableMPGPU);
     XmOption.setTlogMPIisEnable(doTlog);
+    XmOption.setFonesided(Fonesided);
     XmOption.setCoarrayNoUseStatement(coarray_noUseStmt);   // TEMPORARY
-
-    XobjectFile xobjFile;
-    String srcPath = inXmlFile;
-    XcodeMLtools tools = null;
-    if (XmOption.getLanguage() == XmLanguage.F) {
-      tools = new XcodeMLtools_F();
-    } else {
-      tools = new XcodeMLtools_C();
-    }
-
+    XmOption.setIsXcalableACC(xcalableACC);
+    
     // read XcodeML
-    xobjFile = tools.read(reader);
-    if (inXmlFile != null) {
-      reader.close();
-    }
-        
+    XcodeMLtools tools = 
+      (XmOption.getLanguage() == XmLanguage.F)? 
+      new XcodeMLtools_F() : new XcodeMLtools_C();
+    XobjectFile xobjFile = tools.read(reader);
+    
+    if (inXmlFile != null) reader.close();
+    if (xobjFile == null)  System.exit(1);
+
+    String srcPath  = inXmlFile;
     String baseName = null;
     if(dump || srcPath == null || srcPath.indexOf("<") >= 0 ) {
       srcPath = null;
-    } else {
+    }
+    else {
       String fileName = new File(srcPath).getName();
-      //      int idx = fileName.indexOf(".");
       int idx = fileName.lastIndexOf(".");
       if(idx < 0) {
         XmLog.fatal("invalid source file name : " + fileName);
       }
       baseName = fileName.substring(0, idx);
     }
-        
-    if(xobjFile == null)
-      System.exit(1);
-        
+
     // Output Xcode
     if(xcodeWriter != null) {
       xobjFile.Output(xcodeWriter);
@@ -326,9 +315,9 @@ public class omompx
         
     // XcalableMP translation
     if(xcalableMP) {
-      XMPglobalDecl globalDecl = new XMPglobalDecl(xobjFile);
+      XMPglobalDecl globalDecl   = new XMPglobalDecl(xobjFile);
       XMPtranslate xmpTranslator = new XMPtranslate(globalDecl);
-      XMPrealloc xmpReallocator = new XMPrealloc(globalDecl);
+      XMPrealloc xmpReallocator  = new XMPrealloc(globalDecl);
 
       // For profile                                                                            
       if(all_profile){
@@ -353,16 +342,18 @@ public class omompx
       if(all_profile || selective_profile){
         if (doScalasca == true) {
           xobjFile.addHeaderLine("# include \"xmp_scalasca.h\"");
-        }else if (doTlog == true) {
+        }
+        else if (doTlog == true) {
           xobjFile.addHeaderLine("# include \"xmp_tlog.h\"");
         }
       }
+      
       if(openACC){
         if(xobjFile.findIdent("acc_init", IXobject.FINDKIND_ANY) == null){
           xobjFile.addHeaderLine("# include \"openacc.h\"");
         }
       }
-      xmpTranslator.finalize();
+      xmpTranslator.finish();
 
       if(xcodeWriter != null) {
         xobjFile.Output(xcodeWriter);
@@ -370,12 +361,24 @@ public class omompx
       }
     }
 
-    if (xmpf && xmpf_skipCafMode) {
+    if (xmpf && (xmpf_skipCafMode || !XmOption.isCoarray())) {
       System.out.println("<SKIP-CAF MODE> XMP/F Coarray translator is " +
                          "bypassed for " + xobjFile.getSourceFileName() + ".");
     }
 
-    if (xmpf && !xmpf_skipCafMode) {
+    if (xmpf && (!xmpf_skipCafMode && XmOption.isCoarray())) {
+
+      // Coarray Fortran pass#3
+      exc.xmpF.XMPtransCoarray caf_translator3 =
+        new exc.xmpF.XMPtransCoarray(xobjFile, 3, coarray_suboption,
+                                     xmpf_onlyCafMode);
+      xobjFile.iterateDef(caf_translator3);
+
+      // Coarray Fortran pass#4
+      exc.xmpF.XMPtransCoarray caf_translator4 =
+        new exc.xmpF.XMPtransCoarray(xobjFile, 4, coarray_suboption,
+                                     xmpf_onlyCafMode);
+      xobjFile.iterateDef(caf_translator4);
 
       // Coarray Fortran pass#1
       exc.xmpF.XMPtransCoarray caf_translator1 =
@@ -391,8 +394,8 @@ public class omompx
         new exc.xmpF.XMPtransCoarray(xobjFile, 2, coarray_suboption,
                                      xmpf_onlyCafMode);
       xobjFile.iterateDef(caf_translator2);
-      if(exc.xmpF.XMP.hasErrors())
-        System.exit(1);
+      
+      if(exc.xmpF.XMP.hasErrors()) System.exit(1);
       caf_translator2.finish();
 
       if(xcodeWriter != null) {
@@ -408,11 +411,12 @@ public class omompx
 
     if (xmpf && !xmpf_onlyCafMode) {
       // XMP Fortran
-      exc.xmpF.XMPtranslate
-        xmp_translator = new exc.xmpF.XMPtranslate(xobjFile);
+      exc.xmpF.XMPtranslate xmp_translator = new exc.xmpF.XMPtranslate(xobjFile);
       xobjFile.iterateDef(xmp_translator);
+      
       if(exc.xmpF.XMP.hasErrors())
         System.exit(1);
+      
       xmp_translator.finish();
 
       if(xcodeWriter != null) {
@@ -420,9 +424,8 @@ public class omompx
         xcodeWriter.flush();
       }
     }
-
+    
     // OpenMP translation
-//    OMP.debugFlag = true;
     if(openMP) {
       OMPtranslate omp_translator = new OMPtranslate(xobjFile);
       xobjFile.iterateDef(omp_translator);
@@ -443,7 +446,7 @@ public class omompx
         switch(ACC.platform){
           case CUDA:
           case OpenCL:
-            ACC.device = AccDevice.Fermi;
+            ACC.device = AccDevice.getDevice("Fermi");
             break;
         }
       }
@@ -471,48 +474,43 @@ public class omompx
     if(!dump && outputXcode) {
       xcodeWriter.close();
     }
-        
+
     // translate Xcode to XcodeML
-    XmXcodeProgram xmprog = null;
-    Document xcodeDoc = null;
+    // create transformer from Xobject to XcodeML DOM.
     XmXobjectToXcodeTranslator xc2xcodeTranslator = null;
-    
-    if (lang.equals("F")) {
+    if (lang.equals("F"))
       xc2xcodeTranslator = new XmfXobjectToXcodeTranslator();
-    } else {
+    else
       xc2xcodeTranslator = new XmcXobjectToXcodeTranslator();
-    }
+    
+    Document xcodeDoc = xc2xcodeTranslator.write(xobjFile);
 
-    xcodeDoc = xc2xcodeTranslator.write(xobjFile);
-
-    Transformer transformer = null;
-    try {
-      transformer = TransformerFactory.newInstance().newTransformer();
-    } catch(TransformerConfigurationException e) {
-      throw new XmException(e);
-    }
-
-    transformer.setOutputProperty(OutputKeys.METHOD, "xml");
-
-    try {
-      transformer.transform(new DOMSource(xcodeDoc), new StreamResult(xmlWriter));
-    } catch(TransformerException e) {
-      throw new XmException(e);
-    }
-
-    if (!dump && !outputDecomp) {
-      xmprog = null;
-    }
+    // transformation from DOM to the file. It means to output DOM to the file.
+    if(silent == false){
+      try {
+        Transformer transformer = TransformerFactory.newInstance().newTransformer();
+        transformer.setOutputProperty(OutputKeys.METHOD, "xml");
+        Writer xmlWriter = null;
+        if (outXmlFile == null)
+          xmlWriter = new OutputStreamWriter(System.out);
+        else
+          xmlWriter = new BufferedWriter(new FileWriter(outXmlFile));
         
-    xmlWriter.flush();
+        transformer.transform(new DOMSource(xcodeDoc), new StreamResult(xmlWriter));
+        xmlWriter.flush();
         
-    if(outXmlFile != null) {
-      xmlWriter.close();
-      xmlWriter = null;
+        if(outXmlFile != null) {
+          xmlWriter.close();
+          xmlWriter = null;
+        }
+      } catch(TransformerException e) {
+        throw new XmException(e);
+      }
     }
-        
+    
     // Decompile
     XmDecompilerContext context = null;
+    XmToolFactory toolFactory = new XmToolFactory(lang);
     if(lang.equals("F")) {
       context = toolFactory.createDecompilerContext();
       if(maxColumns > 0)
@@ -520,33 +518,29 @@ public class omompx
     }
         
     if(outputDecomp) {
+      Writer decompWriter = null;
       if(dump || srcPath == null) {
         decompWriter = new OutputStreamWriter(System.out);
       } 
       else { // set decompile writer
         String newFileName = baseName + "." + (XmOption.isLanguageC() ? "c" : "F90");
-        String newFileName2 = baseName + "." + (XmOption.isLanguageC() ? "c" : "f90");
         File newFile = new File(dir, newFileName);
-        File newFile2 = new File(dir, newFileName2);
                 
         if(newFile.exists())
           newFile.renameTo(new File(dir, newFileName + ".i"));
-        if(newFile2.exists())
-          newFile2.renameTo(new File(dir, newFileName2 + ".i"));
                 
         decompWriter = new BufferedWriter(new FileWriter(newFile));
       }
-            
-      XmDecompiler decompiler = toolFactory.createDecompiler();
 
       if (xcodeDoc == null) {
-        javax.xml.parsers.DocumentBuilderFactory docFactory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
+        javax.xml.parsers.DocumentBuilderFactory docFactory = 
+          javax.xml.parsers.DocumentBuilderFactory.newInstance();
         javax.xml.parsers.DocumentBuilder builder = docFactory.newDocumentBuilder();
         xcodeDoc = builder.parse(outXmlFile);
       }
+      
+      XmDecompiler decompiler = toolFactory.createDecompiler();
       decompiler.decompile(context, xcodeDoc, decompWriter);
-      // for collect-init
-      //decompWriter.write(xobjFile.getTailText());
       decompWriter.flush();
     
       if(!dump && outputDecomp) {
