@@ -12,7 +12,7 @@ public class XcodeMLNameTable_F extends XcodeMLNameTable {
 			new XcodeMLName(Xcode.LIST, "declarations"),
 			new XcodeMLName(Xcode.LIST, "arguments"),
 			new XcodeMLName(Xcode.LIST, "kind"),
-			
+
 			new XcodeMLName(Xcode.LIST, "list"),
 			new XcodeMLName(Xcode.STRING, "string"),
 
@@ -54,6 +54,7 @@ public class XcodeMLNameTable_F extends XcodeMLNameTable {
 			new XcodeMLName(Xcode.F_ARRAY_REF, "FarrayRef"),
 			new XcodeMLName(Xcode.F_CHARACTER_REF, "FcharacterRef"),
 			new XcodeMLName(Xcode.MEMBER_REF, "FmemberRef"),
+			new XcodeMLName(Xcode.F_COMPLEX_PART_REF, "FcomplexPartRef"),
 			new XcodeMLName(Xcode.CO_ARRAY_REF, "FcoArrayRef"),      // #060
 			new XcodeMLName(Xcode.FUNC_ADDR, "Ffunction"),
 			new XcodeMLName(Xcode.F_VAR_REF, "varRef"),
@@ -86,6 +87,7 @@ public class XcodeMLNameTable_F extends XcodeMLNameTable {
 			new XcodeMLName(Xcode.F_IF_STATEMENT, "FifStatement"),
 			new XcodeMLName(Xcode.F_DO_STATEMENT, "FdoStatement"),
 			new XcodeMLName(Xcode.F_DO_WHILE_STATEMENT, "FdoWhileStatement"),
+			new XcodeMLName(Xcode.F_DO_CONCURRENT_STATEMENT, "FdoConcurrentStatement"),
 			new XcodeMLName(Xcode.F_CONTINUE_STATEMENT, "continueStatement"),
 			new XcodeMLName(Xcode.F_CYCLE_STATEMENT, "FcycleStatement"),
 			new XcodeMLName(Xcode.F_EXIT_STATEMENT, "FexitStatement"),
@@ -94,9 +96,14 @@ public class XcodeMLNameTable_F extends XcodeMLNameTable {
 			new XcodeMLName(Xcode.STATEMENT_LABEL, "statementLabel"),
 			new XcodeMLName(Xcode.F_SELECT_CASE_STATEMENT,
 					"FselectCaseStatement"),
+			new XcodeMLName(Xcode.SELECT_TYPE_STATEMENT,
+					"selectTypeStatement"),
+			new XcodeMLName(Xcode.F_WAIT_STATEMENT, "FwaitStatement"),
+			new XcodeMLName(Xcode.TYPE_GUARD, "typeGuard"),
 			new XcodeMLName(Xcode.F_CASE_LABEL, "FcaseLabel"),
 			new XcodeMLName(Xcode.F_WHERE_STATEMENT, "FwhereStatement"),
 			new XcodeMLName(Xcode.F_STOP_STATEMENT, "FstopStatement"),
+			new XcodeMLName(Xcode.F_ERROR_STOP_STATEMENT, "FerrorStopStatement"),
 			new XcodeMLName(Xcode.F_PAUSE_STATEMENT, "FpauseStatement"),
 			new XcodeMLName(Xcode.F_READ_STATEMENT, "FreadStatement"),
 			new XcodeMLName(Xcode.F_WRITE_STATEMENT, "FwriteStatement"),
@@ -107,12 +114,15 @@ public class XcodeMLNameTable_F extends XcodeMLNameTable {
 			new XcodeMLName(Xcode.F_OPEN_STATEMENT, "FopenStatement"),
 			new XcodeMLName(Xcode.F_CLOSE_STATEMENT, "FcloseStatement"),
 			new XcodeMLName(Xcode.F_INQUIRE_STATEMENT, "FinquireStatement"),
+			new XcodeMLName(Xcode.F_FLUSH_STATEMENT, "FflushStatement"),
 			new XcodeMLName(Xcode.F_FORMAT_DECL, "FformatDecl"),
 			new XcodeMLName(Xcode.F_DATA_DECL, "FdataDecl"),
+			new XcodeMLName(Xcode.F_DATA_STATEMENT, "FdataStatement"),
 			new XcodeMLName(Xcode.F_NAMELIST_DECL, "FnamelistDecl"),
 			new XcodeMLName(Xcode.F_EQUIVALENCE_DECL, "FequivalenceDecl"),
 			new XcodeMLName(Xcode.F_COMMON_DECL, "FcommonDecl"),
 			new XcodeMLName(Xcode.F_ENTRY_DECL, "FentryDecl"),
+			new XcodeMLName(Xcode.F_ENUM_DECL, "FenumDecl"),
 			new XcodeMLName(Xcode.F_ALLOCATE_STATEMENT, "FallocateStatement"),
 			new XcodeMLName(Xcode.F_DEALLOCATE_STATEMENT,
 					"FdeallocateStatement"),
@@ -124,6 +134,32 @@ public class XcodeMLNameTable_F extends XcodeMLNameTable {
 
 			new XcodeMLName(Xcode.OMP_PRAGMA, "OMPPragma"),
 			new XcodeMLName(Xcode.XMP_PRAGMA, "XMPPragma"),
+			new XcodeMLName(Xcode.ACC_PRAGMA, "ACCPragma"),
+
+			new XcodeMLName(Xcode.COMMENT_LINE, "FcommentLine"),
+
+			new XcodeMLName(Xcode.F_SYNCALL_STATEMENT, "syncAllStatement"),
+			new XcodeMLName(Xcode.F_SYNCIMAGE_STATEMENT, "syncImagesStatement"),
+			new XcodeMLName(Xcode.F_SYNCMEMORY_STATEMENT, "syncMemoryStatement"),
+			new XcodeMLName(Xcode.F_CRITICAL_STATEMENT, "criticalStatement"),
+			new XcodeMLName(Xcode.F_LOCK_STATEMENT, "lockStatement"),
+			new XcodeMLName(Xcode.F_UNLOCK_STATEMENT, "unlockStatement"),
+			new XcodeMLName(Xcode.F_SYNC_STAT, "syncStat"),
+			new XcodeMLName(Xcode.F_BLOCK_STATEMENT, "blockStatement"),
+			new XcodeMLName(Xcode.F_ASSOCIATE_STATEMENT, "associateStatement"),
+			new XcodeMLName(Xcode.F_TYPE_PARAM, "typeParam"),
+			new XcodeMLName(Xcode.F_TYPE_PARAMS, "typeParams"),
+			new XcodeMLName(Xcode.F_TYPE_PARAM_VALUES, "typeParamValues"),
+			new XcodeMLName(Xcode.F_LEN, "len"),
+			new XcodeMLName(Xcode.F_IMPORT_STATEMENT, "FimportDecl"),
+			new XcodeMLName(Xcode.F_TYPE_BOUND_PROCEDURES, "typeBoundProcedures"),
+			new XcodeMLName(Xcode.F_TYPE_BOUND_PROCEDURE , "typeBoundProcedure" ),
+			new XcodeMLName(Xcode.F_FINAL_PROCEDURE , "finalProcedure" ),
+			new XcodeMLName(Xcode.F_TYPE_BOUND_GENERIC_PROCEDURE , "typeBoundGenericProcedure"),
+			new XcodeMLName(Xcode.F_BINDING, "binding"),
+			new XcodeMLName(Xcode.F_MODULE_PROCEDURE_DEFINITION, "FmoduleProcedureDefinition"),
+			new XcodeMLName(Xcode.F_FORALL_STATEMENT, "forallStatement"),
+			new XcodeMLName(Xcode.F_CONDITION, "condition"),
 	};
 
 	// constructor
