@@ -1,21 +1,7 @@
-/*
- * $tsukuba_Release: Omni OpenMP Compiler 3 $
- * $TSUKUBA_Copyright:
- *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
- *  $
- */
-// package exc.xcodeml;
-
 package exc.xcodeml;
 
-
-import static xcodeml.util.XmDomUtil.collectChildNodes;
-import static xcodeml.util.XmDomUtil.collectElementsExclude;
-import static xcodeml.util.XmDomUtil.getAttr;
-import static xcodeml.util.XmDomUtil.getAttrBool;
-import static xcodeml.util.XmDomUtil.getContent;
-import static xcodeml.util.XmDomUtil.getContentText;
-import static xcodeml.util.XmDomUtil.getElement;
+import static xcodeml.util.XmDomUtil.*;
+import org.w3c.dom.*;
 
 import java.io.Reader;
 import java.util.ArrayList;
@@ -25,36 +11,10 @@ import java.util.Stack;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 
-import xcodeml.c.decompile.XcConstObj;
-import xcodeml.c.util.XmcBindingUtil;
 import xcodeml.util.XmStringUtil;
-import exc.object.ArrayType;
-import exc.object.BasicType;
-import exc.object.EnumType;
-import exc.object.FunctionType;
-import exc.object.Ident;
-import exc.object.PointerType;
-import exc.object.StorageClass;
-import exc.object.StructType;
-import exc.object.UnionType;
-import exc.object.VarScope;
-import exc.object.Xcode;
-import exc.object.Xcons;
-import exc.object.XmpCoArrayType;
-import exc.object.XobjArgs;
-import exc.object.XobjList;
-import exc.object.XobjString;
-import exc.object.Xobject;
-import exc.object.XobjectFile;
-import exc.object.Xtype;
+import xcodeml.util.XmLongLongConst;
 
-import static xcodeml.util.XmDomUtil.getElement;
-import static xcodeml.util.XmDomUtil.getAttr;
-import static xcodeml.util.XmDomUtil.getContent;
-import static xcodeml.util.XmDomUtil.getContentText;
-import static xcodeml.util.XmDomUtil.getAttrBool;
-import static xcodeml.util.XmDomUtil.collectChildNodes;
-import static xcodeml.util.XmDomUtil.collectElementsExclude;
+import exc.object.*;
 
 /**
  * tools for XcodeML/C to Xcode translation.
@@ -74,27 +34,21 @@ public class XcodeMLtools_C extends XcodeMLtools {
   void enterType(Node n) {
     String name = n.getNodeName();
 
-    if (name.equals("basicType")) {
-      declBasicType(n);
-    } else if (name.equals("pointerType")) {
-      declPointerType(n);
-    } else if (name.equals("functionType")) {
-      declFunctionType(n);
-    } else if (name.equals("arrayType")) {
-      declArrayType(n);
-    } else if (name.equals("structType")) {
-      declStructType(n);
-    } else if (name.equals("unionType")) {
-      declUnionType(n);
-    } else if (name.equals("enumType")) {
-      declEnumType(n);
-    } else if (name.equals("coArrayType")) {
-      declCoArrayType(n);
-    } else {
+    switch (name){
+    case "basicType"   : declBasicType(n);    break;
+    case "pointerType" : declPointerType(n);  break;
+    case "functionType": declFunctionType(n); break;
+    case "arrayType"   : declArrayType(n);    break;
+    case "structType"  : declStructType(n);   break;
+    case "unionType"   : declUnionType(n);    break;
+    case "enumType"    : declEnumType(n);     break;
+    case "classType"   : declStructType(n);   break;
+    case "coArrayType" : declCoArrayType(n);  break;
+    default:
       fatal("Unknown node in typeTable: " + n);
     }
   }
-
+  
   /*
    * global Ident section
    */
@@ -110,16 +64,19 @@ public class XcodeMLtools_C extends XcodeMLtools {
   @Override
   void enterGlobalDecl(Node n) {
     Xobject xobj = toXobject(n);
-    //xobj.setParentRecursively(null);
+
     switch (xobj.Opcode()) {
     case FUNCTION_DEFINITION:
     case VAR_DECL:
     case FUNCTION_DECL:
     case GCC_ASM_DEFINITION:
     case PRAGMA_LINE:
+    case LINEMARKER:
     case TEXT:
     case ACC_PRAGMA:
     case XMP_PRAGMA:
+    case CPP_CLASS_DECL:
+    case CPP_DECL_NAMESPACE:
       xobjFile.add(xobj);
       break;
     default:
@@ -149,16 +106,26 @@ public class XcodeMLtools_C extends XcodeMLtools {
 
       _pScopeStack.push(PragmaScope.Nestedfunc);
 
+      boolean is_op;
+      Node n_name = getElement(n, "name"); // symbolName
+      if (n_name != null)
+        is_op = false;
+      else {
+        is_op = true ;
+        n_name = getElement(n, "operator"); // symbolAlias
+      }
       XobjList xlist = enterAsXobjList(n,
 				       code,
 				       type,
 				       Xcode.IDENT,
 				       null, // symbolTypeName
-				       getContentText(getElement(n, "name")), // symbolName
+				       getContentText(n_name), // symbolName
 				       null, // symbolScope
+				       getAttr(n_name, "fullName"), // symbolAlias
+				       is_op,
 				       getElement(n, "symbols"),
 				       null, // params, process later.
-				       getElement(n, "body"),
+				       getContent(getElement(n, "body")),
 				       getElement(n, "gccAttributes"));
       _pScopeStack.pop();
 
@@ -172,13 +139,14 @@ public class XcodeMLtools_C extends XcodeMLtools {
       XobjArgs arg = xlist.getArgs();
       XobjArgs argBody = arg.nextArgs().nextArgs().nextArgs();
       Xobject stmts = argBody.getArg();
+
       if (stmts != null) {
         switch (stmts.Opcode()) {
-        case LIST:
-          stmts = Xcons.CompoundStatement(
-					  Xcons.IDList(), Xcons.List(), stmts);
-          argBody.setArg(stmts);
-          break;
+//      case LIST:
+//        stmts = Xcons.CompoundStatement(
+//			  Xcons.IDList(), Xcons.List(), stmts);
+//        argBody.setArg(stmts);
+//        break;
         case COMPOUND_STATEMENT:
           break;
         default:
@@ -188,26 +156,44 @@ public class XcodeMLtools_C extends XcodeMLtools {
       return xlist;
     }
 
-    case VAR_DECL:
+    case VAR_DECL: {
+      Node n_name = getElement(n, "name"); // symbolName
       return enterAsXobjList(n,
 			     code,
 			     type,
 			     Xcode.IDENT,
 			     null, // symbolTypeName
-			     getContentText(getElement(n, "name")), // symbolName
+			     getContentText(n_name), // symbolName
 			     null, // symbolScope
+			     getAttr(n_name, "fullName"), // symbolAlias
+			     false, // is_op
 			     getContent(getElement(n, "value")),
 			     getElement(n, "gccAsm"));
+    }
 
-    case FUNCTION_DECL:
+    case FUNCTION_DECL: {
+      boolean is_op;
+      Node n_name = getElement(n, "name"); // symbolName
+      if (n_name != null)
+        is_op = false;
+      else {
+        is_op = true ;
+        n_name = getElement(n, "operator"); // symbolAlias
+      }
       return enterAsXobjList(n,
 			     code,
 			     type,
 			     Xcode.IDENT,
 			     null, // symbolTypeName
-			     getContentText(getElement(n, "name")), // symbolName
+			     getContentText(n_name), // symbolName
 			     null, // symbolScope
+			     getAttr(n_name, "fullName"), // symbolAlias
+			     is_op,
 			     getElement(n, "gccAsm"));
+    }
+
+    case ID: // id
+      return toIdent(n);
 
     case ID_LIST: // symbols
       return toIdentList(n);
@@ -233,6 +219,13 @@ public class XcodeMLtools_C extends XcodeMLtools {
       return x;
     }
 
+    case LINEMARKER:
+      String linemarkerFlag = getAttr(n, "flag");
+      x = Xcons.List(code,
+		     new XobjString(Xcode.STRING, linemarkerFlag));
+      setCommonAttributes(n, x);
+      return x;
+
     case STRING_CONSTANT:
       return Xcons.StringConstant(getContentText(n));
 
@@ -247,16 +240,15 @@ public class XcodeMLtools_C extends XcodeMLtools {
 			 getContentText(n));
 
     case LONGLONG_CONSTANT: {
-      XcConstObj.LongLongConst llConst =
-	XmcBindingUtil.createLongLongConst(getContentText(n),
-					   typeId);
-      if (llConst == null) {
-	fatal("Invalid long long value");
-      }
-      return Xcons.Long(code,
-			type,
-			llConst.getHigh(),
-			llConst.getLow());
+        XmLongLongConst llConst =
+            XmLongLongConst.createLongLongConst(getContentText(n));
+        if (llConst == null) {
+            fatal("Invalid long long value");
+        }
+        return Xcons.Long(code,
+                          type,
+                          llConst.getHigh(),
+                          llConst.getLow());
     }
 
     case MOE_CONSTANT:
@@ -349,11 +341,21 @@ public class XcodeMLtools_C extends XcodeMLtools {
     }
 
     case FUNCTION_CALL:
-      return enterAsXobjList(n,
-			     code,
-			     type,
-			     getContent(getElement(n, "function")),
-			     getElement(n, "arguments"));
+      Node n_addr;
+      Node n_func = getElement(n, "function");
+      if (n_func != null) {
+        n_addr = getContent(n_func);
+      } else {
+        n_addr = getElement(n, "memberRef");
+        if (n_addr == null) {
+          n_addr = getElement(n, "operator");
+        }
+      }
+      if (n_addr == null) {
+        fatal("unknown functionCall function.");
+      }
+      Node n_args = getElement(n, "arguments");
+      return enterAsXobjList(n, code, type, n_addr, n_args);
 
     case SIZE_OF_EXPR:
       return enterSizeOrAlignExpr(code, type, getContent(n));
@@ -370,6 +372,7 @@ public class XcodeMLtools_C extends XcodeMLtools {
       return enterArrayRef(code, type, n);
 
     case FUNC_ADDR:
+    case CPP_OPERATOR_ADDR:
       return Xcons.Symbol(code,
 			  type,
 			  getContentText(n));
@@ -474,6 +477,17 @@ public class XcodeMLtools_C extends XcodeMLtools {
     case COMPOUND_VALUE_ADDR:
       return enterAsXobjList(n, code, type, getContent(getElement(n, "value")));
 
+    case CPP_CLASS_DECL:
+      XobjList classObj = enterAsXobjList(n, code, type);
+      NodeList list = n.getChildNodes();
+      for (int i = 0; i < list.getLength(); i++) {
+        Node nn = list.item(i);
+        if (nn.getNodeType() != Node.ELEMENT_NODE)
+          continue;
+        classObj.add(toXobject(nn));
+      }
+      return classObj;
+
     default: // default action, make list
       XobjList xobjs = new XobjList(code, type);
       //if(code == Xcode.LIST && list.getLength() == 0)
@@ -484,7 +498,19 @@ public class XcodeMLtools_C extends XcodeMLtools {
   }
 
   Ident toIdent(Node n) {
-    String name = getContentText(getElement(n, "name"));
+    boolean isOperator;
+    Node n_name = getElement(n, "name");
+    if (n_name != null)
+      isOperator = false;
+    else {
+      isOperator = true ;
+      n_name = getElement(n, "operator");
+    }
+    String name = getContentText(n_name);
+    String full_name = getAttr(n_name, "fullName");
+
+    // public, protected or private.
+    int access = Ident.as_num(getAttr(n, "access"));
 
     // get type
     String tid = getAttr(n, "type");
@@ -557,7 +583,7 @@ public class XcodeMLtools_C extends XcodeMLtools {
 
     // create ident
     // for coarray, set codiemnsions (#284)
-    Ident ident = new Ident(name, sclass, type, addr,
+    Ident ident = new Ident(name, full_name, access, isOperator, sclass, type, addr,
 			    optionalFlags, gccAttrs,
 			    bitField, bitFieldExpr, enumValue, null, codims);
     //if (codims != null)
@@ -587,6 +613,8 @@ public class XcodeMLtools_C extends XcodeMLtools {
 				   String symbolTypeName,
 				   String symbolName,
 				   String symbolScope,
+				   String symbolAlias,
+				   boolean is_op,
 				   Node ... nodes) {
     Xtype symbolType = null;
     if (symbolTypeName != null) {
@@ -599,7 +627,9 @@ public class XcodeMLtools_C extends XcodeMLtools {
     Xobject symbolObj = Xcons.Symbol(symbolCode,
 				     symbolType,
 				     symbolName,
-				     scope);
+				     scope,
+				     symbolAlias,
+				     is_op);
     XobjList objList = Xcons.List(code, type);
     objList.add(symbolObj);
     return toXobjList(baseNode, objList, nodes);
@@ -692,9 +722,21 @@ public class XcodeMLtools_C extends XcodeMLtools {
 
   /** process arrayRef. */
   private XobjList enterArrayRef(Xcode code, Xtype type, Node arrayRefNode) {
-    ArrayList<Node> childNodes = collectElementsExclude(arrayRefNode,
-							"arrayAddr");
+    // ArrayList<Node> childNodes = collectElementsExclude(arrayRefNode,
+    // 							"arrayAddr");
+
+    ArrayList<Node> childNodes;
     Node arrayAddrNode = getElement(arrayRefNode, "arrayAddr");
+
+    if (arrayAddrNode != null){
+      childNodes = collectElementsExclude(arrayRefNode, "arrayAddr");
+    }
+    else { // pointer
+      childNodes = collectChildNodes(arrayRefNode);
+      arrayAddrNode = childNodes.get(0);
+      childNodes.remove(0);
+    }
+
     XobjList objList = enterAsXobjList(arrayRefNode,
 				       code,
 				       type,
@@ -722,8 +764,11 @@ public class XcodeMLtools_C extends XcodeMLtools {
       childNodes = collectElementsExclude(subArrayRefNode, "arrayAddr");
     }
     else { // pointer
-      arrayAddrNode = getElement(subArrayRefNode, "Var");
-      childNodes = collectElementsExclude(subArrayRefNode, "Var");
+      //arrayAddrNode = getElement(subArrayRefNode, "Var");
+      //childNodes = collectElementsExclude(subArrayRefNode, "Var");
+      childNodes = collectChildNodes(subArrayRefNode);
+      arrayAddrNode = childNodes.get(0);
+      childNodes.remove(0);
     }
 
     XobjList objList = enterAsXobjList(subArrayRefNode,
@@ -733,13 +778,7 @@ public class XcodeMLtools_C extends XcodeMLtools {
 
     XobjList subList = Xcons.List();
     for (Node childNode : childNodes){
-      Xobject indexRange = toXobject(childNode);
-      if (indexRange instanceof XobjList){
-	subList.add(indexRange.getArg(0));
-      }
-      else {
-	subList.add(indexRange);
-      }
+      subList.add(toXobject(childNode));
     }
     objList.add(subList);
 
@@ -764,24 +803,22 @@ public class XcodeMLtools_C extends XcodeMLtools {
       objList.add(toXobject(childNodes.get(0)));
     }
     else {
-      XobjList subList = Xcons.List();
       for (Node childNode : childNodes){
 	Xobject arg = toXobject(childNode);
-	if (arg != null && arg.Nargs() != 0) subList.add(arg.getArg(0));
-	else subList.add(null);
+	if (arg != null && arg.Nargs() != 0) objList.add(arg.getArg(0));
+        else objList.add(null);
       }
-      objList.add(subList);
     }
 
     return objList;
   }
 
-  private int getTypeQualFlags(Node n, boolean isFunctionType) {
-    int tqConst = getAttrBool(n, "is_const") ? Xtype.TQ_CONST : 0;
-    int tqRestrict = getAttrBool(n, "is_restrict") ? Xtype.TQ_RESTRICT : 0;
-    int tqVolatile = getAttrBool(n, "is_volatile") ? Xtype.TQ_VOLATILE : 0;
-    int tqInline = (isFunctionType && getAttrBool(n, "is_inline")) ? Xtype.TQ_INLINE : 0;
-    int tqFuncStatic = (isFunctionType && getAttrBool(n, "is_static")) ? Xtype.TQ_FUNC_STATIC : 0;
+  private long getTypeQualFlags(Node n, boolean isFunctionType) {
+    long tqConst = getAttrBool(n, "is_const") ? Xtype.TQ_CONST : 0;
+    long tqRestrict = getAttrBool(n, "is_restrict") ? Xtype.TQ_RESTRICT : 0;
+    long tqVolatile = getAttrBool(n, "is_volatile") ? Xtype.TQ_VOLATILE : 0;
+    long tqInline = (isFunctionType && getAttrBool(n, "is_inline")) ? Xtype.TQ_INLINE : 0;
+    long tqFuncStatic = (isFunctionType && getAttrBool(n, "is_static")) ? Xtype.TQ_FUNC_STATIC : 0;
 
     return tqConst | tqRestrict | tqVolatile | tqInline | tqFuncStatic;
   }
@@ -793,7 +830,7 @@ public class XcodeMLtools_C extends XcodeMLtools {
   private void declBasicType(Node n) {
     String typeId = getAttr(n, "type");
     String name = getAttr(n, "name");
-    int tq = getTypeQualFlags(n, false);
+    long tq = getTypeQualFlags(n, false);
     Xobject gccAttrs = getGccAttributes(n);
     BasicType.TypeInfo ti = BasicType.getTypeInfoByCName(name);
 
@@ -907,9 +944,17 @@ public class XcodeMLtools_C extends XcodeMLtools {
   }
 
   private void declStructType(Node n) {
+    Node nameNode = getElement(n, "name");
+    XobjString tagNames = null;
+    if (nameNode != null) {
+      tagNames = new XobjString(Xcode.IDENT, getContentText(nameNode));
+      tagNames.setAlias(getAttr(nameNode, "fullName"));
+    }
     XobjList identList = (XobjList)toXobject(getElement(n, "symbols"));
     Xobject gccAttrs = getGccAttributes(n);
     Xtype type = new StructType(getAttr(n, "type"),
+				n.getNodeName().equals("classType"),
+				tagNames,
 				identList,
 				getTypeQualFlags(n, false),
 				gccAttrs);
@@ -976,9 +1021,9 @@ public class XcodeMLtools_C extends XcodeMLtools {
 
    */
   private int getAsCInt(String str) {
-    return XmStringUtil.getAsCInt(null, str);
+    return XmStringUtil.getAsCInt(str);
   }
   private long getAsCLong(String str) {
-    return XmStringUtil.getAsCLong(null, str);
+    return XmStringUtil.getAsCLong(str);
   }
 }
