@@ -38,6 +38,7 @@ import exc.object.XobjString;
 import exc.object.Xobject;
 import exc.object.XobjectFile;
 import exc.object.Xtype;
+import exc.object.EnumType;
 
 
 /**
@@ -58,6 +59,8 @@ public class XcodeMLtools_F extends XcodeMLtools {
       declFfunctionType(n);
     } else if (name == "FstructType") {
       declFstructType(n);
+    } else if (name == "FenumType") {
+      declFenumType(n);
     } else
       fatal("Unknown node in typeTable: " + n);
   }
@@ -80,16 +83,27 @@ public class XcodeMLtools_F extends XcodeMLtools {
 
   private void declFbasicType(Node n) {
     String tid = getAttr(n, "type");
-    BasicType.TypeInfo ti = BasicType.getTypeInfoByFName(getAttr(n, "ref"));
-    int tq = (getAttrBool(n, "is_allocatable") ? Xtype.TQ_FALLOCATABLE : 0)
+    BasicType.TypeInfo ti = BasicType.getTypeInfoByFName(
+                             (getAttrBool(n, "is_class") ||
+                              getAttrBool(n, "is_procedure")) &&
+                             (getAttr(n, "ref") == null) ?
+                               "Fvoid" : getAttr(n, "ref"));
+    long tq = (getAttrBool(n, "is_allocatable") ? Xtype.TQ_FALLOCATABLE : 0)
       | (getAttrBool(n, "is_optional") ? Xtype.TQ_FOPTIONAL : 0)
       | (getAttrBool(n, "is_parameter") ? Xtype.TQ_FPARAMETER : 0)
       | (getAttrBool(n, "is_pointer") ? Xtype.TQ_FPOINTER : 0)
       | (getAttrBool(n, "is_private") ? Xtype.TQ_FPRIVATE : 0)
       | (getAttrBool(n, "is_public") ? Xtype.TQ_FPUBLIC : 0)
+      | (getAttrBool(n, "is_protected") ? Xtype.TQ_FPROTECTED : 0)
       | (getAttrBool(n, "is_save") ? Xtype.TQ_FSAVE : 0)
       | (getAttrBool(n, "is_target") ? Xtype.TQ_FTARGET : 0)
-      | (getAttrBool(n, "is_cray_pointer") ? Xtype.TQ_FCRAY_POINTER : 0); //#060c
+      | (getAttrBool(n, "is_cray_pointer") ? Xtype.TQ_FCRAY_POINTER : 0) //#060c
+      | (getAttrBool(n, "is_volatile") ? Xtype.TQ_FVOLATILE : 0)
+      | (getAttrBool(n, "is_class") ? Xtype.TQ_FCLASS : 0)
+      | (getAttrBool(n, "is_value") ? Xtype.TQ_FVALUE : 0)
+      | (getAttrBool(n, "is_procedure") ? Xtype.TQ_FPROCEDURE : 0)
+      | (getAttrBool(n, "is_contiguous") ? Xtype.TQ_FCONTIGUOUS : 0)
+      | (getAttrBool(n, "is_asynchronous") ? Xtype.TQ_FASYNCHRONOUS : 0);
 
     String intent = getAttr(n, "intent");
 
@@ -105,12 +119,20 @@ public class XcodeMLtools_F extends XcodeMLtools {
 
     Xobject fkind = toXobject(getContent(getElement(n, "kind")));
     Xobject flen = null, sizeExprs[] = null, cosizeExprs[] = null;
+    XobjList typeParamValues = null;
     Node nn, nnn;
 
     if ((nn = getElement(n, "len")) != null) {
       flen = toXobject(getContent(nn));
       if (flen == null)
-	flen = Xcons.IntConstant(-1); // means variable length
+        if (getAttrBool(nn, "is_assumed_shape"))
+	  flen = Xcons.IntConstant(-1); // means variable length, "(len=:)"
+        else if (getAttrBool(nn, "is_assumed_size"))
+	  flen = Xcons.IntConstant(-2); // means variable length, "(len=*)"
+        else
+          fatal("array length unknown:" + nn);
+    } else if ((nn = getElement(n, "typeParamValues")) != null) {
+      typeParamValues = (XobjList)toXobject(nn);
     } else {
       NodeList list = n.getChildNodes();
       if (list.getLength() > 0) {
@@ -141,7 +163,7 @@ public class XcodeMLtools_F extends XcodeMLtools {
                   || xx.Opcode() == Xcode.F_INDEX_RANGE) {
                 cosizeExprList.add(xx);
               } else
-                fatal("bad coindex in type:" + nn); 
+                fatal("bad coindex in type:" + nn);
             }
           } else
 	    fatal("bad index in type:" + nn);
@@ -159,10 +181,16 @@ public class XcodeMLtools_F extends XcodeMLtools {
 
     if (sizeExprs == null) {
       if (ti == null) { // inherited type such as structure
+        String pass = getAttr(n, "pass");
+        String pass_arg_name = getAttr(n, "pass_arg_name");
 	Xtype ref = getType(getAttr(n, "ref"));
+	xobjFile.addType(ref);
 	type = ref.inherit(tid);
 	type.setTypeQualFlags(tq);
         type.setCodimensions(cosizeExprs);                           // #060
+        type.setFTypeParamValues(typeParamValues);
+        type.setPass(pass);
+        type.setPassArgName(pass_arg_name);
       } else {
 	type = new BasicType(ti.type.getBasicType(), tid, tq, null,
 			     fkind, flen, cosizeExprs);             // #060
@@ -172,6 +200,21 @@ public class XcodeMLtools_F extends XcodeMLtools {
       type = new FarrayType(tid, ref, tq, sizeExprs, cosizeExprs);    // #060
     }
 
+    String bind = getAttr(n, "bind");
+    if(bind != null){
+        type.setBind(bind);
+    }
+
+    String bind_name = getAttr(n, "bind_name");
+    if(bind_name != null){
+        type.setBindName(bind_name);
+    }
+
+    Xtype dummy = getType(tid);
+    if (dummy.getKind() == Xtype.UNDEF)
+      dummy.assign(type);
+    else
+      fatal("type table entry is doubled: " + tid);
     xobjFile.addType(type);
   }
 
@@ -188,17 +231,34 @@ public class XcodeMLtools_F extends XcodeMLtools {
     String tid = getAttr(n, "type");
     Xtype retType = getType(getAttr(n, "return_type"));
 
-    int tq = (getAttrBool(n, "is_external") ? Xtype.TQ_FEXTERNAL : 0)
+    long tq = (getAttrBool(n, "is_external") ? Xtype.TQ_FEXTERNAL : 0)
       | (getAttrBool(n, "is_internal") ? Xtype.TQ_FINTERNAL : 0)
       | (getAttrBool(n, "is_intrinsic") ? Xtype.TQ_FINTRINSIC : 0)
       | (getAttrBool(n, "is_private") ? Xtype.TQ_FPRIVATE : 0)
       | (getAttrBool(n, "is_public") ? Xtype.TQ_FPUBLIC : 0)
+      | (getAttrBool(n, "is_protected") ? Xtype.TQ_FPROTECTED : 0)
       | (getAttrBool(n, "is_program") ? Xtype.TQ_FPROGRAM : 0)
-      | (getAttrBool(n, "is_recursive") ? Xtype.TQ_FRECURSIVE : 0);
+      | (getAttrBool(n, "is_elemental") ? Xtype.TQ_FELEMENTAL : 0)
+      | (getAttrBool(n, "is_recursive") ? Xtype.TQ_FRECURSIVE : 0)
+      | (getAttrBool(n, "is_module") ? Xtype.TQ_FMODULE : 0);
 
+    if (getAttr(n, "is_pure") != null)
+      tq |= (getAttrBool(n, "is_pure") ? Xtype.TQ_FPURE : Xtype.TQ_FIMPURE);
+    
     Xobject params = toXobject(getElement(n, "params"));
     FunctionType type = new FunctionType(tid, retType, params, tq, false,
-					 null, getAttr(n, "result_name"));
+        null, getAttr(n, "result_name"));
+
+    String bind = getAttr(n, "bind");
+    if(bind != null){
+        type.setBind(bind);
+    }
+
+    String bind_name = getAttr(n, "bind_name");
+    if(bind_name != null){
+        type.setBindName(bind_name);
+    }
+
     xobjFile.addType(type);
   }
 
@@ -213,17 +273,57 @@ public class XcodeMLtools_F extends XcodeMLtools {
    */
   private void declFstructType(Node n) {
     String tid = getAttr(n, "type");
-    int tq = (getAttrBool(n, "is_internal_private") ? Xtype.TQ_FINTERNAL_PRIVATE
+    String parent_tid = getAttr(n, "extends");
+    long tq = (getAttrBool(n, "is_internal_private") ? Xtype.TQ_FINTERNAL_PRIVATE
 	      : 0)
       | (getAttrBool(n, "is_private") ? Xtype.TQ_FPRIVATE : 0)
       | (getAttrBool(n, "is_public") ? Xtype.TQ_FPUBLIC : 0)
-      | (getAttrBool(n, "is_sequence") ? Xtype.TQ_FSEQUENCE : 0);
+      | (getAttrBool(n, "is_protected") ? Xtype.TQ_FPROTECTED : 0)
+      | (getAttrBool(n, "is_sequence") ? Xtype.TQ_FSEQUENCE : 0)
+      | (getAttrBool(n, "is_abstract") ? Xtype.TQ_FABSTRACT : 0);
 
+    XobjList tparam_list = (XobjList) toXobject(getElement(n, "typeParams"));
     XobjList id_list = (XobjList) toXobject(getElement(n, "symbols"));
-    StructType type = new StructType(tid, id_list, tq, null);
+    XobjList proc_list = (XobjList) toXobject(getElement(n, "typeBoundProcedures"));
+    Xobject finalProcedure = toXobject(getElement(n, "finalProcedure"));
+    StructType type = new StructType(tid, parent_tid, id_list, proc_list, tq, null, tparam_list, finalProcedure);
+
+    String bind = getAttr(n, "bind");
+    if(bind != null){
+        type.setBind(bind);
+    }
+
     xobjFile.addType(type);
   }
 
+  /*
+   * (symbols)
+   *
+   * enum, bind(C)
+   *  enumerator :: open_door=4, close_door=17
+   *  enumerator :: lock_door
+   * end enum
+   *
+   * <FenumType type="TYPE_NAME">
+   *   <name>open_door</name>
+   *   <value>
+   *     <FintConstant type="Fint">4</FintConstant>
+   *   </value>
+   *   <name>close_door</name>
+   *   <value>
+   *     <FintConstant type="Fint">17</FintConstant>
+   *   </value>
+   *   <name>lock_door</name>
+   * </FenumType>
+   */
+  private void declFenumType(Node n) {
+    String tid = getAttr(n, "type");
+    long tq = 0;
+    XobjList moe_list = (XobjList)toXobject(getElement(n, "symbols"));
+    EnumType type = new EnumType(tid, moe_list, tq, null);
+    xobjFile.addType(type);
+  }
+  
   /*
    * global Ident section
    */
@@ -242,6 +342,8 @@ public class XcodeMLtools_F extends XcodeMLtools {
     case FUNCTION_DEFINITION:
     case F_MODULE_DEFINITION:
     case F_BLOCK_DATA_DEFINITION:
+    case PRAGMA_LINE:
+    case COMMENT_LINE:
       xobjFile.add(xobj);
       break;
     default:
@@ -267,6 +369,7 @@ public class XcodeMLtools_F extends XcodeMLtools {
       type = getType(t);
 
     switch (code) {
+    case F_MODULE_PROCEDURE_DEFINITION:
     case FUNCTION_DEFINITION:
       x = Xcons.List(code, type, toXobject(getElement(n, "name")),
 		     toXobject(getElement(n, "symbols")),
@@ -284,6 +387,8 @@ public class XcodeMLtools_F extends XcodeMLtools {
 		     toXobject(getElement(n, "declarations"))
 		     );
       x.add(toXobject(getElement(n, "FcontainsStatement")));
+      x.add(getAttrIntFlag(n, "is_sub"));
+      x.add(getSymbol(n, "parent_name"));
 
       markModuleVariable((XobjList)x.getArgOrNull(1),
 			 (XobjList)x.getArgOrNull(2));
@@ -298,14 +403,26 @@ public class XcodeMLtools_F extends XcodeMLtools {
     case F_USE_DECL:
     case F_USE_ONLY_DECL:
       x = getSymbol(n, "name");
+      boolean isIntrinsic = getAttrBool(n, "intrinsic");
       return setCommonAttributes(n,
-				 Xcons.List(code, type, x, getChildList(n)));
+        Xcons.List(code, type, x, Xcons.IntConstant(isIntrinsic ? 1 : 0), getChildList(n)));
+
+    case F_IMPORT_STATEMENT:
+      return setCommonAttributes(n,
+         Xcons.List(code, type, getChildList(n)));
+
+    case F_MODULE_PROCEDURE_DECL:
+      return setCommonAttributes(n, Xcons.List(code, type,
+                                               getAttrIntFlag(n, "is_module_specified"),
+                                               getChildList(n)));
 
     case F_INTERFACE_DECL:
-      boolean isOperator = getAttrBool(n, "is_operator");
-      boolean isAssignment = getAttrBool(n, "is_assignment");
-      return setCommonAttributes(n,
-				 Xcons.List(code, type, getSymbol(n, "name"), Xcons.IntConstant(isOperator ? 1 : 0), Xcons.IntConstant(isAssignment ? 1 : 0), getChildList(n)));
+      return setCommonAttributes(n, Xcons.List(code, type,
+                                               getSymbol(n, "name"),
+                                               getAttrIntFlag(n, "is_operator"),
+                                               getAttrIntFlag(n, "is_assignment"),
+                                               getChildList(n),
+					       getAttrIntFlag(n, "is_abstract")));
 
     case F_BLOCK_DATA_DEFINITION:
       x = getSymbol(n, "name");
@@ -315,9 +432,10 @@ public class XcodeMLtools_F extends XcodeMLtools {
 
     case FUNCTION_DECL:
       return setCommonAttributes(n, Xcons.List(code, type, toXobject(getElement(n, "name")),
-					       null, null, toXobject(getElement(n, "declarations"))));
+					       toXobject(getElement(n, "symbols")),
+					       null, toXobject(getElement(n, "declarations"))));
 
-    case STRING: 
+    case STRING:
       return Xcons.String(getContentText(n));
 
     case OMP_PRAGMA:
@@ -326,8 +444,9 @@ public class XcodeMLtools_F extends XcodeMLtools {
       return getChildList(n, xlist);
 
     case PRAGMA_LINE:
-      String contentText = getContentText(n);
-      x = Xcons.List(Xcode.PRAGMA_LINE,
+    case COMMENT_LINE:
+	String contentText = getContentText(n);
+      x = Xcons.List(code,
 		     new XobjString(Xcode.STRING, contentText));
       setCommonAttributes(n, x);
       return x;
@@ -358,7 +477,7 @@ public class XcodeMLtools_F extends XcodeMLtools {
       return Xcons.FlogicalConstant(type, value, getAttr(n, "kind"));
     }
 
-    case FUNC_ADDR:   
+    case FUNC_ADDR:
       return Xcons.Symbol(code, type, getContentText(n));
 
     case F_ALLOC:
@@ -388,6 +507,9 @@ public class XcodeMLtools_F extends XcodeMLtools {
     case MEMBER_REF:
       return Xcons.List(code, type, toXobject(getElement(n, "varRef")), getSymbol(n, "member"));
 
+    case F_COMPLEX_PART_REF:
+      return Xcons.List(code, type, toXobject(getElement(n, "varRef")), getSymbol(n, "part"));
+
     case F_USER_BINARY_EXPR: {
       XobjList xx = Xcons.List(code, type);
       x = getChildList(n, xx);
@@ -414,6 +536,12 @@ public class XcodeMLtools_F extends XcodeMLtools {
 			getAttrIntFlag(n, "is_assumed_shape"),
 			getAttrIntFlag(n, "is_assumed_size"));
 
+    case F_LEN:
+      return Xcons.List(code, type,
+			toXobject(getContent(n)),
+			getAttrIntFlag(n, "is_assumed_shape"),
+			getAttrIntFlag(n, "is_assumed_size"));
+
     case F_VALUE:
       {
 	x = null;
@@ -436,7 +564,14 @@ public class XcodeMLtools_F extends XcodeMLtools {
     }
 
     case FUNCTION_CALL:
-      return Xcons.List(code, type, toXobject(getElement(n, "name")),
+      Node n_func = getElement(n, "name");
+      if (n_func == null) {
+        n_func = getElement(n, "FmemberRef");
+      }
+      if (n_func == null) {
+        fatal("unknown functionCall function.");
+      }
+      return Xcons.List(code, type, toXobject(n_func),
 			toXobject(getElement(n, "arguments")),
 			getAttrIntFlag(n, "is_intrinsic"));
 
@@ -460,9 +595,14 @@ public class XcodeMLtools_F extends XcodeMLtools {
 	return getChildList(n, xx);
       }
 
+    case F_RENAME:
+      boolean isOperator = getAttrBool(n, "is_operator");
+      return Xcons.List(code, type, Xcons.IntConstant(isOperator ? 1 : 0),
+			getSymbol(n, "use_name"), getSymbol(n, "local_name"));
+      
     case F_RENAMABLE:
       return Xcons.List(code, type, getSymbol(n, "use_name"), getSymbol(n, "local_name"));
-		
+
     case F_DO_WHILE_STATEMENT:
       attr = getSymbol(n, "construct_name");
       return setCommonAttributes(n, Xcons.List(code, type, attr,
@@ -527,20 +667,61 @@ public class XcodeMLtools_F extends XcodeMLtools {
 	return setCommonAttributes(n, x);
       }
 
+    case SELECT_TYPE_STATEMENT:
+        {
+          x = new XobjList(code, type);
+          x.add(getSymbol(n, "construct_name"));
+          NodeList list = n.getChildNodes();
+          XobjList typeGuards = new XobjList();
+          for (int i = 0; i < list.getLength(); i++) {
+              Node nn = list.item(i);
+              if (nn.getNodeType() != Node.ELEMENT_NODE)
+                  continue;
+              if (nn.getNodeName() == "typeGuard") {
+                typeGuards.add(toXobject(nn));
+              } else if (nn.getNodeName() == "id") {
+                x.add(toIdent(nn));
+              }
+          }
+          x.add(typeGuards);
+          return setCommonAttributes(n, x);
+        }
+    case TYPE_GUARD:
+        {
+            x = new XobjList(code, type);
+            x.add(getSymbol(n, "construct_name"));
+            x.add(getSymbol(n, "kind"));
+            x.add(getSymbol(n, "type"));
+            NodeList list = n.getChildNodes();
+            XobjList values = new XobjList();
+            x.add(toXobject(getElement(n, "body")));
+            return setCommonAttributes(n, x);
+        }
+
+
+
     case F_WHERE_STATEMENT:
       x = new XobjList(code, type);
-      x.add(null);
+      x.add(getSymbol(n, "construct_name"));
       x.add(toXobject(getContent(getElement(n, "condition"))));
       x.add(toXobject(getContent(getElement(n, "then"))));
       x.add(toXobject(getContent(getElement(n, "else"))));
       return setCommonAttributes(n, x);
 
     case F_STOP_STATEMENT:
+    case F_ERROR_STOP_STATEMENT:
       {
 	t = getAttr(n, "code");
 	Xobject cd = (t == null ? null : Xcons.String(t));
 	t = getAttr(n, "message");
-	Xobject mes = (t == null ? null : Xcons.FcharacterConstant(Xtype.FcharacterType, t, null));
+	Xobject mes = null;
+	if (t != null){
+	  mes = Xcons.FcharacterConstant(Xtype.FcharacterType, t, null);
+	}
+	else {
+	  mes = toXobject(getContent(getElement(n, "message")));
+	}
+	//	Xobject mes = (t == null ? null : Xcons.FcharacterConstant(Xtype.FcharacterType, t, null));
 	return setCommonAttributes(n, Xcons.List(code, type, cd, mes));
       }
 
@@ -552,13 +733,13 @@ public class XcodeMLtools_F extends XcodeMLtools {
 	Xobject mes = (t == null) ? null : Xcons.FcharacterConstant(Xtype.FcharacterType, t, null);
 	return setCommonAttributes(n, Xcons.List(code, type, cd, mes));
       }
-		
+
     case F_READ_STATEMENT:
     case F_WRITE_STATEMENT:
-      return Xcons.List(code, type,
-			toXobject(getElement(n, "namedValueList")),
-			toXobject(getElement(n, "valueList"))
-			);
+      return setCommonAttributes(n, Xcons.List(code, type,
+					       toXobject(getElement(n, "namedValueList")),
+					       toXobject(getElement(n, "valueList"))
+					       ));
 
     case F_PRINT_STATEMENT:
       t = getAttr(n, "format");
@@ -567,6 +748,11 @@ public class XcodeMLtools_F extends XcodeMLtools {
 					       toXobject(getElement(n, "valueList"))
 					       ));
 
+    case F_FLUSH_STATEMENT:
+      return setCommonAttributes(n, Xcons.List(code, type,
+					       toXobject(getElement(n, "namedValueList"))
+					       ));
+
     case F_DO_LOOP:
       x = Xcons.List();
       {
@@ -595,6 +781,7 @@ public class XcodeMLtools_F extends XcodeMLtools {
       x = Xcons.String(t);
       return setCommonAttributes(n, Xcons.List(code, type, x));
 
+    case F_DATA_STATEMENT:
     case F_DATA_DECL:
       return setCommonAttributes(n, Xcons.List(code, type, Xcons.List(toXobject(getElement(n, "varList")), toXobject(getElement(n, "valueList")))));
 
@@ -603,17 +790,215 @@ public class XcodeMLtools_F extends XcodeMLtools {
 
     case F_ALLOCATE_STATEMENT:
     case F_DEALLOCATE_STATEMENT:
-      x = Xcons.List(code, type);
-      x.add(getSymbol(n, "stat_name"));
-      x.add(getChildList(n));
-      return setCommonAttributes(n, x);
+      {
+	// x = Xcons.List(code, type);
+	// //x.add(getSymbol(n, "stat_name"));
+	// //x.add(getChildList(n));
 
+	// XobjList xx = Xcons.List();
+
+	// NodeList list = n.getChildNodes();
+	// for (int i = 0; i < list.getLength(); i++) {
+	//   Node nn = list.item(i);
+	//   String name = nn.getNodeName();
+	//   if (name == "allocOpt"){
+	//     switch (getAttr(nn, "kind")){
+	//     case "stat":
+	//       XobjList v = getChildList(nn);
+	//       x.add(v.getArg(0));
+	//       break;
+	//     default:
+	//       // for this moment, do nothing.
+	//     }
+	//     continue;
+	//   }
+	//   else if (name == "alloc"){
+	//     xx.add(toXobject(nn));
+	//   }
+	//   else
+	//     continue;
+	// }
+
+	// if (x.Nargs() == 0) x.add(null);
+	// x.add(xx);
+
+	x = Xcons.List(code, type, null, null, null, null, null);
+	XobjList xx = Xcons.List();
+	Xobject v = null;
+
+	NodeList list = n.getChildNodes();
+	for (int i = 0; i < list.getLength(); i++) {
+	  Node nn = list.item(i);
+	  String name = nn.getNodeName();
+	  if (name == "allocOpt"){
+	    switch (getAttr(nn, "kind")){
+	    case "stat":
+	      v = getChildList(nn);
+	      x.setArg(0, v.getArg(0));
+	      break;
+	    case "source":
+	      v = getChildList(nn);
+	      x.setArg(2, v.getArg(0));
+	      break;
+	    case "mold":
+	      v = getChildList(nn);
+	      x.setArg(3, v.getArg(0));
+	      break;
+	    case "errmsg":
+	      v = getChildList(nn);
+	      x.setArg(4, v.getArg(0));
+	      break;
+	    default:
+	      fatal("Unknown AllocOpt: " + nn);
+	    }
+	    continue;
+	  }
+	  else if (name == "alloc"){
+	    xx.add(toXobject(nn));
+	  }
+	  else
+	    continue;
+	}
+
+	x.setArg(1, xx);
+	  
+	return setCommonAttributes(n, x);
+      }
     case F_CONTAINS_STATEMENT:
       {
 	XobjList xx = Xcons.List(code, type);
 	return getChildList(n, xx);
       }
 
+    case F_CRITICAL_STATEMENT:
+      {
+        attr = getSymbol(n, "construct_name");
+        return setCommonAttributes(n, Xcons.List(code, type, attr,
+						 toXobject(getElement(n, "body"))
+						 ));
+      }
+    case F_BLOCK_STATEMENT:
+      {
+        attr = getSymbol(n, "construct_name");
+        return setCommonAttributes(n, Xcons.List(code, type, attr,
+						 toXobject(getElement(n, "symbols")),
+						 toXobject(getElement(n, "declarations")),
+						 toXobject(getElement(n, "body"))
+						 ));
+      }
+
+    case F_ASSOCIATE_STATEMENT:
+      {
+        attr = getSymbol(n, "construct_name");
+        return setCommonAttributes(n, Xcons.List(code, type, attr,
+						 toXobject(getElement(n, "symbols")),
+						 toXobject(getElement(n, "body"))
+						 ));
+      }
+      
+    case F_SYNC_STAT:
+      {
+        attr = getSymbol(n, "kind");
+        return setCommonAttributes(n, Xcons.List(code, type, attr,
+                                                 toXobject(getContent(n))
+                                                 ));
+      }
+
+    case F_TYPE_PARAM:
+      {
+        attr = getSymbol(n, "attr");
+        return setCommonAttributes(n, Xcons.List(code, type, attr,
+						 toXobject(getElement(n, "name")),
+						 toXobject(getElement(n, "value"))
+						 ));
+      }
+
+    case F_TYPE_BOUND_PROCEDURE:
+      {
+        XobjString pass     = Xcons.String(getAttr(n, "pass"         ));
+        XobjString pass_arg = Xcons.String(getAttr(n, "pass_arg_name"));
+        long tq = (getAttrBool(n, "is_private") ? Xtype.TQ_FPRIVATE : 0)
+               | (getAttrBool(n, "is_public" ) ? Xtype.TQ_FPUBLIC  : 0)
+	       | (getAttrBool(n, "is_protected") ? Xtype.TQ_FPROTECTED : 0)
+	       | (getAttrBool(n, "is_deferred") ? Xtype.TQ_FDEFERRED : 0);
+        Node bdg = getElement(n, "binding");
+        return setCommonAttributes(n, Xcons.List(code, type, pass, pass_arg,
+                                                 toXobject(getElement(n, "name")),
+                                                 Xcons.LongConstant(tq),
+                                                 (bdg != null) ? toXobject(getContent(bdg)) : null,
+                                                 getAttrIntFlag(n, "is_non_overridable")
+                                                ));
+      }
+
+    case F_FINAL_PROCEDURE:
+      {
+	return setCommonAttributes(n, Xcons.List(code, toXobject(getElement(n, "name"))
+                                                ));
+      }
+      
+    case F_TYPE_BOUND_GENERIC_PROCEDURE:
+      {
+        long tq = (getAttrBool(n, "is_private") ? Xtype.TQ_FPRIVATE : 0)
+               | (getAttrBool(n, "is_public" ) ? Xtype.TQ_FPUBLIC  : 0)
+	       | (getAttrBool(n, "is_protected") ? Xtype.TQ_FPROTECTED : 0);
+        Node bdg = getElement(n, "binding");
+
+	Xobject name = toXobject(getElement(n, "name"));
+	if (name == null){
+	  String defined_io = getAttr(n, "is_defined_io");
+	  name = Xcons.String(defined_io);
+	}
+	
+        return setCommonAttributes(n, Xcons.List(code, (Xtype)null,
+                                                 getAttrIntFlag(n, "is_operator"),
+                                                 getAttrIntFlag(n, "is_assignment"),
+                                                 name,
+                                                 Xcons.LongConstant(tq),
+                                                 toXobject(bdg)
+                                                ));
+      }
+
+    case F_ARRAY_CONSTRUCTOR:
+      {
+        XobjList xobjp = new XobjList(code, type);
+        xobjp.add(getSymbol(n, "element_type"));
+        XobjList xobjs = new XobjList();
+        xobjp.add(xobjs);
+        NodeList list = n.getChildNodes();
+        for (int i = 0; i < list.getLength(); i++) {
+          Node nn = list.item(i);
+          if (nn.getNodeType() != Node.ELEMENT_NODE)
+            continue;
+          xobjs.add(toXobject(nn));
+        }
+        return setCommonAttributes(n, xobjp);
+      }
+
+    case F_CONDITION:
+      return toXobject(getContent(n));
+
+    case F_FORALL_STATEMENT:
+    case F_DO_CONCURRENT_STATEMENT:
+      {
+        XobjList xobj = new XobjList(code, type);
+        xobj.add(getSymbol(n, "construct_name"));
+        NodeList list = n.getChildNodes();
+        for (int i = 0; i < list.getLength(); i++) {
+          Node nn = list.item(i);
+          if (nn.getNodeType() != Node.ELEMENT_NODE)
+            continue;
+          xobj.add(toXobject(nn));
+        }
+        return setCommonAttributes(n, xobj);
+      }
+
+    case F_TYPE_BOUND_PROCEDURES:
+    case F_BINDING:
+    case F_SYNCALL_STATEMENT:
+    case F_SYNCIMAGE_STATEMENT:
+    case F_SYNCMEMORY_STATEMENT:
+    case F_LOCK_STATEMENT:
+    case F_UNLOCK_STATEMENT:
     default: // default action, make list
       NodeList list = n.getChildNodes();
       if(code == Xcode.LIST && list.getLength() == 0)
@@ -675,8 +1060,10 @@ public class XcodeMLtools_F extends XcodeMLtools {
 	addr.setScope(VarScope.LOCAL);
 	break;
       }
-    } else if (valueNode != null) {
-        addr = toXobject(valueNode);
+    }
+    
+    if (valueNode != null) {
+      addr = toXobject(valueNode);
     }
 
     // create ident
