@@ -2,23 +2,16 @@ package exc.xcodeml;
 
 import static xcodeml.util.XmLog.fatal;
 import static xcodeml.util.XmLog.fatal_dump;
+import xcodeml.util.ILineNo;
 
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 
-import xcodeml.ILineNo;
-import exc.object.BasicType;
-import exc.object.Ident;
-import exc.object.StorageClass;
-import exc.object.Xcode;
-import exc.object.Xcons;
-import exc.object.XobjList;
-import exc.object.XobjString;
-import exc.object.Xobject;
-import exc.object.XobjectDef;
-import exc.object.XobjectDefEnv;
-import exc.object.Xtype;
+import exc.object.*;
 
+/**
+ * convert Xobject/C to XcodeML(DOM)/C
+ */
 public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
     private XcodeMLNameTable_C nameTable = new XcodeMLNameTable_C();
 
@@ -26,12 +19,16 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
         if (xobj == null) {
             return null;
         }
-
-        Element e = createElement("name");
+        Element e = (xobj instanceof XobjString && ((XobjString)xobj).isOp()) ?
+                    createElement("operator") : createElement("name");
         if (xobj.Type() != null) {
             addAttributes(e,
                           "type", xobj.Type().getXcodeCId());
         }
+        if (xobj instanceof XobjString) {
+            addAttributes(e,
+                          "fullName", ((XobjString)xobj).getAlias());
+        }
         addChildNodes(e, trans(xobj.getString()));
 
         return e;
@@ -77,9 +74,15 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
         }
 
         // name
-        addChildNodes(e,
-                      addChildNodes(createElement("name"),
-                                    trans(ident.getName())));
+        Element en = ident.isOp() ? createElement("operator") : createElement("name");
+        String full_name = ident.getFullName();
+        if (full_name != null) {
+            addAttributes(en, "fullName", full_name);
+        }
+        addChildNodes(e, addChildNodes(en, trans(ident.getName())));
+
+        // public, protected or private
+        addAttributes(e, "access", ident.getAccessStr());
 
         // sclass
         if (ident.getStorageClass() != null) {
@@ -278,8 +281,14 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
                                   transSymbols(type.getMemberList()));
                 break;
             case Xtype.STRUCT:
-                e = addChildNodes(createTypeElement("structType", type),
-                                  transSymbols(type.getMemberList()));
+                e = ((StructType)type).isClass() ? createTypeElement("classType", type) :
+                                                   createTypeElement("structType", type);
+                XobjString tagNames = ((CompositeType)type).getTagNames();
+                if (tagNames != null) {
+                  Element e_tagnames = transName(tagNames);
+                  e = addChildNodes(e, e_tagnames);
+                }
+                e = addChildNodes(e, transSymbols(type.getMemberList()));
                 break;
             case Xtype.UNION:
                 e = addChildNodes(createTypeElement("unionType", type),
@@ -494,6 +503,7 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
         case FUNCTION_DECL:
             e = addChildNodes(createElement(name),
                               transName(xobj.getArg(0)),
+                              trans(xobj.getArgOrNull(1)),
                               trans(xobj.getArgOrNull(2)));
             break;
 
@@ -656,12 +666,20 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
             case VAR:
             case VAR_ADDR:
                 nSymAddr = createElement("varAddr");
+                if (operand.getScope() != null) {
+                    addAttributes(nSymAddr,
+                                  "scope", operand.getScope().toXcodeString());
+                }
                 break;
             case ARRAY_REF:
               isArrayRef = true;
               break;
             case ARRAY_ADDR: /* illegal but convert */
                 nSymAddr = createElement("arrayAddr");
+                if (operand.getScope() != null) {
+                    addAttributes(nSymAddr,
+                                  "scope", operand.getScope().toXcodeString());
+                }
                 break;
             case FUNC_ADDR:
                 nSymAddr = createElement("funcAddr");
@@ -698,26 +716,37 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
             break;
         case FUNCTION_CALL: {
             e = createElement(name);
-            Element nFunc = createElement("function");
+            Element nFunc;
+            switch (xobj.getArg(0).Opcode()) {
+            case MEMBER_REF:
+            case CPP_OPERATOR_ADDR:
+              nFunc = transExprOrError(xobj.getArg(0));
+              break;
+            case FUNC_ADDR:
+            case POINTER_REF:
+            default:
+              nFunc = createElement("function");
+              addChildNodes(nFunc,
+                            transExprOrError(xobj.getArg(0)));
+            }
             Element nArgs = createElement("arguments");
-            addChildNodes(nFunc,
-                          transExprOrError(xobj.getArg(0)));
             XobjList params = (XobjList)xobj.getArg(1);
             if (params != null) {
                 for (Xobject a : params) {
                     addChildNodes(nArgs, transExprOrError(a));
                 }
             }
-
-            addChildNodes(e,
-                          nFunc,
-                          nArgs);
+            addChildNodes(e, nFunc, nArgs);
         }
             break;
         case SIZE_OF_EXPR:
             e = transSizeOrAlignOf(xobj);
             break;
         case CAST_EXPR:
+        case CPP_CONST_CAST_EXPR:
+        case CPP_STATIC_CAST_EXPR:
+        case CPP_REINTERPRET_CAST_EXPR:
+        case CPP_DYNAMIC_CAST_EXPR:
             e = addChildNodes(createElement(name),
                               transOrError(xobj.getArg(0)));
             break;
@@ -807,6 +836,7 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
             }
             break;
         case FUNC_ADDR:
+        case CPP_OPERATOR_ADDR:
             e = addChildNodes(createElement(name),
                               trans(xobj.getName()));
             break;
@@ -955,13 +985,15 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
 
         // directive
         case PRAGMA_LINE:
-            e = addChildNodes(createElement(name),
-                              trans(xobj.getArg(0).getString()));
-            break;
         case TEXT:
             e = addChildNodes(createElement(name),
                               trans(xobj.getArg(0).getString()));
             break;
+	case LINEMARKER:
+            e = addChildNodes(createElement(name));
+	    addAttributes(e,
+			  "flag", xobj.getArg(0).getString());
+            break;
         case OMP_PRAGMA: {
             e = createElement(name);
 
@@ -987,7 +1019,7 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
 			  Xobject vars = a.getArg(1);
 			  if (vars != null){
 
-			    if (vars instanceof XobjString){
+			    if (vars instanceof XobjString || vars instanceof XobjInt){
 			      addChildNode(g, trans(vars));
 			    }
 			    else {
@@ -1008,6 +1040,10 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
             }
             addChildNode(e, f1);
 
+            // barrier directive
+            if (xobj.Nargs() < 3)
+                break;
+
             Element f2 = createElement("list");
             Xobject body = xobj.getArg(2);
             if (body != null){
@@ -1042,19 +1078,21 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
 
             Xobject clause = xobj.getArg(1);
             if (clause != null){
-        	if(clause instanceof XobjList){
+        	if(clause.Opcode() == Xcode.LIST){
         	    //clause list
         	    Element f1 = createElement("list");
 
         	    for(Xobject a : (XobjList)clause){
-        		if(a instanceof XobjList){
+        		if(a.Opcode() == Xcode.LIST){
         		    //clause name
         		    Element g = createElement("list");
-        		    addChildNode(g, trans(a.getArg(0).getString()));
+        		    Element g0 = createElement("string");
+        		    addChildNode(g0, trans(a.getArg(0).getString()));
+                            addChildNode(g, g0);
 
         		    Xobject vars = a.getArgOrNull(1);
         		    if (vars != null){
-        			if (vars instanceof XobjList){
+        			if (vars.Opcode() == Xcode.LIST){
         			    Element g1 = createElement("list");
         			    for (Xobject b : (XobjList)vars){
         				addChildNode(g1, transACCPragmaVarOrArray(b));
@@ -1104,9 +1142,9 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
             break;
         case INDEX_RANGE:
             e = addChildNodes(createElement(name),
-                              trans(xobj.getArg(0)),
-                              trans(xobj.getArg(1)),
-                              trans(xobj.getArg(2)));
+                    addChildNode(createElement("base"),   trans(xobj.getArg(0))),
+                    addChildNode(createElement("length"), trans(xobj.getArg(1))),
+                    addChildNode(createElement("step"),   trans(xobj.getArg(2))));
             break;
         case LOWER_BOUND:
         case UPPER_BOUND:
@@ -1114,10 +1152,27 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
             e = addChildNodes(createElement(name),
                               trans(xobj.getArg(0)));
             break;
+        case CPP_DECL_NAMESPACE:
+        case CPP_CLASS_DECL:
+            e = createElement(name);
+            for (Xobject a : (XobjList)xobj) {
+                Element b;
+                if (a instanceof Ident)
+                   b = transIdent((Ident)a);
+                else
+                   b = trans(a);
+                addChildNode(e, b);
+            }
+            break;
         // case ADDR_OF_EXPR:
         //     e = addChildNodes(createElement(name),
         //                       transExpr(xobj.getArg(0)));
         //     break;
+        case CPP_CONSTRUCT_EXPR_CLASS_STATEMENT:
+        case CPP_THIS_EXPR:
+        case CPP_NESTEDNAMESPECIFIER_TYPESPEC:
+            e = createElement(name);
+            break;
         default:
             fatal_dump("cannot convert Xcode to XcodeML.", xobj);
         }
@@ -1159,7 +1214,7 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
     }
 
     /* Copied from XmcXobjectToXmObjTranslator.java */
-    private XobjList getDeclForNotDeclared(XobjList identList) {
+    public static XobjList getDeclForNotDeclared(XobjList identList) {
         if (identList == null) {
             return null;
         }
