@@ -1,18 +1,12 @@
-/* 
- * $TSUKUBA_Release: Omni XcalableMP Compiler 3 $
- * $TSUKUBA_Copyright:
- *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
- *  $
- */
 package exc.xmpF;
 
 import java.util.ArrayList;
 import java.util.List;
-
 import exc.object.*;
 import exc.block.*;
 import java.util.*;
 import xcodeml.util.XmOption;
+import exc.xcalablemp.*;
 
 /**
  * process analyze XcalableMP pragma
@@ -69,6 +63,7 @@ public class XMPanalyzePragma
     }
   }
 
+  // check use and import if use of module is found.
   private void checkUseDecl(Xobject decls){
     if(decls == null) return;
     for(Xobject decl: (XobjList)decls){
@@ -194,11 +189,38 @@ public class XMPanalyzePragma
       analyzeTemplateFix(pb.getClauses(), info, pb);
       break;
 
-    case LOOP:
-      analyzeLoop(pb.getClauses(), pb.getBody(), pb,info);
-      break; 
+    case LOOP: {
 
+      Block newBlock = null;
+      XobjList expandOpt = (XobjList)pb.getClauses().getArg(2);
+
+      if (expandOpt == null || expandOpt.hasNullArg()){
+	;
+      }
+      else if (expandOpt.getArg(0).getInt() == XMP.LOOP_MARGIN){
+	newBlock = divideMarginLoop(pb);
+      }
+      else if (expandOpt.getArg(0).getInt() == XMP.LOOP_PEEL_AND_WAIT){
+	newBlock = peelLoop(pb);
+      }
+      
+      if (newBlock != null){
+	Block next;
+	for (Block b = newBlock.getBody().getHead(); b != null; b = next){
+	  next = b.getNext();
+	  analyzePragma((PragmaBlock)b);
+	}
+	pb.replace(newBlock);
+      }
+      else {
+	analyzeLoop(pb.getClauses(), pb.getBody(), pb, info);
+      }
+      break;
+      
+    }
+      
     case REFLECT:
+    case REDUCE_SHADOW:
       analyzeReflect(pb.getClauses(),info,pb);
       break;
 
@@ -231,11 +253,19 @@ public class XMPanalyzePragma
       break;
 
     case COARRAY:
-      XMPcoarray.analyzeCoarrayDirective(pb.getClauses(), env, pb);
+      if (XmOption.isCoarray()) {
+        XMPcoarray.analyzeCoarrayDirective(pb.getClauses(), env, pb);
+      } else {
+        XMP.error("coarray directive is specified.");
+      }
       break;
 
     case IMAGE:
-      XMPtransCoarrayRun.analyzeImageDirective(pb.getClauses(), env, pb);
+      if (XmOption.isCoarray()) {
+        XMPtransCoarrayRun.analyzeImageDirective(pb.getClauses(), env, pb);
+      } else {
+        XMP.error("image directive is specified.");
+      }
       break;
 
     case ARRAY:
@@ -358,62 +388,58 @@ public class XMPanalyzePragma
 
   }
 
-
   /* 
    * analyze Loop directive:
    *  loopDecl = (on_ref | ...)
    */
   void analyzeLoop(Xobject loopDecl, BlockList loopBody,
 		   PragmaBlock pb, XMPinfo info) {
+
     // get block to schedule
     Vector<XMPdimInfo> dims = new Vector<XMPdimInfo>();
-    
+
     // schedule loop
     XobjList loopIterList = (XobjList)loopDecl.getArg(0);
+    
     if (loopIterList == null || loopIterList.Nargs() == 0) {
+      XobjList onRef          = (XobjList)loopDecl.getArg(1);
+      XobjList onRefIterList  = (XobjList)onRef.getArg(1);
+      loopIterList = XMPutil.getLoopIterListFromOnRef(onRefIterList);
+    }
+    
+    while(true){
       ForBlock loopBlock = getOutermostLoopBlock(loopBody);
-      if(loopBlock == null){
-    	  XMP.errorAt(pb,"loop is not found after loop directive");
-    	  return;
-      }
-      dims.add(XMPdimInfo.loopInfo(loopBlock));
-      loopBody = loopBlock.getBody();
-    } else {
-      while(true){
-    	  ForBlock loopBlock = getOutermostLoopBlock(loopBody);
-    	  if(loopBlock == null) break;
-    	  boolean is_found = false;
-    	  for(Xobject x: loopIterList){
-    		  if(x.Opcode() == Xcode.LIST) x = x.getArg(0);
-    		  if(isEqualVar(loopBlock.getInductionVar(),x)){
-    			  is_found = true;
-    			  break;
-    		  }
-    	  }
-    	  if(is_found)
-    		  dims.add(XMPdimInfo.loopInfo(loopBlock));
-    	  loopBody = loopBlock.getBody();
-      }
-
-      /* check loopIterList */
+      if(loopBlock == null) break;
+      boolean is_found = false;
       for(Xobject x: loopIterList){
-    	  if(x.Opcode() == Xcode.LIST){
-    		  if(x.getArgOrNull(1) != null ||
-    		     x.getArgOrNull(2) != null){
-    			  XMP.errorAt(pb,"bad syntax in loop directive");
-    		  }
-    		  x = x.getArg(0);
-    	  }
-    	  boolean is_found = false;
-    	  for(XMPdimInfo d_info: dims){
-    		  if(isEqualVar(d_info.getLoopVar(),x)){
-    			  is_found = true;
-    			  break;
-    		  }
-    	  }
-    	  if(!is_found)
-    		  XMP.errorAt(pb,"loop index is not found in loop varaibles");
+	if(x.Opcode() == Xcode.LIST) x = x.getArg(0);
+	if(isEqualVar(loopBlock.getInductionVar(),x)){
+	  is_found = true;
+	  break;
+	}
       }
+      if(is_found)
+	dims.add(XMPdimInfo.loopInfo(loopBlock));
+      loopBody = loopBlock.getBody();
+    }
+    
+    /* check loopIterList */
+    for(Xobject x: loopIterList){
+      if(x.Opcode() == Xcode.LIST){
+	if(x.getArgOrNull(1) != null || x.getArgOrNull(2) != null){
+	  XMP.errorAt(pb,"bad syntax in loop directive");
+	}
+	x = x.getArg(0);
+      }
+      boolean is_found = false;
+      for(XMPdimInfo d_info: dims){
+	if(isEqualVar(d_info.getLoopVar(),x)){
+	  is_found = true;
+	  break;
+	}
+      }
+      if(!is_found)
+	XMP.errorAt(pb,"loop index is not found in loop varaibles");
     }
     
     XMPobjectsRef on_ref = XMPobjectsRef.parseDecl(loopDecl.getArg(1),env,pb);
@@ -470,7 +496,27 @@ public class XMPanalyzePragma
       }
     }
     
-    Xobject reductionSpec = loopDecl.getArg(2);
+    // width list
+    XobjList expandOpt = (XobjList)loopDecl.getArg(2);
+    int loopType;
+    Vector<XMPdimInfo> widthList = new Vector<XMPdimInfo>();
+    if (expandOpt == null || expandOpt.hasNullArg()){
+      loopType = XMP.LOOP_NONE;
+    }
+    else {
+      loopType = expandOpt.getArg(0).getInt();
+      for (Xobject x: (XobjList)expandOpt.getArg(1)){
+    	XMPdimInfo width = new XMPdimInfo();
+
+    	if (XMP.debugFlag)
+    	  System.out.println("width = ("+x.getArg(0)+":"+x.getArg(1)+":"+x.getArg(2)+")");
+
+    	width.parse(x);
+    	widthList.add(width);
+      }
+    }
+    
+    Xobject reductionSpec = loopDecl.getArg(3);
     if(reductionSpec != null) 
       analyzeReductionSpec(info, reductionSpec, pb);
 
@@ -479,7 +525,129 @@ public class XMPanalyzePragma
     checkLocalizableLoop(dims,on_ref,pb);
 
     info.setBody(loopBody);  // inner most body
-    info.setLoopInfo(dims, on_ref);
+    info.setLoopInfo(dims, on_ref, loopType, widthList);
+  }
+
+  private static Block divideMarginLoop(PragmaBlock pb){
+
+    // The type is XMP.LOOP_MARGIN
+
+    BlockList loops = Bcons.emptyBody();
+    boolean flag = false;
+    
+    XobjList expandOpt = (XobjList)pb.getClauses().getArg(2);
+
+    // System.out.println("("+expandOpt.getArg(1).getArg(0).getArg(0).getInt()+" : "+expandOpt.getArg(1).getArg(0).getArg(1).getInt()+" ,"
+    // 		       +expandOpt.getArg(1).getArg(1).getArg(0).getInt()+" : "+expandOpt.getArg(1).getArg(1).getArg(1).getInt()+")");
+
+    PragmaBlock pb1, pb2;
+    XobjList expandOpt1 = null, expandOpt2 = null;
+
+    for (int i = 0; i < expandOpt.getArg(1).Nargs(); i++){
+
+      Xobject expandWidth = expandOpt.getArg(1).getArg(i);
+      Xobject lower = expandWidth.getArg(0);
+      Xobject upper = expandWidth.getArg(1);
+
+      Xobject stride = expandWidth.getArg(2);
+      if (stride.isIntConstant() && stride.getInt() == -1) continue;
+
+      if (!lower.isZeroConstant() && !upper.isZeroConstant()){
+
+	flag = true;
+
+	// PragmaBlock pb1, pb2;
+	// XobjList expandOpt1, expandOpt2;
+	
+	// for lower margin
+	pb1 = (PragmaBlock)pb.copy();
+	expandOpt1 = (XobjList)pb1.getClauses().getArg(2);
+
+	for (int j = 0; j < expandOpt1.getArg(1).Nargs(); j++){
+	  Xobject expandWidth1 = expandOpt1.getArg(1).getArg(j);
+	  if (j == i){
+	    expandWidth1.setArg(0, lower);
+	    expandWidth1.setArg(1, Xcons.IntConstant(0));
+	  }
+	  else if (j > i){
+	    expandWidth1.setArg(2, Xcons.IntConstant(-1)); // edge of margin
+	  }
+	  else { // j < i
+	    expandWidth1.setArg(0, Xcons.IntConstant(0));
+	    expandWidth1.setArg(1, Xcons.IntConstant(0));
+	  }
+	}
+
+	// System.out.println(" ("+expandOpt1.getArg(1).getArg(0).getArg(0).getInt()+" : "+expandOpt1.getArg(1).getArg(0).getArg(1).getInt()+" ,"
+	// 		   +expandOpt1.getArg(1).getArg(1).getArg(0).getInt()+" : "+expandOpt1.getArg(1).getArg(1).getArg(1).getInt()+")");
+
+	loops.add(pb1);
+			     
+	// for upper margin
+	pb2 = (PragmaBlock)pb.copy();
+	expandOpt2 = (XobjList)pb2.getClauses().getArg(2);
+
+	for (int j = 0; j < expandOpt1.getArg(1).Nargs(); j++){
+	  Xobject expandWidth2 = expandOpt2.getArg(1).getArg(j);
+	  if (j == i){
+	    expandWidth2.setArg(0, Xcons.IntConstant(0));
+	    expandWidth2.setArg(1, upper);
+	  }
+	  else if (j > i){
+	    expandWidth2.setArg(2, Xcons.IntConstant(-1)); // edge of margin
+	  }
+	  else { // j < i
+	    expandWidth2.setArg(0, Xcons.IntConstant(0));
+	    expandWidth2.setArg(1, Xcons.IntConstant(0));
+	  }	    
+	}
+
+      // System.out.println(" ("+expandOpt2.getArg(1).getArg(0).getArg(0).getInt()+" : "+expandOpt2.getArg(1).getArg(0).getArg(1).getInt()+" ,"
+      // 			 +expandOpt2.getArg(1).getArg(1).getArg(0).getInt()+" : "+expandOpt2.getArg(1).getArg(1).getArg(1).getInt()+")");
+
+    
+	loops.add(pb2);
+
+      }
+
+    }
+
+    if (flag){
+      return Bcons.COMPOUND(loops);
+    }
+    else {
+      return null;
+    }
+
+  }
+
+  private static Block peelLoop(PragmaBlock pb){
+
+    // The type is XMP.LOOP_PEEL_AND_WAIT
+
+    BlockList bl = Bcons.emptyBody();
+
+    // First, create the kernel loop
+    PragmaBlock pb3 = (PragmaBlock)pb.copy();
+    pb3.getClauses().getArg(2).setArg(0, Xcons.IntConstant(XMP.LOOP_EXPAND));
+    pb3.getClauses().getArg(2).setArg(1, pb.getClauses().getArg(2).getArg(2));
+    bl.add(pb3);
+
+    // Second, create wait_async
+    XobjList clauses = Xcons.List();
+    clauses.add(Xcons.List(pb.getClauses().getArg(2).getArg(1)));
+    clauses.add(null);
+    PragmaBlock pb2 = new PragmaBlock(Xcode.XMP_PRAGMA, "WAIT_ASYNC", clauses, null);
+    bl.add(pb2);
+    
+    // Third, create the peeled Loop
+    PragmaBlock pb1 = (PragmaBlock)pb.copy();
+    pb1.getClauses().getArg(2).setArg(0, Xcons.IntConstant(XMP.LOOP_MARGIN));
+    pb1.getClauses().getArg(2).setArg(1, pb.getClauses().getArg(2).getArg(2));
+    bl.add(pb1);
+
+    return Bcons.COMPOUND(bl);
+
   }
 
   private static ForBlock getOutermostLoopBlock(BlockList body) {
@@ -500,7 +668,7 @@ public class XMPanalyzePragma
       }
       else if (b.Opcode() == Xcode.COMPOUND_STATEMENT)
     	  return getOutermostLoopBlock(b.getBody());
-      else if (b.Opcode() == Xcode.OMP_PRAGMA)
+      else if (b.Opcode() == Xcode.OMP_PRAGMA || b.Opcode() == Xcode.ACC_PRAGMA)
     	  return getOutermostLoopBlock(b.getBody());
 //       else if(b.Opcode() == Xcode.F_STATEMENT_LIST &&
 // 	      b.getBasicBlock().getHead().getExpr().Opcode() 
@@ -555,6 +723,7 @@ public class XMPanalyzePragma
     XobjList reflectNameList = (XobjList) reflectDecl.getArg(0);
     XobjList widthOpt = (XobjList) reflectDecl.getArg(1);
     Xobject asyncOpt = reflectDecl.getArg(2);
+    Xobject accOpt = reflectDecl.getArg(3);
 
 //     if(reflectOpt != null){
 //       XMP.fatal("reflect opt is not supported yet, sorry!");
@@ -597,6 +766,10 @@ public class XMPanalyzePragma
     info.setReflectArrays(reflectArrays, widthList);
     info.setAsyncId(asyncOpt);
 
+    info.setAcc(accOpt);
+    if (info.isAcc() && !XmOption.isXcalableACC()){
+      XMP.errorAt(pb, "Enable XcalableACC to use the acc clause");
+    }
   }
 
   void analyzeBarrier(Xobject barrierDecl, 
@@ -618,6 +791,7 @@ public class XMPanalyzePragma
     Xobject reductionSpec = reductionDecl.getArg(0);
     Xobject reductionOnRef = reductionDecl.getArg(1);
     Xobject asyncOpt = reductionDecl.getArg(2);
+    Xobject accOpt = reductionDecl.getArg(3);
 
     // if(reductionOpt != null){
     //   XMP.fatal("redution opt is not supported yet, sorry!");
@@ -632,6 +806,11 @@ public class XMPanalyzePragma
     }
 
     info.setAsyncId(asyncOpt);
+
+    info.setAcc(accOpt);
+    if (info.isAcc() && !XmOption.isXcalableACC()){
+      XMP.errorAt(pb, "Enable XcalableACC to use the acc clause");
+    }
   }
 
   private void analyzeReductionSpec(XMPinfo info, Xobject reductionSpec,
@@ -677,6 +856,7 @@ public class XMPanalyzePragma
     Xobject fromRef = bcastDecl.getArg(1);
     Xobject onRef = bcastDecl.getArg(2);
     Xobject asyncOpt = bcastDecl.getArg(3);
+    Xobject accOpt = bcastDecl.getArg(4);
 
     // if(bcastOpt != null){
     //   XMP.fatal("bcast opt is not supported yet, sorry!");
@@ -704,6 +884,10 @@ public class XMPanalyzePragma
     }
 
     info.setAsyncId(asyncOpt);
+    info.setAcc(accOpt);
+    if (info.isAcc() && !XmOption.isXcalableACC()){
+      XMP.errorAt(pb, "Enable XcalableACC to use the acc clause");
+    }
   }
 
   private void analyzeWaitAsync(Xobject waitAsyncDecl, 
@@ -748,6 +932,7 @@ public class XMPanalyzePragma
 			    XMPinfo info, PragmaBlock pb) {
     Xobject gmoveOpt = gmoveDecl.getArg(0); // NORMAL | IN | OUT
     Xobject asyncOpt = gmoveDecl.getArg(1);
+    Xobject accOpt   = gmoveDecl.getArg(2);
     //Xobject Opt = gmoveDecl.getArg(2);
 
     // check body is single statement.
@@ -814,6 +999,11 @@ public class XMPanalyzePragma
     }
 
     info.setAsyncId(asyncOpt);
+
+    info.setAcc(accOpt);
+    if (info.isAcc() && !XmOption.isXcalableACC()){
+      XMP.errorAt(pb, "Enable XcalableACC to use the acc clause");
+    }
   }
 
   private boolean checkGmoveOperand(Xobject x, boolean remotely_accessed, PragmaBlock pb){
@@ -888,12 +1078,6 @@ public class XMPanalyzePragma
   }
 
   private boolean convertGmoveToArray(PragmaBlock pb, Xobject left, Xobject right){
-
-    // boolean lhs_is_scalar = isScalar(left);
-    // boolean rhs_is_scalar = isScalar(right);
-
-    // if (!lhs_is_scalar && rhs_is_scalar){
-
       pb.setPragma("ARRAY");
 
       Xobject onRef = Xcons.List();
@@ -931,26 +1115,9 @@ public class XMPanalyzePragma
       Xobject decl = Xcons.List(onRef);
       pb.setClauses(decl);
 
-    // }
-    // else { // lhs_is_scalar || !rhs_is_scalar){
-    //   return false;
-    // }
-
     return true;
-
   }
 
-  // private boolean isScalar(Xobject x){
-  //   if (x.Opcode() == Xcode.F_ARRAY_REF){
-  //     for (XobjArgs args = x.getArg(1).getArgs(); args != null;
-  // 	   args = args.nextArgs()){
-  // 	if (args.getArg().Opcode() != Xcode.F_ARRAY_INDEX) return false;
-  //     }
-  //   }
-  //   return true;
-  // }
-
-
   private Block analyzeArray(PragmaBlock pb){
 
     Xobject arrayDecl = pb.getClauses();
@@ -1047,7 +1214,7 @@ public class XMPanalyzePragma
 	int tidx = leftArray.getAlignSubscriptIndexAt(i);
 	if (tidx == -1) continue;
 
-    	var = env.declIdent(XMP.genSym("XMP_loop_i"), Xtype.intType, pb);
+    	var = env.declIdent(XMP.genSym("loop_i"), Xtype.intType, pb);
     	varList.add(var);
 	varListTemplate.set(tidx, var);
 
@@ -1056,20 +1223,24 @@ public class XMPanalyzePragma
 	  if (!left_var.Type().isFallocatable()){
 	    lb = sizeExprs[i].getArg(0);
 	  }
-	  else {
-	    lb = env.declIntrinsicIdent("lbound", Xtype.FintFunctionType).
-	      Call(Xcons.List(left_var, Xcons.IntConstant(i+1)));
+	  if (lb == null){
+	    // lb = env.declIntrinsicIdent("lbound", Xtype.FintFunctionType).
+	    //   Call(Xcons.List(left_var, Xcons.IntConstant(i+1)));
+	    lb = env.declInternIdent("xmp_lbound", Xtype.FintFunctionType).
+	      Call(Xcons.List(leftArray.getDescId().Ref(), Xcons.IntConstant(i+1)));
 	  }
 	}
 
 	ub = ((XobjList)sub).getArg(1);
 	if (ub == null){
-	  if(!left_var.Type().isFallocatable()){
+	  if (!left_var.Type().isFallocatable()){
 	    ub = sizeExprs[i].getArg(1);
 	  }
-	  else {
-	    ub = env.declIntrinsicIdent("ubound", Xtype.FintFunctionType).
-	      Call(Xcons.List(left_var, Xcons.IntConstant(i+1)));
+	  if (ub == null){
+	    // ub = env.declIntrinsicIdent("ubound", Xtype.FintFunctionType).
+	    //   Call(Xcons.List(left_var, Xcons.IntConstant(i+1)));
+	    ub = env.declInternIdent("xmp_ubound", Xtype.FintFunctionType).
+	      Call(Xcons.List(leftArray.getDescId().Ref(), Xcons.IntConstant(i+1)));
 	  }
 	}
 
@@ -1122,9 +1293,11 @@ public class XMPanalyzePragma
 	      if (!x_var.Type().isFallocatable()){
 		lb = sizeExprs1[i].getArg(0);
 	      }
-	      else {
-		lb = env.declIntrinsicIdent("lbound", Xtype.FintFunctionType).
-		  Call(Xcons.List(x_var, Xcons.IntConstant(i+1)));
+	      if (lb == null){
+		// lb = env.declIntrinsicIdent("lbound", Xtype.FintFunctionType).
+		//   Call(Xcons.List(x_var, Xcons.IntConstant(i+1)));
+		lb = env.declInternIdent("xmp_lbound", Xtype.FintFunctionType).
+		  Call(Xcons.List(array1.getDescId().Ref(), Xcons.IntConstant(i+1)));
 	      }
 	    }
 
@@ -1199,7 +1372,6 @@ public class XMPanalyzePragma
     Xobject onSubscripts = pb.getClauses().getArg(0).getArg(1);
 
     if (onSubscripts != null && !onSubscripts.isEmptyList()){
-      int k = 0;
       for (int i = 0; i < onSubscripts.Nargs(); i++){
     	Xobject sub = onSubscripts.getArg(i);
     	if (sub.Opcode() == Xcode.LIST){ // triplet
@@ -1210,8 +1382,8 @@ public class XMPanalyzePragma
 	      lb = template.getLowerAt(i);
 	    }
 	    else {
-	      Ident ret = env.declIdent(XMP.genSym("XMP_ret_"), Xtype.intType, pb);
-	      Ident tlb = env.declIdent(XMP.genSym("XMP_" + template.getName() + "_lb"), Xtype.intType, pb);
+	      Ident ret = env.declIdent(XMP.genSym("ret_"), Xtype.intType, pb);
+	      Ident tlb = env.declIdent(XMP.genSym(template.getName() + "_lb"), Xtype.intType, pb);
 	      
 	      Ident f = env.declInternIdent("xmp_template_lbound", Xtype.FintFunctionType);
 	      Xobject args1 = Xcons.List(template.getDescId().Ref(), Xcons.IntConstant(i+1), tlb.Ref());
@@ -1247,14 +1419,16 @@ public class XMPanalyzePragma
 	    }
 	  }
 	  else st = Xcons.IntConstant(1);
-	  Xobject expr;
-	  //expr = Xcons.binaryOp(Xcode.MUL_EXPR, varList.get(k).Ref(), st);
+
 	  Ident loopVar = varListTemplate.get(i);
-	  if (loopVar == null) XMP.fatal("template-ref does not conform to the array on lhs.");
-	  expr = Xcons.binaryOp(Xcode.MUL_EXPR, loopVar.Ref(), st);
-	  expr = Xcons.binaryOp(Xcode.PLUS_EXPR, expr, lb);
-    	  subscriptList.add(expr);
-	  k++;
+	  if (loopVar != null){
+	    Xobject expr = Xcons.binaryOp(Xcode.MUL_EXPR, loopVar.Ref(), st);
+	    expr = Xcons.binaryOp(Xcode.PLUS_EXPR, expr, lb);
+	    subscriptList.add(expr);
+	  }
+	  else {
+	    subscriptList.add(null);
+	  }
     	}
 	else { // scalar
     	  subscriptList.add(sub);
@@ -1269,8 +1443,8 @@ public class XMPanalyzePragma
 	  lb = template.getLowerAt(i);
 	}
 	else {
-	  Ident ret = env.declIdent(XMP.genSym("XMP_ret_"), Xtype.intType, pb);
-	  Ident tlb = env.declIdent(XMP.genSym("XMP_" + template.getName() + "_lb"), Xtype.intType, pb);
+	  Ident ret = env.declIdent(XMP.genSym("ret_"), Xtype.intType, pb);
+	  Ident tlb = env.declIdent(XMP.genSym(template.getName() + "_lb"), Xtype.intType, pb);
 	      
 	  Ident f = env.declInternIdent("xmp_template_lbound", Xtype.FintFunctionType);
 	  Xobject args1 = Xcons.List(template.getDescId().Ref(), Xcons.IntConstant(i+1), tlb.Ref());
@@ -1279,17 +1453,21 @@ public class XMPanalyzePragma
 	  lb = tlb.Ref();
 	}
 
-	//Xobject expr = Xcons.binaryOp(Xcode.PLUS_EXPR, varList.get(i).Ref(), lb);
 	Ident loopVar = varListTemplate.get(i);
-	if (loopVar == null) XMP.fatal("template-ref does not conform to the array on lhs.");
-	Xobject expr = Xcons.binaryOp(Xcode.PLUS_EXPR, loopVar.Ref(), lb);
-    	subscriptList.add(expr);
+	if (loopVar != null){
+	  Xobject expr = Xcons.binaryOp(Xcode.PLUS_EXPR, loopVar.Ref(), lb);
+	  subscriptList.add(expr);
+	}
+	else {
+	  subscriptList.add(null);
+	}
       }
     }
 
     onRef.add(subscriptList);
     args.add(onRef);
 
+    args.add(null);
     args.add(null);
 
     return Bcons.PRAGMA(Xcode.XMP_PRAGMA, "LOOP", args, loop);
