@@ -1,9 +1,3 @@
-/*
- * $TSUKUBA_Release: $
- * $TSUKUBA_Copyright:
- *  $
- */
-
 package exc.xmpF;
 
 import exc.block.*;
@@ -18,6 +12,7 @@ public class XMParray {
   private final static String XMP_ARRAY_PROP = "XMP_ARRAY_PROP";
 
   private Ident	arrayId; // global/original ident
+  private Block arrayIdBlock; // Block ident found
   private String name;   // original name
   private Xtype	type;    // original type
   private Xtype elementType;
@@ -61,6 +56,7 @@ public class XMParray {
   public String toString(){
     String s = "{Array("+name+", id="+arrayId+"):";
     s += dims;
+    s += " localId:" + localId;
     return s+"}";
   }
 
@@ -207,11 +203,12 @@ public class XMParray {
     // get array information
     name = a.getString();
 
-    arrayId = env.findVarIdent(name,pb);  // find ident, in this block(func)
+    arrayId = env.findVarIdent(name, pb); // find ident, in this block(func)
     if (arrayId == null) {
       XMP.errorAt(pb,"array '" + name + "' is not declared");
       return;
     }
+    arrayIdBlock = env.findVarIdentBlock(name, pb);
 
     type  = arrayId.Type();
     if (type.getKind() != Xtype.F_ARRAY) {
@@ -249,10 +246,10 @@ public class XMParray {
       XMP.errorAt(pb,"template '" + templateName + "' is not declared");
     }
 
-    if (!template.isFixed() && !type.isFallocatable()) {
-      XMP.errorAt(pb, "non-allocatable array cannot be aligned with an unfixed template");
-      //XMP.errorAt(pb,"template '" + templateName + "' is not fixed");
-    }
+    // if (!template.isFixed() && !type.isFallocatable()) {
+    //   XMP.errorAt(pb, "non-allocatable array cannot be aligned with an unfixed template");
+    //   //XMP.errorAt(pb,"template '" + templateName + "' is not fixed");
+    // }
 
     if (!template.isDistributed() && !type.isFallocatable()) {
       XMP.errorAt(pb,"template '" + templateName + "' is not distributed");
@@ -270,80 +267,9 @@ public class XMParray {
 
     // declare array address pointer, array descriptor
     String desc_id_name = XMP.DESC_PREFIX_ + name;
-    descId = env.declObjectId(desc_id_name, pb);
+    descId = env.declObjectId(desc_id_name, arrayIdBlock);
     elementType = type.getRef();
     
-    Xtype localType = null;
-    Xobject sizeExprs[];
-    switch(sclass){
-    case FPARAM: {
-
-      if (type.isFallocatable()){
-	XMP.errorAt(pb, "allocatable dummy arrays cannot be distributed.");
-      }
-
-      Xtype ftype = env.getCurrentDef().getDef().getNameObj().Type();
-
-      if (ftype.isFunction() && !ftype.isFsubroutine() && type.isFassumedShape()){
-	XMP.fatal("assumed-shape distributed arrays in functions are not supported.");
-      }
-
-      if (ftype.isFsubroutine() && type.isFassumedShape()){
-	Xobject id_list = env.getCurrentDef().getBlock().getBody().getIdentList();
-	int k = 0;
-	for (Xobject i: (XobjList) id_list){
-	  Ident id = (Ident)i;
-	  if (id.Type().isFassumedShape()){
-	    if (id.getSym().equals(name)){
-	      NthAssumedShape = k;
-	      break;
-	    }
-	    k++;
-	  }
-	}
-	if (NthAssumedShape == -1) XMP.fatal("non-dummy argument cannot have a deferred shape.");
-	if (NthAssumedShape >= XMP.MAX_ASSUMED_SHAPE){
-	  XMP.fatal("too many assumed-shape arguments (MAX = " + XMP.MAX_ASSUMED_SHAPE + ").");
-	}
-      }
-
-      if (env.getCurrentDef().getDef().getParent() != null ||
-	  (NthAssumedShape >= 0 && NthAssumedShape < XMP.MAX_ASSUMED_SHAPE)){ // to assumed-shape
-	sizeExprs = new Xobject[arrayDim];
-	for (int i = 0; i < arrayDim; i++)
-	  sizeExprs[i] = Xcons.FindexRangeOfAssumedShape(Xcons.IntConstant(0));
-	localType = Xtype.Farray(elementType, sizeExprs);
-	localType.setTypeQualFlags(type.getTypeQualFlags());
-      }
-      else { // now linearize it
-	sizeExprs = new Xobject[1];
-	sizeExprs[0] = Xcons.FindexRange(Xcons.IntConstant(0),
-					 Xcons.IntConstant(1));
-	localType = Xtype.Farray(elementType, sizeExprs);
-	setLinearized(true);
-      }
-      break;
-
-    }
-    case FLOCAL:
-    case FSAVE:
-      sizeExprs = new Xobject[arrayDim];
-      for(int i = 0; i < arrayDim; i++)
-	sizeExprs[i] = Xcons.FindexRangeOfAssumedShape();
-      localType = Xtype.Farray(elementType,sizeExprs);
-      localType.setTypeQualFlags(type.getTypeQualFlags());
-      localType.setIsFallocatable(true);
-      break;
-    default:
-      XMP.fatal("XMP_array: unknown sclass");
-    }
-    String localName = XMP.PREFIX_+name;
-    localId = env.declIdent(localName,localType,false,pb);
-    localId.setStorageClass(arrayId.getStorageClass());
-    localId.setValue(Xcons.Symbol(Xcode.VAR,localType,localName));
-    
-    setArray(arrayId,this);
-
     Vector<XMPdimInfo> src_dims = XMPdimInfo.parseSubscripts(alignSourceList);
     Vector<XMPdimInfo> tmpl_dims = XMPdimInfo.parseSubscripts(alignScriptList);
 
@@ -439,13 +365,90 @@ public class XMParray {
     for(XMPdimInfo info: dims){
       // allocate variables for size and offset, use fixed name
       info.setArrayInfoVar(env.declIdent(desc_id_name+"_size_"+dim_i,
-					 Xtype.FintType),
+					 Xtype.FintType,arrayIdBlock),
 			   env.declIdent(desc_id_name+"_off_"+dim_i,
-					 Xtype.FintType),
+					 Xtype.FintType,arrayIdBlock),
 			   env.declIdent(desc_id_name+"_blkoff_"+dim_i,
-					 Xtype.FintType));
+					 Xtype.FintType,arrayIdBlock));
       dim_i++;
     }
+
+    Xtype localType = null;
+    Xobject sizeExprs[];
+    switch(sclass){
+    case FPARAM: {
+
+      if (type.isFallocatable()){
+	XMP.errorAt(pb, "allocatable dummy arrays cannot be distributed.");
+      }
+
+      Xtype ftype = env.getCurrentDef().getDef().getNameObj().Type();
+
+      if (ftype.isFunction() && !ftype.isFsubroutine() && type.isFassumedShape()){
+	XMP.fatal("assumed-shape distributed arrays in functions are not supported.");
+      }
+
+      if (ftype.isFsubroutine() && type.isFassumedShape()){
+	Xobject id_list = env.getCurrentDef().getBlock().getBody().getIdentList();
+	int k = 0;
+	for (Xobject i: (XobjList) id_list){
+	  Ident id = (Ident)i;
+	  if (id.Type().isFassumedShape()){
+	    if (id.getSym().equals(name)){
+	      NthAssumedShape = k;
+	      break;
+	    }
+	    k++;
+	  }
+	}
+	if (NthAssumedShape == -1) XMP.fatal("non-dummy argument cannot have a deferred shape.");
+	if (NthAssumedShape >= XMP.MAX_ASSUMED_SHAPE){
+	  XMP.fatal("too many assumed-shape arguments (MAX = " + XMP.MAX_ASSUMED_SHAPE + ").");
+	}
+      }
+
+      if (env.getCurrentDef().getDef().getParent() != null ||
+	  (NthAssumedShape >= 0 && NthAssumedShape < XMP.MAX_ASSUMED_SHAPE)){ // to assumed-shape
+	sizeExprs = new Xobject[arrayDim];
+	for (int i = 0; i < arrayDim; i++){
+	  if (isDistributed(i))
+	    sizeExprs[i] = Xcons.FindexRangeOfAssumedShape(Xcons.IntConstant(0));
+	  else
+	    sizeExprs[i] = Xcons.FindexRangeOfAssumedShape(type.getFarraySizeExpr()[i].getArg(0));
+	}
+	localType = Xtype.Farray(elementType, sizeExprs);
+	localType.setTypeQualFlags(type.getTypeQualFlags());
+      }
+      else { // now linearize it
+	sizeExprs = new Xobject[1];
+	sizeExprs[0] = Xcons.FindexRange(Xcons.IntConstant(0),
+					 Xcons.IntConstant(1));
+	localType = Xtype.Farray(elementType, sizeExprs);
+	setLinearized(true);
+      }
+      break;
+
+    }
+    case FLOCAL:
+    case FSAVE:
+      sizeExprs = new Xobject[arrayDim];
+      for(int i = 0; i < arrayDim; i++)
+	sizeExprs[i] = Xcons.FindexRangeOfAssumedShape();
+      localType = Xtype.Farray(elementType,sizeExprs);
+      localType.setTypeQualFlags(type.getTypeQualFlags());
+      localType.setIsFallocatable(true);
+      break;
+    default:
+      XMP.fatal("XMP_array: unknown sclass");
+    }
+    
+    String localName = XMP.PREFIX_+name;
+    localId = env.declIdent(localName,localType,false,arrayIdBlock);
+    localId.setStorageClass(arrayId.getStorageClass());
+    localId.setValue(Xcons.Symbol(Xcode.VAR,localType,localName));
+    
+    setArray(arrayId,this);
+
   }
 
   public static void analyzeShadow(Xobject a, Xobject shadow_w_list,
@@ -551,8 +554,7 @@ public class XMParray {
    *  ! _xmpf_array_set_local_array(a_desc,a_local)
    */
 
-  public void buildConstructor(BlockList body, XMPenv env){
-
+  public void buildConstructor(BlockList body, XMPenv env, Block block){
     if (is_saveDesc && type.isFallocatable())
       XMP.fatal("an allocatable array cannot have the save_desc attribute.");
 
@@ -598,13 +600,13 @@ public class XMParray {
     Ident f;
     Xobject args;
     
-    f = env.declInternIdent(XMP.array_alloc_f,Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.array_alloc_f,Xtype.FsubroutineType, block);
     args = Xcons.List(descId.Ref(),Xcons.IntConstant(dims.size()),
 		      XMP.typeIntConstant(elementType),
 		      template.getDescId().Ref());
     b.add(f.callSubroutine(args));
 
-    f = env.declInternIdent(XMP.init_allocated_f, Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.init_allocated_f, Xtype.FsubroutineType, block);
     args = Xcons.List(descId.Ref());
     b.add(f.callSubroutine(args));
 
@@ -612,10 +614,10 @@ public class XMParray {
 
     Ident sizeArray = null;
     if (type.isFassumedShape()){
-      sizeArray = env.declOrGetSizeArray(b.getHead());
+      sizeArray = env.declOrGetSizeArray(block);
     }
 
-    f = env.declInternIdent(XMP.array_align_info_f,Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.array_align_info_f,Xtype.FsubroutineType, block);
     for(int i = 0; i < dims.size(); i++){
       XMPdimInfo info = dims.elementAt(i);
 
@@ -647,7 +649,7 @@ public class XMParray {
     }
 
     if(hasShadow()){
-      f = env.declInternIdent(XMP.array_init_shadow_f,Xtype.FsubroutineType);
+      f = env.declInternIdent(XMP.array_init_shadow_f,Xtype.FsubroutineType, block);
       for(int i = 0; i < dims.size(); i++){
 	if(hasShadow(i)){
 	  int left = getShadowLeft(i);
@@ -662,7 +664,7 @@ public class XMParray {
       }
     }
 
-    f = env.declInternIdent(XMP.array_init_f,Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.array_init_f,Xtype.FsubroutineType, block);
     b.add(f.callSubroutine(Xcons.List(descId.Ref())));
 
     Xobject allocate_statement = null;
@@ -672,7 +674,7 @@ public class XMParray {
       for(int i = 0; i < dims.size(); i++){
 	XMPdimInfo info = dims.elementAt(i);
 	f = env.declInternIdent(XMP.array_get_local_size_f,
-			      Xtype.FsubroutineType);
+			      Xtype.FsubroutineType, block);
 	b.add(f.callSubroutine(Xcons.List(descId.Ref(),
 					  Xcons.IntConstant(i),
 					  info.getArraySizeVar().Ref(),
@@ -696,7 +698,7 @@ public class XMParray {
       for(int i = 0; i < dims.size(); i++){
 	XMPdimInfo info = dims.elementAt(i);
 	f = env.declInternIdent(XMP.array_get_local_size_f,
-			      Xtype.FsubroutineType);
+			      Xtype.FsubroutineType, block);
 	b.add(f.callSubroutine(Xcons.List(descId.Ref(),
 					  Xcons.IntConstant(i),
 					  info.getArraySizeVar().Ref(),
@@ -739,7 +741,7 @@ public class XMParray {
     }
     
     // set
-    f = env.declInternIdent(XMP.array_set_local_array_f,Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.array_set_local_array_f,Xtype.FsubroutineType, block);
     Xobject isCoarray = (sclass == StorageClass.FSAVE) ? Xcons.IntConstant(1) : Xcons.IntConstant(0);
     body.add(f.callSubroutine(Xcons.List(descId.Ref(), localId.Ref(), isCoarray)));
 
@@ -756,13 +758,15 @@ public class XMParray {
     Ident f;
     Xobject args;
 
+    Xobject orig_lower[] = new Xobject[dims.size()];
+
     // Following codes come from XMParray.buildConstructor
 
-    f = env.declInternIdent(XMP.init_allocated_f, Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.init_allocated_f, Xtype.FsubroutineType, block);
     args = Xcons.List(descId.Ref());
     st.insert(f.callSubroutine(args));
 
-    f = env.declInternIdent(XMP.array_align_info_f,Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.array_align_info_f,Xtype.FsubroutineType, block);
     for(int i = 0; i < dims.size(); i++){
 
       XobjList bound = (XobjList)boundList.getArg(i);
@@ -777,6 +781,8 @@ public class XMParray {
 	upper = bound.getArg(1);
       }
 
+      orig_lower[i] = lower;
+      
       XMPdimInfo info = dims.elementAt(i);
       if(info.isAlignAny()){
 	args = Xcons.List(descId.Ref(),Xcons.IntConstant(i),
@@ -795,7 +801,7 @@ public class XMParray {
     }
 
     if(hasShadow()){
-      f = env.declInternIdent(XMP.array_init_shadow_f,Xtype.FsubroutineType);
+      f = env.declInternIdent(XMP.array_init_shadow_f,Xtype.FsubroutineType, block);
       for(int i = 0; i < dims.size(); i++){
 	if(hasShadow(i)){
 	  int left = getShadowLeft(i);
@@ -810,7 +816,7 @@ public class XMParray {
       }
     }
 
-    f = env.declInternIdent(XMP.array_init_f,Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.array_init_f,Xtype.FsubroutineType, block);
     st.insert(f.callSubroutine(Xcons.List(descId.Ref())));
 
     if(isLinearized()){
@@ -819,7 +825,7 @@ public class XMParray {
       for(int i = 0; i < dims.size(); i++){
 	XMPdimInfo info = dims.elementAt(i);
 	f = env.declInternIdent(XMP.array_get_local_size_f,
-			      Xtype.FsubroutineType);
+			      Xtype.FsubroutineType, block);
 	st.insert(f.callSubroutine(Xcons.List(descId.Ref(),
 					      Xcons.IntConstant(i),
 					      info.getArraySizeVar().Ref(),
@@ -843,7 +849,7 @@ public class XMParray {
       for(int i = 0; i < dims.size(); i++){
 	XMPdimInfo info = dims.elementAt(i);
 	f = env.declInternIdent(XMP.array_get_local_size_f,
-			      Xtype.FsubroutineType);
+			      Xtype.FsubroutineType, block);
 	st.insert(f.callSubroutine(Xcons.List(descId.Ref(),
 					      Xcons.IntConstant(i),
 					      info.getArraySizeVar().Ref(),
@@ -858,7 +864,7 @@ public class XMParray {
 	}
 	else {
 	    // not distributed
-	    alloc_args.add(Xcons.FindexRange(info.getLower(),
+	    alloc_args.add(Xcons.FindexRange(orig_lower[i],
 					     info.getArraySizeVar().Ref()));
 	}
       }
@@ -869,7 +875,7 @@ public class XMParray {
     }
       
     // set
-    f = env.declInternIdent(XMP.array_set_local_array_f,Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.array_set_local_array_f,Xtype.FsubroutineType, block);
     Xobject isCoarray = (sclass == StorageClass.FSAVE) ? Xcons.IntConstant(1) : Xcons.IntConstant(0);
     st.add(f.callSubroutine(Xcons.List(descId.Ref(), localId.Ref(), isCoarray)));
 
@@ -883,7 +889,7 @@ public class XMParray {
     Ident f;
 
     // deallocate desc
-    f = env.declInternIdent(XMP.array_deallocate_f, Xtype.FsubroutineType);
+    f = env.declInternIdent(XMP.array_deallocate_f, Xtype.FsubroutineType, block);
     st.insert(f.callSubroutine(Xcons.List(descId.Ref())));
 
     dealloc.setArg(0, localId.Ref());
@@ -897,12 +903,12 @@ public class XMParray {
     // not yet
   }
 
-  public void buildDestructor(BlockList body, XMPenv env){
+  public void buildDestructor(BlockList body, XMPenv env, Block block){
     if (!is_saveDesc){
       Ident f;
       Xobject args;
     
-      f = env.declInternIdent(XMP.array_dealloc_f,Xtype.FsubroutineType);
+      f = env.declInternIdent(XMP.array_dealloc_f,Xtype.FsubroutineType, block);
       args = Xcons.List(descId.Ref());
       body.add(f.callSubroutine(args));
     }
@@ -918,7 +924,7 @@ public class XMParray {
       Xobject offset = info.getAlignSubscriptOffset();
       Xobject alb = info.getLower();
       Xobject tlb = this.getAlignTemplate().getLowerAt(info.getAlignSubscriptIndex());
-      if ((offset != null && !offset.isZeroConstant()) || !alb.equals(tlb))
+      if ((offset != null && !offset.isZeroConstant()) || !alb.equals(tlb) || !getAlignTemplate().isFixed())
 	  return info.getArrayOffsetVar().Ref();
       else
 	  return null;
