@@ -1,13 +1,8 @@
-/* 
- * $TSUKUBA_Release: Omni XMP Compiler $
- * $TSUKUBA_Copyright:
- *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
- *  $
- */
 package exc.xmpF;
 
 import exc.object.*;
 import exc.block.*;
+import exc.xcalablemp.*;
 import java.util.*;
 import static xcodeml.util.XmLog.fatal;
 
@@ -40,6 +35,7 @@ public class XMPrewriteExpr
     // rewrite allocate, deallocate, and stop statements
     BasicBlockIterator iter3 = new BasicBlockIterator(fb);
     for (iter3.init(); !iter3.end(); iter3.next()){
+      Block b = iter3.getBasicBlock().getParent();
       StatementIterator iter4 = iter3.getBasicBlock().statements();
       while (iter4.hasNext()){
 	Statement st = iter4.next();
@@ -59,7 +55,7 @@ public class XMPrewriteExpr
 		     obj.Opcode() == Xcode.F_VAR_REF)
 		  obj = obj.getArg(0).getArg(0);
 
-	      Ident id = env.findVarIdent(obj.getName(), fb);
+	      Ident id = env.findVarIdent(obj.getName(), b);
 	      if (id == null) break;
 	      XMParray array = XMParray.getArray(id);
 	      if (array == null) break;
@@ -79,7 +75,7 @@ public class XMPrewriteExpr
 		     obj.Opcode() == Xcode.F_ARRAY_REF ||
 		     obj.Opcode() == Xcode.F_VAR_REF)
 		obj = obj.getArg(0).getArg(0);
-	      Ident id = env.findVarIdent(obj.getName(), fb);
+	      Ident id = env.findVarIdent(obj.getName(), b);
 	      if (id == null) break;
 	      XMParray array = XMParray.getArray(id);
 	      if (array == null) break;
@@ -122,46 +118,61 @@ public class XMPrewriteExpr
       }
     }
 
+    // rewrite ACC pragma
+    for (iter2.init(); !iter2.end(); iter2.next()){
+      Block block = iter2.getBlock();
+      if (block.Opcode() == Xcode.ACC_PRAGMA){
+        rewriteAccClauses((PragmaBlock)block, fb);
+      }
+    }
+
     // rewrite id_list, decles, parameters
     rewriteDecls(fb);
   }
 
   private void rewriteDecls(FunctionBlock funcBlock){
-    Xobject decl_list = funcBlock.getBody().getDecls();
-    Xobject id_list = funcBlock.getBody().getIdentList();
-    Xtype f_type = funcBlock.getNameObj().Type();
-    Xobject f_params = null;
-    if(f_type != null && f_type.isFunction())
-      f_params = f_type.getFuncParam();
+    BlockIterator biter = new topdownBlockIterator(funcBlock);
+    for (biter.init(); !biter.end(); biter.next()){
+      if (biter.getBlock().Opcode() == Xcode.F_BLOCK_STATEMENT ||
+          biter.getBlock().Opcode() == Xcode.FUNCTION_DEFINITION){
+        Xobject decl_list = biter.getBlock().getBody().getDecls();
+        Xobject id_list = biter.getBlock().getBody().getIdentList();
+        Xtype f_type = biter.getBlock().Opcode() == Xcode.FUNCTION_DEFINITION ?
+                         funcBlock.getNameObj().Type() : null;
+        Xobject f_params = null;
+        if(f_type != null && f_type.isFunction())
+          f_params = f_type.getFuncParam();
     
-    for(Xobject i: (XobjList) id_list){
-      Ident id = (Ident)i;
-      XMParray array = XMParray.getArray(id);
-      if(array == null) continue;
+        for(Xobject i: (XobjList) id_list){
+          Ident id = (Ident)i;
+          XMParray array = XMParray.getArray(id);
+          if(array == null) continue;
       
-      // write id
-      String a_name = id.getName();
-      String xmp_name = array.getLocalName();
-      Xtype xmp_type = array.getLocalType();
+          // write id
+          String a_name = id.getName();
+          String xmp_name = array.getLocalName();
+          Xtype xmp_type = array.getLocalType();
 
-      for(Xobject decl: (XobjList) decl_list){
-	if(decl.Opcode() != Xcode.VAR_DECL) continue;
-	Xobject decl_id = decl.getArg(0);
-	if(decl_id.Opcode() == Xcode.IDENT && 
-	   decl_id.getName().equals(a_name)){
-	  decl.setArg(0,Xcons.Symbol(Xcode.IDENT,xmp_type,xmp_name));
-	  break;
-	}
-      }
-      if(f_params != null && id.getStorageClass() == StorageClass.FPARAM){
-	// rewrite parameter
-	for(Xobject param: (XobjList)f_params){
-	  if(param.Opcode() == Xcode.IDENT &&
-	     param.getName().equals(a_name)){
-	    param.setName(xmp_name);
-	    param.setType(xmp_type);
-	  }
-	}
+          for(Xobject decl: (XobjList) decl_list){
+            if(decl.Opcode() != Xcode.VAR_DECL) continue;
+            Xobject decl_id = decl.getArg(0);
+            if(decl_id.Opcode() == Xcode.IDENT && 
+               decl_id.getName().equals(a_name)){
+              decl.setArg(0,Xcons.Symbol(Xcode.IDENT,xmp_type,xmp_name));
+              break;
+            }
+          }
+          if(f_params != null && id.getStorageClass() == StorageClass.FPARAM){
+            // rewrite parameter
+            for(Xobject param: (XobjList)f_params){
+              if(param.Opcode() == Xcode.IDENT &&
+                 param.getName().equals(a_name)){
+                param.setName(xmp_name);
+                param.setType(xmp_type);
+              }
+            }
+          }
+        }
       }
     }
   }
@@ -181,16 +192,15 @@ public class XMPrewriteExpr
 	{
 	  if(x.getProp(XMP.RWprotected) != null) break;
 
-	  Ident id = env.findVarIdent(x.getName(),block);
+	  Ident id = env.findVarIdent(x.getName(),bb.getParent());
 	  if(id == null) break;
 	  XMParray array = XMParray.getArray(id);
 	  if(array == null) break;
 	  
 	  // replace with local decl
-	  Xobject var = Xcons.Symbol(Xcode.VAR,array.getLocalType(),
-				                 array.getLocalName());
-	  var.setProp(XMP.arrayProp,array);
-	  iter.setXobject(var);
+          ((XobjString)x).setName(array.getLocalName());
+          x.setType(array.getLocalType());
+          x.setProp(XMP.arrayProp,array);
 	  break;
 	}
       case F_ARRAY_REF:
@@ -237,6 +247,7 @@ public class XMPrewriteExpr
 
       case FUNCTION_CALL:
 	{
+	  if (x.getArg(0).Opcode() == Xcode.MEMBER_REF) break;
 	  String fname = x.getArg(0).getString();
 	  if (fname.equalsIgnoreCase("xmp_desc_of")){
 
@@ -404,6 +415,86 @@ public class XMPrewriteExpr
 
   }
 
+  private void rewriteAccClauses(PragmaBlock pragmaBlock, Block block) {
+    Xobject clauses = pragmaBlock.getClauses();
+
+    XobjectIterator iter = new bottomupXobjectIterator(clauses);
+    for (iter.init(); !iter.end();iter.next()){
+      Xobject x = iter.getXobject();
+      if(x == null) continue;
+
+      switch(x.Opcode()) {
+      case VAR: {
+        if (x.getProp(XMP.RWprotected) != null) break;
+
+        Ident id = env.findVarIdent(x.getName(), block);
+        if (id == null) break;
+
+        XMParray array = XMParray.getArray(id);
+
+        if (array == null) break;
+
+        // replace with local decl
+        Xobject var = Xcons.Symbol(Xcode.VAR, array.getLocalType(), array.getLocalName());
+        var.setProp(XMP.arrayProp, array);
+        iter.setXobject(var);
+      }
+      break;
+      case F_ARRAY_REF: {
+        Xobject a = x.getArg(0);
+        if(a.Opcode() != Xcode.F_VAR_REF)
+          XMP.fatal("not F_VAR_REF for F_ARRAY_REF");
+        a = a.getArg(0);
+        XMParray array = (XMParray) a.getProp(XMP.arrayProp);
+        if(array == null) break;
+
+        Xobject indexRanges = x.getArg(1);
+        Fshape arrayShape = new Fshape((FarrayType)array.getType(), block);
+
+        int dim = array.getDim();
+        Xobject subscripts[] = new Xobject[dim];
+        for(int i = 0; i < dim; i++) subscripts[i] = indexRanges.getArg(i);
+        Fshape refShape = new Fshape(new FindexRange(subscripts, block));
+        FarrayType farrayType = (FarrayType)array.getType();
+        Xobject[] arraySizes = farrayType.getFarraySizeExpr();
+
+        for(int i = 0; i < dim; i++){
+          Xobject refIndexRange = indexRanges.getArg(i);
+          Xobject refIndexRangeAssumedShape = refIndexRange.getArgOrNull(3);
+
+          if((refIndexRangeAssumedShape != null) && (refIndexRangeAssumedShape.getInt() != 0)) continue; //refIndexRange is assumed shape
+
+          Xobject arraySizeAssumedShape = arraySizes[i].getArgOrNull(3);
+          if((arraySizeAssumedShape != null) && (arraySizeAssumedShape.getInt() != 0)){
+            XMP.fatal("subarray shape must be assumed shape for distributed assumed shape array in OpenACC pragma");
+          }
+
+          Xobject refLbound = refShape.lbound(i);
+          Xobject refUbound = refShape.ubound(i);
+          if(! arrayShape.lbound(i).equals(refLbound)
+                  || ! arrayShape.ubound(i).equals(refUbound)){
+            XMP.fatal("subarray shape must be same to the distributed array shape in OpenACC pragma");
+          }
+
+          //set is assumed shape
+          refIndexRange.setArg(0, null);
+          refIndexRange.setArg(1, null);
+          refIndexRange.setArg(3, Xcons.IntConstant(1)); //set to assumed shape
+        }
+      } break;
+      case LIST: {
+        if (x.left() == null || x.left().Opcode() != Xcode.STRING) continue;
+
+        String clauseName = x.left().getString();
+        if(clauseName.equals("PRIVATE")){
+          //need to rename induction variable
+        }
+      } break;
+      }
+
+    }
+  }
+
   Xobject localIndexOffset;
 
   Xobject convertLocalIndex(Xobject v, int dim_i, XMParray a, 
@@ -689,8 +780,11 @@ public class XMPrewriteExpr
 
     Ident sizeArray = env.declOrGetSizeArray(fb);
 
-    String fname = x.getArg(0).getString();
+    String fname = x.getArg(0).Opcode() != Xcode.MEMBER_REF ?
+                   x.getArg(0).getString() : null;
     Xtype ftype = x.getArg(0).Type();
+    while((ftype != null) && (ftype.copied != null)/*ftype.isFprocedure()*/)
+      ftype = ftype.copied;
     XobjList arg_list = (XobjList)x.getArg(1);
 
     //
@@ -698,12 +792,12 @@ public class XMPrewriteExpr
     //
 
     XobjList param_list = null;
-    if (ftype != null){
+    if ((ftype != null) && ftype instanceof FunctionType/*not VOID type*/){
       // internal or module procedures
       param_list = (XobjList)ftype.getFuncParam();
     }
 
-    if (param_list == null){
+    if (param_list == null && fname != null){
 	    
       XobjList decl_list = (XobjList)fb.getBody().getDecls();
 
@@ -712,7 +806,8 @@ public class XMPrewriteExpr
 	if (decl.Opcode() == Xcode.F_INTERFACE_DECL){
 	  XobjList func_list = (XobjList)decl.getArg(3);
 	  for (Xobject func: func_list){
-	    if (func.getArg(0).getString().equals(fname)){
+	    if (func.Opcode() == Xcode.FUNCTION_DECL &&
+		func.getArg(0).getString().equals(fname)){
 	      ftype = func.getArg(0).Type();
 	      param_list = (XobjList)ftype.getFuncParam();
 	      break DECLLOOP;
