@@ -3,6 +3,7 @@ package exc.xmpF;
 import exc.object.*;
 import exc.util.MachineDep;
 import exc.block.*;
+import xcodeml.util.XmOption;
 
 import java.io.File;
 import java.util.*;
@@ -30,36 +31,48 @@ public class XMPtransPragma
     // scan by bottom-up iterator
     BlockIterator i = new bottomupBlockIterator(fblock.getBody().getHead());
     for(i.init(); !i.end(); i.next()) {
-      b = i.getBlock();
-      if(b.Opcode() == Xcode.XMP_PRAGMA) {
-	b = transPragma((PragmaBlock)b);
-	if(b != null) i.setBlock(b);
+      Block bc = i.getBlock();
+      if(bc.Opcode() == Xcode.XMP_PRAGMA) {
+	bc = transPragma((PragmaBlock)bc);
+	if(bc != null) i.setBlock(bc);
       }
     }
     
-    Xobject f_name = fblock.getNameObj();
+    run_block(def, env, null);
+  }
+
+  public Vector<Block> run_block(FuncDefBlock def, XMPenv env, Block b) {
+    FunctionBlock fblock = def.getBlock();
     BlockList prolog = Bcons.emptyBody();
     BlockList epilog = Bcons.emptyBody();
-    buildXMPobjectBlock(prolog, epilog);
+    buildXMPobjectBlock(prolog, epilog, b);
 
-    if (def.getDef().getName().equals(XMPtranslate.XMPmainFunc)){
-      Ident f = env.declIdent("xmp_barrier", Xtype.FsubroutineType);
-      epilog.add(f.callSubroutine(Xcons.List()));
-    }
-    
-    // move OMP_THREADPRIVATE to the head of prolog
-    // NOTE: Hereafter any addition of statements to prolog must be done
-    //       at its tail.
-    BlockIterator j = new bottomupBlockIterator(fblock.getBody().getHead());
-    for(j.init(); !j.end(); j.next()){
-      b = j.getBlock();
-      if (b.Opcode() == Xcode.OMP_PRAGMA &&
-	  ((PragmaBlock)b).getPragma().equals("THREADPRIVATE")){
-	b.remove();
-	prolog.insert(b);
+    if (b == null){
+      b = fblock.getBody().getHead();
+      if (def.getDef().getName().equals(XMPtranslate.XMPmainFunc)){
+        Ident f = env.declIdent("xmp_barrier", Xtype.FsubroutineType);
+        epilog.add(f.callSubroutine(Xcons.List()));
       }
     }
 
+    BlockIterator i = new topdownBlockIterator(b);
+    for(i.init(); !i.end(); i.next()){
+      Block bc = i.getBlock();
+      // move OMP_THREADPRIVATE to the head of prolog
+      // NOTE: Hereafter any addition of statements to prolog must be done
+      //       at its tail.
+      if (bc.Opcode() == Xcode.OMP_PRAGMA &&
+	  ((PragmaBlock)bc).getPragma().equals("THREADPRIVATE")){
+	bc.remove();
+	prolog.insert(bc);
+      } else if (bc.Opcode() == Xcode.F_BLOCK_STATEMENT){
+        if (b==bc)
+          i.setAside(bc);
+        else
+          i.setAside(run_block(def, env, bc));
+      }
+    }
+    
     if(env.currentDefIsModule()){
 
       Xtype save_logical = Xtype.FlogicalType.copy();
@@ -96,7 +109,7 @@ public class XMPtransPragma
       	env.declIdent("xmpf_traverse_module_"+env.currentDefName(),
 		      Xtype.FsubroutineType);
       XobjString modName = Xcons.Symbol(Xcode.IDENT, env.getCurrentDef().getDef().getName());
-      Xobject decls = Xcons.List(Xcons.List(Xcode.F_USE_DECL, modName));
+      Xobject decls = Xcons.List(Xcons.List(Xcode.F_USE_DECL, modName, Xcons.IntConstant(0)));
       XobjectDef prolog_def = 
       	XobjectDef.Func(prolog_f, null, decls, prolog.toXobject());
       env.getEnv().add(prolog_def);
@@ -104,7 +117,7 @@ public class XMPtransPragma
     } else {
       // fblock = (FunckBlock ident <param_env> [BlockList 
       //   (CompoundBlock <local_env> [BlockList statment ...]))
-      BlockList f_body = fblock.getBody().getHead().getBody();
+      BlockList f_body = b.getBody();
       
       // not need to call init_module__
 //       XobjectDef parent = env.getCurrentDef().getDef().getParent();
@@ -123,30 +136,40 @@ public class XMPtransPragma
       f_body.add(Bcons.COMPOUND(epilog));
     }
 
+    return i.getContainer();
   }
 
-  void buildXMPobjectBlock(BlockList prolog, BlockList epilog){
-    XMPsymbolTable table = env.getXMPsymbolTable();
-    epilog.add(Xcons.StatementLabel(XMP.epilog_label_f));
-    epilog.add(Xcons.List(Xcode.F_CONTINUE_STATEMENT));
+  void buildXMPobjectBlock(BlockList prolog, BlockList epilog, Block block){
+    XMPsymbolTable table = (block != null) ? block.getXMPsymbolTable() : env.getXMPsymbolTable();
+    if (block == null) {
+      epilog.add(Xcons.StatementLabel(XMP.epilog_label_f));
+      epilog.add(Xcons.List(Xcode.F_CONTINUE_STATEMENT));
+    }
     if(table != null){
       for(XMPobject o: table.getXMPobjects()){
-	o.buildConstructor(prolog,env);
-	o.buildDestructor(epilog,env);
+	o.buildConstructor(prolog,env,block);
+	o.buildDestructor(epilog,env,block);
       }
       for(XMParray a: table.getXMParrays()){
-	a.buildConstructor(prolog,env);
-	a.buildDestructor(epilog,env);
+	a.buildConstructor(prolog,env,block);
+	a.buildDestructor(epilog,env,block);
       }
-      XobjectDef def = env.getCurrentDef().getDef();
-      Xobject id_list = def.getDef().getArg(1);
-      for(Xobject id: (XobjList)id_list){
-        Ident ident = (Ident)id;
-        if (ident.isCoarray()) {
-          XMPcoarray coarray = new XMPcoarray(ident, env);
-          coarray.build_setMappingNodes(prolog);
+      Xobject id_list;
+      if (block == null) {
+        XobjectDef def = env.getCurrentDef().getDef();
+        id_list = def.getDef().getArg(1);
+      } else {
+        id_list = block.getBody().getLocalCoarrays();
+      }
+      if (id_list != null)
+        for(Xobject id: (XobjList)id_list) {
+          Ident ident = (Ident)id;
+          if (ident.isCoarray() && ((block == null && ident.getAlias() == null) ||
+                                    (block != null && ident.getAlias() != null)    )) {
+            XMPcoarray coarray = new XMPcoarray(ident, env);
+            coarray.build_setMappingNodes(prolog, block);
+          }
         }
-      }
     }
   }
 
@@ -172,6 +195,7 @@ public class XMPtransPragma
       return translateLoop(pb,info);
 
     case REFLECT:
+    case REDUCE_SHADOW:
       return translateReflect(pb,info);
     case BARRIER:
       return translateBarrier(pb,info);
@@ -206,6 +230,8 @@ public class XMPtransPragma
 
   private Block translateLoop(PragmaBlock pb, XMPinfo info){
     BlockList ret_body = Bcons.emptyBody();
+    int origLoopType = info.getLoopType();
+    Vector<XMPdimInfo> widthList = info.getWidthList();
     XMPobjectsRef on_ref = info.getOnRef();
 
     // generate on_ref object
@@ -238,6 +264,10 @@ public class XMPtransPragma
     BasicBlock entry_bb = entry_block.getBasicBlock();
 
     for(int k = 0; k < info.getLoopDim(); k++){
+
+      XMPtemplate t = on_ref.getTemplate();
+      int t_idx = on_ref.getLoopOnIndex(k);
+
       XMPdimInfo d_info = info.getLoopDimInfo(k);
       Ident local_loop_var = d_info.getLoopLocalVar();
 
@@ -257,9 +287,9 @@ public class XMPtransPragma
 
       // transform
       Xtype btype = local_loop_var.Type();
-      Ident lb_var = env.declIdent(XMP.genSym("XMP_loop_lb"), btype,pb);
-      Ident ub_var = env.declIdent(XMP.genSym("XMP_loop_ub"), btype,pb);
-      Ident step_var = env.declIdent(XMP.genSym("XMP_loop_step"), step_type,pb);
+      Ident lb_var = env.declIdent(XMP.genSym("loop_lb"), btype,pb);
+      Ident ub_var = env.declIdent(XMP.genSym("loop_ub"), btype,pb);
+      Ident step_var = env.declIdent(XMP.genSym("loop_step"), step_type,pb);
       
       Xobject org_loop_ind_var = for_block.getInductionVar();
 
@@ -270,18 +300,40 @@ public class XMPtransPragma
       entry_bb.add(Xcons.Set(ub_var.Ref(), for_block.getUpperBound()));
       entry_bb.add(Xcons.Set(step_var.Ref(), for_block.getStep()));
 
+      Xobject lower, upper, unboundFlag;
+      if (widthList.isEmpty()){
+	lower = Xcons.IntConstant(0);
+	upper = Xcons.IntConstant(0);
+	unboundFlag = Xcons.IntConstant(0);
+      }
+      else {
+	// Here the stride means an unbound flag.
+	XMPdimInfo w = widthList.get(t_idx);
+	lower = w.getLower();
+	upper = w.getUpper();
+	unboundFlag = w.getStride();
+      }
+
+      int loopType;
+      if (origLoopType == XMP.LOOP_MARGIN && unboundFlag.getInt() == -1){
+	loopType = XMP.LOOP_EXPAND;
+      }
+      else {
+	loopType = origLoopType;
+      }
+      
       Ident schd_f = 
 	env.declInternIdent(XMP.loop_sched_f,Xtype.FsubroutineType);
       Xobject args = Xcons.List(lb_var.Ref(), ub_var.Ref(), step_var.Ref(),
 				Xcons.IntConstant(k),
-				on_ref.getDescId().Ref());
+				on_ref.getDescId().Ref(),
+				Xcons.IntConstant(loopType),
+				lower, upper, unboundFlag);
       entry_bb.add(schd_f.callSubroutine(args));
 
       for_block.setLowerBound(lb_var.Ref());
       for_block.setUpperBound(ub_var.Ref());
 
-      XMPtemplate t = on_ref.getTemplate();
-      int t_idx = on_ref.getLoopOnIndex(k);
       if (for_block.getStep().isOneConstant() && (t.getDistMannerAt(t_idx) != XMPtemplate.CYCLIC ||
 						  t.getDistArgAt(t_idx) == null ||
 						  t.getDistArgAt(t_idx).isOneConstant())){
@@ -293,13 +345,52 @@ public class XMPtransPragma
       
       if(isVarUsed(for_block.getBody(),org_loop_ind_var)){
 	// if global variable is used in this block, convert local to global
-	Ident l2g_f = 
-	  env.declInternIdent(XMP.l2g_f,Xtype.FsubroutineType);
-	args = Xcons.List(org_loop_ind_var,
-			  local_loop_var.Ref(),
-			  Xcons.IntConstant(k),
-			  on_ref.getDescId().Ref());
-	for_block.getBody().insert(l2g_f.callSubroutine(args));
+        if(XmOption.isXcalableACC()) {
+          switch (t.getDistMannerAt(t_idx)){
+          case XMPtemplate.BLOCK:
+          case XMPtemplate.GBLOCK:
+          {
+            Ident l2g_off_var = env.declIdent(XMP.genSym("loop_l2goff"), Xtype.FintType, pb);
+            Ident l2g_f =
+                    env.declInternIdent(XMP.l2g_f, Xtype.FsubroutineType);
+            args = Xcons.List(l2g_off_var.Ref(),
+                    Xcons.IntConstant(0),
+                    Xcons.IntConstant(k),
+                    on_ref.getDescId().Ref());
+            entry_bb.add(l2g_f.callSubroutine(args));
+            for_block.getBody().insert(Xcons.Set(org_loop_ind_var, Xcons.binaryOp(Xcode.PLUS_EXPR, l2g_off_var.Ref(), local_loop_var.Ref())));
+            break;
+          }
+          default:
+            for_block.getBody().insert(Xcons.Set(org_loop_ind_var, calcLtoG(t, t_idx, local_loop_var.Ref())));
+          }
+        }else {
+          Ident l2g_f =
+                  env.declInternIdent(XMP.l2g_f, Xtype.FsubroutineType);
+
+          switch (t.getDistMannerAt(t_idx)){
+          case XMPtemplate.BLOCK:
+          case XMPtemplate.GBLOCK:
+            {
+              Ident l2g_off_var = env.declIdent(XMP.genSym("loop_l2goff"), Xtype.FintType, pb);
+              args = Xcons.List(l2g_off_var.Ref(),
+                                Xcons.IntConstant(0),
+                                Xcons.IntConstant(k),
+                                on_ref.getDescId().Ref());
+              entry_bb.add(l2g_f.callSubroutine(args));
+              for_block.getBody().insert(Xcons.Set(org_loop_ind_var, Xcons.binaryOp(Xcode.PLUS_EXPR, l2g_off_var.Ref(), local_loop_var.Ref())));
+              break;
+            }
+          default:
+            {
+              args = Xcons.List(org_loop_ind_var,
+                                local_loop_var.Ref(),
+                                Xcons.IntConstant(k),
+                                on_ref.getDescId().Ref());
+              for_block.getBody().insert(l2g_f.callSubroutine(args));
+            }
+          }
+        }
       }
     }
 
@@ -327,6 +418,95 @@ public class XMPtransPragma
     return Bcons.COMPOUND(ret_body);
   }
 
+  private Xobject calcLtoG(XMPtemplate t, int tIdx, Xobject local) {
+    if (!t.isDistributed() || t.getDistMannerAt(tIdx) == XMPtemplate.DUPLICATION) {
+      return local;
+    }
+
+    //t.getOntoNodesIndexAt(ti).getInt(); //obtains node idx which corresponds to template idx
+    int nIdx = 0;
+    for(int i = 0; i < tIdx; i++){
+      if(t.getDistMannerAt(i) != XMPtemplate.DUPLICATION){
+        nIdx++;
+      }
+    }
+
+    XMPnodes n = t.getOntoNodes();
+
+    Xobject nodeRank = n.getInfoAt(nIdx).getNodeRankVar().Ref(); /* _p */
+    Xobject nodeSize = n.getInfoAt(nIdx).getSize(); /* _P */
+    if(nodeSize == null){
+      nodeSize = n.getInfoAt(nIdx).getNodeSizeVar().Ref();
+    }
+    Xobject total = t.getSizeAt(tIdx); /* _N */
+    Xobject base = t.getLowerAt(tIdx); /* _m */
+    Xobject newExpr = null;
+
+    switch (t.getDistMannerAt(tIdx)) {
+    //case XMPtemplate.DUPLICATION: unreachable
+    case XMPtemplate.BLOCK:
+      // _XMP_M_LTOG_TEMPLATE_BLOCK(_l, _m, total, _P, _p)  (((_p) * _XMP_M_CEILi(total, _P)) + (_m) + (_l))
+      Xobject ceili = Xcons.binaryOp(Xcode.PLUS_EXPR,
+              Xcons.binaryOp(Xcode.DIV_EXPR,
+                      Xcons.binaryOp(Xcode.MINUS_EXPR,
+                              total,
+                              Xcons.IntConstant(1)),
+                      nodeSize),
+              Xcons.IntConstant(1)
+      );
+      newExpr = Xcons.binaryOp(Xcode.PLUS_EXPR,
+              Xcons.binaryOp(Xcode.PLUS_EXPR,
+                      Xcons.binaryOp(Xcode.MUL_EXPR,
+                              ceili,
+                              nodeRank),
+                      base),
+              local);
+      break;
+    case XMPtemplate.CYCLIC:
+      //case XMPtemplate.BLOCK_CYCLIC: block-cyclic is not used. it is same as cyclic
+      Xobject distArg = t.getDistArgAt(tIdx);
+      if(distArg == null) {
+        //cyclic distribution
+        // #define _XMP_M_LTOG_TEMPLATE_CYCLIC(_l, _m, _P, _p)  (((_l) * (_P)) + (_p) + (_m))
+        newExpr = Xcons.binaryOp(Xcode.PLUS_EXPR,
+                Xcons.binaryOp(Xcode.PLUS_EXPR,
+                        Xcons.binaryOp(Xcode.MUL_EXPR,
+                                local,
+                                nodeSize),
+                        nodeRank),
+                base);
+      }else {
+        //block-cyclic distribution
+        Xobject _b = distArg; //FIXME if distArg is variable and changed, the global index is not correct.
+        Ident modFuncId = env.declIntrinsicIdent("mod", Xtype.FintFunctionType);
+        //#define _XMP_M_LTOG_TEMPLATE_BLOCK_CYCLIC(_l, _b, _m, _P, _p) \
+        //((((_l) / (_b)) * (_b) * (_P)) + ((_b) * (_p)) + ((_l) % (_b)) + (_m))
+        newExpr = Xcons.binaryOp(Xcode.PLUS_EXPR,
+                Xcons.binaryOp(Xcode.PLUS_EXPR,
+                        Xcons.binaryOp(Xcode.PLUS_EXPR,
+                                Xcons.binaryOp(Xcode.MUL_EXPR,
+                                        Xcons.binaryOp(Xcode.MUL_EXPR,
+                                                Xcons.binaryOp(Xcode.DIV_EXPR,
+                                                        local,
+                                                        _b),
+                                                _b),
+                                        nodeSize),
+                                Xcons.binaryOp(Xcode.MUL_EXPR,
+                                        _b,
+                                        nodeRank)),
+                        modFuncId.Call(Xcons.List(local,_b))),
+                base);
+      }
+      break;
+    case XMPtemplate.GBLOCK:
+      XMP.fatal("gblock is not implemented");
+    default:
+      XMP.fatal("unknown distribute manner");
+    }
+
+    return newExpr;
+  }
+
   boolean isVarUsed(BlockList body,Xobject v){
     BasicBlockExprIterator iter = new BasicBlockExprIterator(body);
     for (iter.init(); !iter.end(); iter.next()) {
@@ -351,8 +531,12 @@ public class XMPtransPragma
     Block b = Bcons.emptyBlock();
     BasicBlock bb = b.getBasicBlock();
     Ident f, g, h;
+    boolean isAcc = info.isAcc();
 
-    f = env.declInternIdent(XMP.reflect_f,Xtype.FsubroutineType);
+    if (info.pragma == XMPpragma.REFLECT)
+      f = env.declInternIdent(isAcc? XMP.reflect_acc_f : XMP.reflect_f, Xtype.FsubroutineType);
+    else
+      f = env.declInternIdent(isAcc? XMP.reduce_shadow_acc_f : XMP.reduce_shadow_f, Xtype.FsubroutineType);
     
     if (info.getAsyncId() != null){
       Xobject arg = Xcons.List(info.getAsyncId());
@@ -363,7 +547,10 @@ public class XMPtransPragma
     Vector<XMParray> reflectArrays = info.getReflectArrays();
     for(XMParray a: reflectArrays){
       for (int i = 0; i < info.widthList.size(); i++){
-	  g = env.declInternIdent(XMP.set_reflect_f,Xtype.FsubroutineType);
+	  if (info.pragma == XMPpragma.REFLECT)
+	    g = env.declInternIdent(isAcc? XMP.set_reflect_acc_f : XMP.set_reflect_f, Xtype.FsubroutineType);
+	  else
+	    g = env.declInternIdent(isAcc? XMP.set_reduce_shadow_acc_f : XMP.set_reduce_shadow_f, Xtype.FsubroutineType);
 	  XMPdimInfo w = info.widthList.get(i);
 
 	  // Here the stride means the periodic flag.
@@ -378,11 +565,17 @@ public class XMPtransPragma
       }
 
       if (info.getAsyncId() != null){
-	  h = env.declInternIdent(XMP.reflect_async_f,Xtype.FsubroutineType);
+	  if (info.pragma == XMPpragma.REFLECT)
+	    h = env.declInternIdent(isAcc? XMP.reflect_async_acc_f : XMP.reflect_async_f, Xtype.FsubroutineType);
+	  else
+	    h = f; // no change for REDUCE_SHADOW
 	  bb.add(h.callSubroutine(Xcons.List(a.getDescId().Ref(), info.getAsyncId())));
       }
       else {
-	  bb.add(f.callSubroutine(Xcons.List(a.getDescId().Ref())));
+        Xobject arg = Xcons.List();
+        if(isAcc) arg.add(a.getLocalId().Ref());
+        arg.add(a.getDescId().Ref());
+        bb.add(f.callSubroutine(arg));
       }
     }
 
@@ -392,6 +585,14 @@ public class XMPtransPragma
       bb.add(g.callSubroutine(arg));
     }
 
+    if(isAcc) {
+      XobjList vars = Xcons.List();
+      for(XMParray a : reflectArrays){
+        vars.add(a.getLocalId().Ref());
+      }
+      b = buildAccHostData(vars, Bcons.blockList(b));
+    }
+
     return b;
   }
 
@@ -429,20 +630,16 @@ public class XMPtransPragma
     return Bcons.COMPOUND(ret_body);
   }
 
-  private Block translateReduction(PragmaBlock pb, XMPinfo info){
-
-    //Block b = Bcons.emptyBlock();
-    //BasicBlock bb = b.getBasicBlock();
-
+  private Block translateReduction(PragmaBlock pb, XMPinfo info)
+  {
     BlockList ret_body = Bcons.emptyBody();
-
     XMPobjectsRef on_ref = info.getOnRef();
     Xobject on_ref_arg;
 
     Ident xmp_null = env.findVarIdent("XMP_NULL", pb);
     if (xmp_null == null){
-	xmp_null = env.declObjectId("XMP_NULL", null,
-				    Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0)));
+      xmp_null = env.declObjectId("XMP_NULL", null,
+                                  Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0)));
     }
 
     if (info.getAsyncId() != null){
@@ -466,24 +663,23 @@ public class XMPtransPragma
 
     // object size
     int op = info.getReductionOp();
-    // boolean reduce_minus = false;
-    // if (op == XMP.REDUCE_MINUS){
-    //   op = XMP.REDUCE_SUM;
-    //   reduce_minus = true;
-    // }
+    boolean isAcc = info.isAcc();
+    Ident f = env.declInternIdent(isAcc? XMP.reduction_acc_f : XMP.reduction_f, Xtype.FsubroutineType);
+    Ident f2 = env.declInternIdent(isAcc? XMP.reduction_loc_acc_f : XMP.reduction_loc_f, Xtype.FsubroutineType);
 
-    Ident f = env.declInternIdent(XMP.reduction_f, Xtype.FsubroutineType);
-    Ident f2 = env.declInternIdent(XMP.reduction_loc_f, Xtype.FsubroutineType);
-
-    //for(Ident id: info.getReductionVars()){
     for (int i = 0; i < info.getReductionVars().size(); i++){
-
       Ident id = info.getReductionVars().elementAt(i);
       Vector<Ident> pos_vars = info.getReductionPosVars().elementAt(i);
 
       Xtype type = id.Type();
       Xobject size_expr = Xcons.IntConstant(1);
-      if(type.isFarray()){
+
+      if(type.isFallocatable()){
+        Ident size_func = env.declIntrinsicIdent("size", Xtype.FintFunctionType);
+        size_expr = size_func.Call(Xcons.List(id.Ref()));
+        type = type.getRef();
+      }
+      else if(type.isFarray()){
 	for(Xobject s: type.getFarraySizeExpr()){
           Xobject length = Xcons.binaryOp(Xcode.PLUS_EXPR, Xcons.IntConstant(1),
                                           Xcons.binaryOp(Xcode.MINUS_EXPR, s.getArg(1), s.getArg(0)));
@@ -498,6 +694,9 @@ public class XMPtransPragma
 	XMP.fatal("reduction for non-basic type ="+type);
       }
 
+      if(type.getFlen() != null)
+        size_expr = Xcons.binaryOp(Xcode.MUL_EXPR, size_expr, type.getFlen());
+      
       Xobject args = Xcons.List(id.Ref(),size_expr,
 				XMP.typeIntConstant(type),
 				Xcons.IntConstant(op),
@@ -531,20 +730,25 @@ public class XMPtransPragma
       ret_body.add(g.callSubroutine(Xcons.List(on_ref.getDescId())));
     }
 
+    if(isAcc){
+      XobjList vars = Xcons.List();
+      for(Ident id : info.getReductionVars()){
+        vars.add(id.Ref());
+      }
+      ret_body = Bcons.blockList(buildAccHostData(vars, ret_body));
+    }
     return Bcons.COMPOUND(ret_body);
   }
 
-  private Block translateBcast(PragmaBlock pb, XMPinfo info){
-
-    //Block b = Bcons.emptyBlock();
-    //BasicBlock bb = b.getBasicBlock();
-
+  private Block translateBcast(PragmaBlock pb, XMPinfo info)
+  {
     BlockList ret_body = Bcons.emptyBody();
+    boolean isAcc = info.isAcc();
 
     Ident xmp_null = env.findVarIdent("XMP_NULL", pb);
     if (xmp_null == null){
-	xmp_null = env.declObjectId("XMP_NULL", null,
-				    Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0)));
+      xmp_null = env.declObjectId("XMP_NULL", null,
+                                  Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0)));
     }
 
     if (info.getAsyncId() != null){
@@ -569,22 +773,19 @@ public class XMPtransPragma
     }
     else on_ref_arg = xmp_null;
 
-    Ident f = env.declInternIdent(XMP.bcast_f, Xtype.FsubroutineType);
+    Ident f = env.declInternIdent(isAcc? XMP.bcast_acc_f : XMP.bcast_f, Xtype.FsubroutineType);
 
     for(Ident id: info.getInfoVarIdents()){
       Xtype type = id.Type();
       Xobject size_expr = Xcons.IntConstant(1);
 
       if (type.isFarray()){
-
 	if (type.isFassumedSize()){
 	  XMP.fatal("assumed-size array cannot be the target of bcast.");
 	}
-
+        
 	if (!type.isFassumedShape() && !type.isFallocatable()){
 	  for (Xobject s: type.getFarraySizeExpr()){
-	    //size_expr = Xcons.binaryOp(Xcode.MUL_EXPR,size_expr,s);
-
 	    Xobject size;
 	    if (s.Opcode() == Xcode.F_INDEX_RANGE){
 	      Xobject lb = s.getArg(0);
@@ -598,13 +799,11 @@ public class XMPtransPragma
 
 	    size_expr = Xcons.binaryOp(Xcode.MUL_EXPR, size_expr, size);
 	  }
-
 	}
 	else {
 	  Ident size_func = env.declIntrinsicIdent("size", Xtype.FintFunctionType);
 	  size_expr = size_func.Call(Xcons.List(id.Ref()));
 	}
-
 	type = type.getRef();
       }
 
@@ -612,17 +811,15 @@ public class XMPtransPragma
 	XMP.fatal("bcast for non-basic type ="+type);
       }
 
-      Xobject args = Xcons.List(id.Ref(), size_expr,
-				XMP.typeIntConstant(type),
-				from_ref_arg,
-				on_ref_arg);
+      if(type.getFlen() != null)
+        size_expr = Xcons.binaryOp(Xcode.MUL_EXPR, size_expr, type.getFlen());
+
+      Xobject args = Xcons.List(id.Ref(), size_expr, XMP.typeIntConstant(type), from_ref_arg, on_ref_arg);
 
       ret_body.add(f.callSubroutine(args));
-
     }
 
     if (info.getAsyncId() != null){
-      //      Xobject arg = Xcons.List(info.getAsyncId());
       Xobject arg = Xcons.List();
       Ident g = env.declInternIdent(XMP.start_async_f, Xtype.FsubroutineType);
       ret_body.add(g.callSubroutine(arg));
@@ -638,13 +835,19 @@ public class XMPtransPragma
       ret_body.add(g.callSubroutine(Xcons.List(from_ref.getDescId())));
     }
 
+    if(isAcc) {
+      XobjList vars = Xcons.List();
+      for (Ident id : info.getInfoVarIdents()) {
+        vars.add(id.Ref());
+      }
+      ret_body = Bcons.blockList(buildAccHostData(vars, ret_body));
+    }
+
     return Bcons.COMPOUND(ret_body);
   }
 
-  private Block translateWaitAsync(PragmaBlock pb, XMPinfo info){
-    //Block b = Bcons.emptyBlock();
-    //BasicBlock bb = b.getBasicBlock();
-
+  private Block translateWaitAsync(PragmaBlock pb, XMPinfo info)
+  {
     BlockList ret_body = Bcons.emptyBody();
 
     XMPobjectsRef on_ref = info.getOnRef();
@@ -681,16 +884,16 @@ public class XMPtransPragma
     Block b = on_ref.buildConstructor(env);
     BasicBlock bb = b.getBasicBlock();
 
-    Ident taskNodesDescId = env.declObjectId(XMP.genSym("XMP_TASK_NODES"), pb);
+    Ident taskNodesDescId = env.declObjectId(XMP.genSym("TASK_NODES"), pb);
 
     Ident f;
     if (!info.isNocomm()){
-      f = env.declInternIdent(XMP.create_task_nodes_f, Xtype.FsubroutineType);
+      f = env.declInternIdent(XMP.create_task_nodes_f, Xtype.FsubroutineType, pb);
       bb.add(f.callSubroutine(Xcons.List(taskNodesDescId, on_ref.getDescId().Ref())));
     }
 
-    Ident g1 = env.declInternIdent(XMP.nodes_dealloc_f, Xtype.FsubroutineType);
-    Ident g2 = env.declInternIdent(XMP.ref_dealloc_f, Xtype.FsubroutineType);
+    Ident g1 = env.declInternIdent(XMP.nodes_dealloc_f, Xtype.FsubroutineType, pb);
+    Ident g2 = env.declInternIdent(XMP.ref_dealloc_f, Xtype.FsubroutineType, pb);
 
     if (tasksFlag){
       //parentBlock.insert(on_ref.buildConstructor(env));
@@ -706,7 +909,7 @@ public class XMPtransPragma
     Xobject cond = null;
     if (!info.isNocomm()){
       f = env.declInternIdent(XMP.test_task_on_f,
-			      Xtype.FlogicalFunctionType);
+			      Xtype.FlogicalFunctionType, pb);
       //Xobject cond = f.Call(Xcons.List(on_ref.getDescId().Ref()));
       cond = f.Call(Xcons.List(taskNodesDescId.Ref()));
     }
@@ -855,20 +1058,19 @@ public class XMPtransPragma
 			  Xcons.IntConstant(type.getNumDimensions()));
 	bb.add(f.callSubroutine(args));
 
-	Xobject lower_b = 
-	  env.declIntrinsicIdent("lbound",Xtype.FintFunctionType).
-	  Call(Xcons.List(a));
-	Xobject upper_b = 
-	  env.declIntrinsicIdent("ubound",Xtype.FintFunctionType).
-	  Call(Xcons.List(a));
+	Ident lower_f = env.declIntrinsicIdent("lbound", Xtype.FintFunctionType);
+	Ident upper_f = env.declIntrinsicIdent("ubound", Xtype.FintFunctionType);
 	
 	f = env.declInternIdent(XMP.gmove_l_dim_info_f, Xtype.FsubroutineType);
  	int idx = 0;
  	for(Xobject e: (XobjList) x.getArg(1)){
- 	  switch(e.Opcode()){
+
+	  Xobject dim = Xcons.binaryOp(Xcode.PLUS_EXPR, Xcons.IntConstant(idx), Xcons.IntConstant(1));
+	  
+	  switch(e.Opcode()){
 	  case F_ARRAY_INDEX:
-	    args = Xcons.List(descId.Ref(),Xcons.IntConstant(idx),
-			      lower_b, upper_b,
+	    args = Xcons.List(descId.Ref(), Xcons.IntConstant(idx),
+			      lower_f.Call(Xcons.List(a, dim)), upper_f.Call(Xcons.List(a, dim)),
 			      Xcons.IntConstant(GMOVE_INDEX),
 			      e.getArg(0),
 			      Xcons.IntConstant(0),Xcons.IntConstant(0));
@@ -876,7 +1078,7 @@ public class XMPtransPragma
 	  case F_INDEX_RANGE:
 	    if(e.getArg(0) == null && e.getArg(1) == null){
 	      args = Xcons.List(descId.Ref(),Xcons.IntConstant(idx),
-				lower_b,upper_b, 
+				lower_f.Call(Xcons.List(a, dim)), upper_f.Call(Xcons.List(a, dim)),
 				Xcons.IntConstant(GMOVE_ALL),
 				Xcons.IntConstant(0),
 				Xcons.IntConstant(0),Xcons.IntConstant(0));
@@ -884,7 +1086,7 @@ public class XMPtransPragma
 	      Xobject stride = e.getArg(2);
 	      if(stride == null) stride = Xcons.IntConstant(1);
 	      args = Xcons.List(descId.Ref(),Xcons.IntConstant(idx),
-				lower_b,upper_b,
+				lower_f.Call(Xcons.List(a, dim)), upper_f.Call(Xcons.List(a, dim)),
 				Xcons.IntConstant(GMOVE_RANGE),
 				e.getArg(0),e.getArg(1),stride);
 	    }
@@ -1032,4 +1234,10 @@ public class XMPtransPragma
     return b;
   }
 
+  private Block buildAccHostData(Xobject useDeviceArg, BlockList body){
+    return Bcons.PRAGMA(Xcode.ACC_PRAGMA,
+            "HOST_DATA",
+            Xcons.List(Xcons.List(Xcons.String("USE_DEVICE"), useDeviceArg)),
+            body);
+  }
 }
