@@ -13,6 +13,10 @@ struct _ACC_memory_type{
   bool is_pagelocked;
   bool is_registered;
   unsigned int ref_count;
+  bool is_pointer;
+  unsigned int num_pointers;
+  struct _ACC_memory_type **pointees;
+  ptrdiff_t *pointee_offsets;
 };
 
 #endif //_ACC_GPU_DATA_STRUCT
