@@ -63,10 +63,10 @@ extern void _XMP_bcast_acc_NODES_ENTIRE_GLOBAL(void *bcast_nodes, void *addr, in
 extern void _XMP_bcast_acc_NODES_ENTIRE_NODES(void *bcast_nodes, void *addr, int count, size_t datatype_size, void *from_nodes, ...);
 
 // xmp_coarray.c
-extern void _XMP_gasnet_not_continuous_put();
-extern void _XMP_gasnet_continuous_put();
-extern void _XMP_gasnet_not_continuous_get();
-extern void _XMP_gasnet_continuous_get();
+extern void _XMP_gasnet_not_contiguous_put();
+extern void _XMP_gasnet_contiguous_put();
+extern void _XMP_gasnet_not_contiguous_get();
+extern void _XMP_gasnet_contiguous_get();
 extern void _XMP_coarray_malloc_info_1(const long, const size_t);
 extern void _XMP_coarray_malloc_info_2(const long, const long, const size_t);
 extern void _XMP_coarray_malloc_info_3(const long, const long, const long, const size_t);
@@ -87,9 +87,8 @@ extern void _XMP_coarray_malloc_image_info_5(const int, const int, const int, co
 extern void _XMP_coarray_malloc_image_info_6(const int, const int, const int, const int, const int);
 extern void _XMP_coarray_malloc_image_info_7(const int, const int, const int, const int, const int, const int);
 
-extern void _XMP_coarray_malloc_do_f(void **, void *);
-extern void _XMP_coarray_malloc_do(void **, void *);
-extern void _XMP_coarray_regmem_do(void **, void *);   // for Coarray F Ver.4
+extern void _XMP_coarray_malloc(void **, void *);
+extern void _XMP_coarray_regmem(void **, void *);   // for Coarray Fortran CA-method
 extern void _XMP_coarray_attach(void **, void *, const size_t);
 extern void _XMP_coarray_detach(void **);
 extern void _XMP_coarray_lastly_deallocate();
@@ -111,34 +110,34 @@ extern void _XMP_coarray_rdma_coarray_set_7(const long, const long, const long,
                                             const long, const long, const long, const long, const long, const long,
 					    const long, const long, const long);
 
-extern void _XMP_coarray_rdma_array_set_1(const long, const long, const long, const long, const long);
-extern void _XMP_coarray_rdma_array_set_2(const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long);
-extern void _XMP_coarray_rdma_array_set_3(const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long);
-extern void _XMP_coarray_rdma_array_set_4(const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long);
-extern void _XMP_coarray_rdma_array_set_5(const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long);
-extern void _XMP_coarray_rdma_array_set_6(const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long);
-extern void _XMP_coarray_rdma_array_set_7(const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long,
-					  const long, const long, const long, const long, const long);
+extern void _XMP_coarray_rdma_array_set_1(const long, const long, const long, const long, const size_t);
+extern void _XMP_coarray_rdma_array_set_2(const long, const long, const long, const long,
+					  const long, const long, const long, const long, const size_t);
+extern void _XMP_coarray_rdma_array_set_3(const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, const size_t);
+extern void _XMP_coarray_rdma_array_set_4(const long, const long, const long, const long,
+					  const long, const long, const long, const long,
+					  const long, const long, const long, const long,
+					  const long, const long, const long, const long, const size_t);
+extern void _XMP_coarray_rdma_array_set_5(const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, const size_t);
+extern void _XMP_coarray_rdma_array_set_6(const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, const size_t);
+extern void _XMP_coarray_rdma_array_set_7(const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, 
+					  const long, const long, const long, const long, const size_t);
 
 extern void _XMP_coarray_rdma_image_set_1(const int);
 extern void _XMP_coarray_rdma_image_set_2(const int, const int);
@@ -148,92 +147,94 @@ extern void _XMP_coarray_rdma_image_set_5(const int, const int, const int, const
 extern void _XMP_coarray_rdma_image_set_6(const int, const int, const int, const int, const int, const int);
 extern void _XMP_coarray_rdma_image_set_7(const int, const int, const int, const int, const int, const int, const int);
 
-extern void _XMP_coarray_rdma_do_f(const int*, void*, void*, void*);
-extern void _XMP_coarray_rdma_do(const int, void*, void*, void *);
+extern void _XMP_coarray_put(void*, void*, void *);
+extern void _XMP_coarray_get(void*, void*, void *);
 extern void _XMP_coarray_sync_all();
 extern void _XMP_coarray_sync_memory();
 extern void xmp_sync_memory(const int* status);
 extern void xmp_sync_all(const int* status);
 extern void xmp_sync_image(int image, int* status);
-extern void xmp_sync_image_f(int *image, int* status);
 extern void xmp_sync_images(const int num, int* image_set, int* status);
-extern void xmp_sync_images_f(const int *num, int* image_set, int* status);
 extern void xmp_sync_images_all(int* status);
-extern void _XMP_coarray_shortcut_put(const int, void*, const void*, const long, const long, const long, const long);
-extern void _XMP_coarray_shortcut_put_f(const int*, void*, const void*, const long*, const long*, const long*, const long*);
-extern void _XMP_coarray_shortcut_get(const int, void*, const void*, const long, const long, const long, const long);
-extern void _XMP_coarray_shortcut_get_f(const int*, void*, const void*, const long*, const long*, const long*, const long*);
+extern void _XMP_coarray_contiguous_put(const int, void*, const void*, const long, const long, const long, const long);
+extern void _XMP_coarray_contiguous_get(const int, void*, const void*, const long, const long, const long, const long);
 
 // xmp_coarray_acc.c
 int _XMP_coarray_get_total_elmts(void *coarray_desc);
-void _XMP_coarray_malloc_do_acc(void **coarray_desc, void *addr);
-void _XMP_coarray_shortcut_put_acc(const int target_image, const void *dst_desc, const void *src_desc, 
+void _XMP_coarray_malloc_acc(void **coarray_desc, void *addr);
+void _XMP_coarray_contiguous_put_acc(const int target_image, const void *dst_desc, const void *src_desc, 
 				   const size_t dst_offset, const size_t src_offset, 
 				   const size_t dst_elmts, const size_t src_elmts,
 				   const int is_dst_on_acc, const int is_src_on_acc);
-void _XMP_coarray_shortcut_get_acc(const int target_image, const void *dst_desc, const void *src_desc, 
+void _XMP_coarray_contiguous_get_acc(const int target_image, const void *dst_desc, const void *src_desc, 
 				   const size_t dst_offset, const size_t src_offset, 
 				   const size_t dst_elmts, const size_t src_elmts,
 				   const int is_dst_on_acc, const int is_src_on_acc);
-extern void _XMP_coarray_rdma_do_acc(const int, void*, void*, void *, const int, const int);
+extern void _XMP_coarray_put_acc(void*, void*, void *, const int, const int);
+extern void _XMP_coarray_get_acc(void*, void*, void *, const int, const int);
 
 // xmp_reflect_acc.c
 extern void _XMP_reflect_init_acc(void *, void *);
 extern void _XMP_reflect_do_acc(void *);
-extern void _XMP_reflect_acc(void *);
+extern void _XMP_reflect_acc__(void *acc_addr, void *array_desc);
 extern void _XMP_set_reflect_acc__(void *a, int dim, int lwidth, int uwidth, int is_periodic);
 
 #ifdef _XMP_TCA
 // xmp_tca.c
 extern void _XMP_init_tca();
 extern void _XMP_alloc_tca(void *);
-extern void _XMP_create_TCA_handle(void *, void *);
-extern void _XMP_create_TCA_desc(void *);
-extern void _XMP_reflect_do_tca(_XMP_array_t *);
 #endif
 
 // xmp_gmove.c
-extern void _XMP_gmove_BCAST_SCALAR(void *dst_addr, void *src_addr, void *array, ...);
-extern int _XMP_gmove_HOMECOPY_SCALAR(void *array, ...);
-extern void _XMP_gmove_SENDRECV_SCALAR(void *dst_addr, void *src_addr, void *dst_array, void *src_array, ...);
-extern void _XMP_gmove_LOCALCOPY_ARRAY(int type, size_t type_size, ...);
-extern void _XMP_gmove_BCAST_ARRAY(void *src_array, int type, size_t type_size, ...);
-extern void _XMP_gmove_HOMECOPY_ARRAY(void *dst_array, int type, size_t type_size, ...);
-extern void _XMP_gmove_SENDRECV_ARRAY(void *dst_array, void *src_array, int type, size_t type_size, ...);
-extern void _XMP_gmove_BCAST_TO_NOTALIGNED_ARRAY(void *dst_array, void *src_array, int type, size_t type_size, ...);
-extern void _XMP_gmove_GSECTION_GSCALAR(void *dst_array, void *src_array, int type, size_t type_size, ...);
-extern void _XMP_gmove_LSECTION_GSCALAR(void *src_array, int type, size_t type_size, void *dst, int dst_dim, ...);
-extern void _XMP_gmove_INOUT_SCALAR(void *dst_array, void *scalar, ...);
+/* extern void _XMP_gmove_BCAST_SCALAR(void *dst_addr, void *src_addr, void *array, ...); */
+/* extern int _XMP_gmove_HOMECOPY_SCALAR(void *array, ...); */
+/* extern void _XMP_gmove_SENDRECV_SCALAR(void *dst_addr, void *src_addr, void *dst_array, void *src_array, ...); */
+/* extern void _XMP_gmove_LOCALCOPY_ARRAY(int type, size_t type_size, ...); */
+/* extern void _XMP_gmove_BCAST_ARRAY(void *src_array, int type, size_t type_size, ...); */
+/* extern void _XMP_gmove_HOMECOPY_ARRAY(void *dst_array, int type, size_t type_size, ...); */
+/* extern void _XMP_gmove_SENDRECV_ARRAY(void *dst_array, void *src_array, int type, size_t type_size, ...); */
+/* extern void _XMP_gmove_BCAST_TO_NOTALIGNED_ARRAY(void *dst_array, void *src_array, int type, size_t type_size, ...); */
+/* extern void _XMP_gmove_GSECTION_GSCALAR(void *dst_array, void *src_array, int type, size_t type_size, ...); */
+/* extern void _XMP_gmove_LSECTION_GSCALAR(void *src_array, int type, size_t type_size, void *dst, int dst_dim, ...); */
+/* extern void _XMP_gmove_INOUT_SCALAR(void *dst_array, void *scalar, ...); */
+
+extern void xmpc_gmv_g_alloc(void **gmv_desc, void *ap);
+extern void xmpc_gmv_g_dim_info(void *gp, int i, int kind, int lb, int len, int st);
+extern void xmpc_gmv_l_alloc(void **gmv_desc, void *local_data, int n);
+extern void xmpc_gmv_l_dim_info(void *gp, int i, int a_lb, int a_len, int kind, int lb, int len, int st);
+extern void xmpc_gmv_dealloc(void *gp);
+extern void xmpc_gmv_do(void *gmv_desc_leftp, void *gmv_desc_rightp, int mode);
+
 
 // xmp_gmove_acc.c
-extern void _XMP_gmove_acc_BCAST_SCALAR(void *dst_addr, void *src_addr, void *array, ...);
-extern int _XMP_gmove_acc_HOMECOPY_SCALAR(void *array, ...);
-extern void _XMP_gmove_acc_SENDRECV_SCALAR(void *dst_addr, void *src_addr, void *dst_array, void *src_array, ...);
-extern void _XMP_gmove_acc_LOCALCOPY_ARRAY(int type, size_t type_size, ...);
-extern void _XMP_gmove_acc_BCAST_ARRAY(void *src_array, int type, size_t type_size, ...);
-extern void _XMP_gmove_acc_HOMECOPY_ARRAY(void *dst_array, int type, size_t type_size, ...);
-extern void _XMP_gmove_acc_SENDRECV_ARRAY(void *dst_array, void *src_array, void *dst_array_dev, void *src_array_dev, int type, size_t type_size, ...);
-extern void _XMP_gmove_acc_BCAST_TO_NOTALIGNED_ARRAY(void *dst_array, void *src_array, int type, size_t type_size, ...);
+extern void xmpc_gmv_g_alloc_acc(void **gmv_desc, void *ap, void *dev_addr);
+extern void xmpc_gmv_l_alloc_acc(void **gmv_desc, void *local_data, int n);
+extern void xmpc_gmv_do_acc(void *gmv_desc_leftp, void *gmv_desc_rightp, int mode);
 
 // xmp_loop.c
-extern void _XMP_sched_loop_template_DUPLICATION(int ser_init, int ser_cond, int ser_step,
-                                                 int *par_init, int *par_cond, int *par_step,
-                                                 void *template, int template_index);
-extern void _XMP_sched_loop_template_BLOCK(int ser_init, int ser_cond, int ser_step,
-                                           int *par_init, int *par_cond, int *par_step,
-                                           void *template, int template_index);
-extern void _XMP_sched_loop_template_CYCLIC(int ser_init, int ser_cond, int ser_step,
-                                            int *par_init, int *par_cond, int *par_step,
-                                            void *template, int template_index);
-extern void _XMP_sched_loop_template_BLOCK_CYCLIC(int ser_init, int ser_cond, int ser_step,
-                                           int *par_init, int *par_cond, int *par_step,
-                                           void *template, int template_index);
-extern void _XMP_sched_loop_nodes(int ser_init, int ser_cond, int ser_step,
-                                  int *par_init, int *par_cond, int *par_step,
-                                  void *nodes, int nodes_index);
-extern void _XMP_sched_loop_template_GBLOCK(int ser_init, int ser_cond, int ser_step,
-					    int *par_init, int *par_cond, int *par_step,
-					    void *template, int template_index);
+extern void xmpc_loop_sched(int ser_init, int ser_cond, int ser_step,
+			    int *par_init, int *par_cond, int *par_step,
+			    void *t_desc, int t_idx,
+			    int expand_type, int lwidth, int uwidth, int unbound_flag);
+/* extern void _XMP_sched_loop_template_DUPLICATION(int ser_init, int ser_cond, int ser_step, */
+/*                                                  int *par_init, int *par_cond, int *par_step, */
+/*                                                  void *template, int template_index); */
+/* extern void _XMP_sched_loop_template_BLOCK(int ser_init, int ser_cond, int ser_step, */
+/*                                            int *par_init, int *par_cond, int *par_step, */
+/*                                            void *template, int template_index, */
+/* 					   int expand_type, int lwidth, int uwidth, int unbound_flag); */
+/* extern void _XMP_sched_loop_template_CYCLIC(int ser_init, int ser_cond, int ser_step, */
+/*                                             int *par_init, int *par_cond, int *par_step, */
+/*                                             void *template, int template_index); */
+/* extern void _XMP_sched_loop_template_BLOCK_CYCLIC(int ser_init, int ser_cond, int ser_step, */
+/*                                            int *par_init, int *par_cond, int *par_step, */
+/*                                            void *template, int template_index); */
+/* extern void _XMP_sched_loop_nodes(int ser_init, int ser_cond, int ser_step, */
+/*                                   int *par_init, int *par_cond, int *par_step, */
+/*                                   void *nodes, int nodes_index); */
+/* extern void _XMP_sched_loop_template_GBLOCK(int ser_init, int ser_cond, int ser_step, */
+/* 					    int *par_init, int *par_cond, int *par_step, */
+/* 					    void *template, int template_index); */
 
 // xmp_math_function.c
 extern int _XMP_modi_ll_i(long long value, int cycle);
@@ -292,10 +293,13 @@ extern void _XMP_reflect__(char *a);
 extern void _XMP_wait_async__(int async_id);
 extern void _XMP_reflect_async__(void *a, int async_id);
 
+// xmp_reduce_shadow.c
+extern void _XMP_set_reduce_shadow__(void *a, int dim, int lwidth, int uwidth, int is_periodic);
+extern void _XMP_reduce_shadow__(void *a);
+
 // xmp_runtime.c
-//extern void _XMP_init(void);
-extern void _XMP_init(int, char**); 
-extern void _XMP_finalize(int);
+//extern void _XMP_init(int, char**, MPI_Comm); 
+//extern void _XMP_finalize(bool);
 extern char *_XMP_desc_of(void *p);
 
 // xmp_shadow.c
@@ -325,7 +329,7 @@ extern void _XMP_dist_template_GBLOCK(void *template, int template_index, int no
 
 extern int _XMP_exec_task_TEMPLATE_PART(void **task_desc, void *ref_template, ...);
 extern int _XMP_exec_task_TEMPLATE_PART_nocomm(void **task_desc, void *ref_template, ...);
-extern long long int _XMP_L2G_GBLOCK(int local_idx, void *template, int template_index);
+extern /*long long*/ int xmpc_ltog(int local_idx, void *template, int template_index, int offset);
 
 // xmp_util.c
 extern void *_XMP_alloc(size_t size);
