@@ -36,6 +36,8 @@ void _XMP_init_array_desc(_XMP_array_t **array, _XMP_template_t *template, int d
 {
   _XMP_array_t *a = _XMP_alloc(sizeof(_XMP_array_t) + sizeof(_XMP_array_info_t) * (dim - 1));
 
+  a->desc_kind            = _XMP_DESC_ARRAY;
+  
   a->is_allocated         = template->is_owner;
   a->is_align_comm_member = false;
   a->dim                  = dim;
@@ -43,7 +45,8 @@ void _XMP_init_array_desc(_XMP_array_t **array, _XMP_template_t *template, int d
   a->type_size            = type_size;
   
   size_t dummy;
-  _XMP_setup_reduce_type(&a->mpi_type, &dummy, type);
+  if(type != _XMP_N_TYPE_NONBASIC)
+    _XMP_setup_reduce_type(&a->mpi_type, &dummy, type);
   
   a->order           = MPI_ORDER_C;
   a->array_addr_p    = NULL;
@@ -90,6 +93,7 @@ void _XMP_init_array_desc(_XMP_array_t **array, _XMP_template_t *template, int d
     ai->shadow_size_hi  = 0;
 
     ai->reflect_sched = NULL;
+    ai->reflect_acc_sched = NULL;
 
     ai->shadow_comm = NULL;
     ai->shadow_comm_size = 1;
@@ -108,6 +112,8 @@ void _XMP_init_array_desc_NOT_ALIGNED(_XMP_array_t **adesc, _XMP_template_t *tem
 
   _XMP_array_t *a = _XMP_alloc(sizeof(_XMP_array_t) + sizeof(_XMP_array_info_t) * (ndims - 1));
 
+  a->desc_kind    = _XMP_DESC_ARRAY;
+
   a->is_allocated = template->is_owner;
   a->is_align_comm_member = false;
   a->dim = ndims;
@@ -166,6 +172,7 @@ void _XMP_init_array_desc_NOT_ALIGNED(_XMP_array_t **adesc, _XMP_template_t *tem
     //ai->shadow_size_hi  = 0;
 
     ai->reflect_sched = NULL;
+    ai->reflect_acc_sched = NULL;
     ai->shadow_comm = NULL;
     ai->shadow_comm_size = 1;
     ai->shadow_comm_rank = _XMP_N_INVALID_RANK;
@@ -189,27 +196,21 @@ void _XMP_finalize_array_desc(_XMP_array_t *array)
     if(ai->is_shadow_comm_member)
       _XMP_finalize_comm(ai->shadow_comm);
 
+    _Bool free_buf = (_XMPF_running && i != dim -1) || (_XMPC_running && i != 0);
+
     _XMP_reflect_sched_t *reflect_sched;
-
     if((reflect_sched = ai->reflect_sched)){
-      if(reflect_sched->datatype_lo != MPI_DATATYPE_NULL)
-	MPI_Type_free(&reflect_sched->datatype_lo);
-      if(reflect_sched->datatype_hi != MPI_DATATYPE_NULL)
-	MPI_Type_free(&reflect_sched->datatype_hi);
-
-      for(int j=0;j<4;j++)
-	if(reflect_sched->req[j] != MPI_REQUEST_NULL)
-	  MPI_Request_free(&reflect_sched->req[j]);
-
-      if((_XMPF_running && i != dim -1) || (_XMPC_running && i != 0)){
-	_XMP_free(reflect_sched->lo_send_buf);
-	_XMP_free(reflect_sched->lo_recv_buf);
-	_XMP_free(reflect_sched->hi_send_buf);
-	_XMP_free(reflect_sched->hi_recv_buf);
-      }
-      
+      _XMP_finalize_reflect_sched(reflect_sched, free_buf);
       _XMP_free(reflect_sched);
     }
+
+#ifdef _XMP_XACC
+    _XMP_reflect_sched_t *reflect_acc_sched;
+    if((reflect_acc_sched = ai->reflect_acc_sched)){
+      _XMP_finalize_reflect_sched_acc(reflect_acc_sched, free_buf);
+      _XMP_free(reflect_acc_sched);
+    }
+#endif
   }
 
   _XMP_async_reflect_t *async_reflect = array->async_reflect;
@@ -805,6 +806,24 @@ void _XMP_init_array_addr(void **array_addr, void *init_addr,
     _XMP_calc_array_dim_elmts(array_desc, i);
   }
 
+  // clear reflect schedule
+  if (array_desc->array_addr_p && array_desc->array_addr_p != init_addr){
+    for (int i = 0; i < dim; i++) {
+      _XMP_reflect_sched_t *sched = array_desc->info[i].reflect_sched;
+      if (sched){
+	_XMP_finalize_reflect_sched(sched, (i != 0));
+	_XMP_init_reflect_sched(sched);
+      }
+#ifdef _XMP_XACC
+      _XMP_reflect_sched_t *sched_acc = array_desc->info[i].reflect_acc_sched;
+      if (sched_acc){
+	_XMP_finalize_reflect_sched_acc(sched_acc, (i != 0));
+	_XMP_init_reflect_sched_acc(sched_acc);
+      }
+#endif
+    }
+  }
+
   *array_addr = init_addr;
 
   // set members
@@ -958,7 +977,6 @@ unsigned long long _XMP_get_array_total_elmts(_XMP_array_t *array) {
 
 void _XMP_align_array_noalloc(_XMP_array_t *a, int adim, int tdim, long long align_subscript, 
 			     int *temp0, unsigned long long *acc0){
-  _XMP_ASSERT(a->dim == 1);
 
   a->is_allocated = false;
 
